@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,66 +16,82 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#ifndef _DB_SQL_EDITOR_BE_H_
-#define _DB_SQL_EDITOR_BE_H_
+#ifndef _WB_SQL_EDITOR_FORM_H_
+#define _WB_SQL_EDITOR_FORM_H_
 
 // renamed from db_sql_editor_be.h
 
 #include "workbench/wb_backend_public_interface.h"
 #include "sqlide/recordset_be.h"
 #include "sqlide/sql_editor_be.h"
-#include "sqlide/db_sql_editor_log_be.h"
+#include "sqlide/db_sql_editor_log.h"
 #include "sqlide/db_sql_editor_history_be.h"
 #include "cppdbc.h"
 #include "grts/structs.workbench.h"
 #include "grts/structs.db.mgmt.h"
 #include "base/file_utilities.h"
 #include "base/ui_form.h"
+#include "base/notifications.h"
 #include "grt/refresh_ui.h"
-#include "grt/action_list.h"
 #include "sqlide/wb_context_sqlide.h"
 #include "sqlide/wb_live_schema_tree.h"
-#include "sqlide/wb_overview_live_physical.h"
 #include <boost/enable_shared_from_this.hpp>
 
 #include "mforms/task_sidebar.h"
-#include "mforms/textbox.h"
-#include "mforms/splitter.h"
+
+namespace mforms {
+  class ToolBar;
+  class Splitter;
+  class TabView;
+  class HyperText;
+};
 
 namespace bec
 {
   class DBObjectEditorBE;
 }
 
-class MYSQLWBBACKEND_PUBLIC_FUNC Db_sql_editor : public bec::UIForm, public bec::RefreshUI, public boost::enable_shared_from_this<Db_sql_editor>
+
+class QuerySidePalette;
+
+typedef std::vector<Recordset::Ref> Recordsets;
+typedef boost::shared_ptr<std::vector<Recordset::Ref> > RecordsetsRef;
+
+#define Db_sql_editor SqlEditorForm
+class MYSQLWBBACKEND_PUBLIC_FUNC SqlEditorForm : public bec::UIForm, public bec::RefreshUI, base::Observer,
+                                                public wb::LiveSchemaTree::FetchDelegate, public wb::LiveSchemaTree::Delegate,
+                                                public boost::enable_shared_from_this<SqlEditorForm>
 {
 public:
+#if defined(ENABLE_TESTING)
+  friend class Db_sql_editor_tester;
+#endif
+
   enum
   {
-    RefreshSidebar, // update the sidebar
-//deprecated    RefreshActiveSchemaSelector, // refresh active schema selector
-//deprecated    RefreshSchemaTree, // refresh live schema tree
-    RefreshEditor, // refresh the text editor using data from backend
-    RefreshEditorBackend, // save text editor contents to backend so that it can be saved to file
-    RefreshEditorTitle,  // refresh the caption of active editor
-    RefreshSnippets, // snippets list
-    RefreshSnippetCollections, // snippet collections list
-    RefreshOverview, // refresh the live overview
-    RefreshOverviewFully, // full refresh of the live overview
-    RefreshRecordsetTitle, // refresh caption of active recordset
-    RunCurrentScript, // running sql needs to be initiated by front-end
-    RunCurrentStatement // run only statement under cursor position (if there any statement at all)
+    RefreshEditor,             // refresh the text editor using data from backend
+    RefreshEditorBackend,      // save text editor contents to backend so that it can be saved to file
+    RefreshEditorTitle,        // refresh the caption of active editor
+    RefreshRecordsetTitle,     // refresh caption of active recordset
+    RunCurrentScript,          // running sql needs to be initiated by front-end
+    RunCurrentStatement,       // run only statement under cursor position (if there any statement at all)
+    ShowFindPanel,             // show the Find panel for the active editor
+    ShowSpecialCharacters,     // show special (tab, space, newline etc) in active editor
+    HideSpecialCharacters,     // hide special chars
+    RefreshMainTitle,          // refresh the caption of sql editor tab
+    SaveRecordsetChanges,      // commit changes being made to active recordset (as if focus was removed)
+    DiscardRecordsetChanges   // revert changes being made to active recordset (as if esc was pressed)
   };
 
 public:
-  typedef boost::shared_ptr<Db_sql_editor> Ref;
-  typedef boost::weak_ptr<Db_sql_editor> Ptr;
-  static Db_sql_editor::Ref create(wb::WBContextSQLIDE *wbsql, const db_mgmt_ConnectionRef &conn);
+  typedef boost::shared_ptr<SqlEditorForm> Ref;
+  typedef boost::weak_ptr<SqlEditorForm> Ptr;
+  static SqlEditorForm::Ref create(wb::WBContextSQLIDE *wbsql, const db_mgmt_ConnectionRef &conn);
   
 protected:
-  Db_sql_editor(wb::WBContextSQLIDE *wbsql, const db_mgmt_ConnectionRef &conn);
+  SqlEditorForm(wb::WBContextSQLIDE *wbsql, const db_mgmt_ConnectionRef &conn);
 public:
-  virtual ~Db_sql_editor();
+  virtual ~SqlEditorForm();
 
 public:
   virtual bool close();
@@ -95,6 +111,7 @@ public:
   void context_ui(wb::WBContextUI *val);
 
   void update_menu_and_toolbar();
+  void validate_menubar();
 private:
   wb::WBContextSQLIDE *_wbsql;
   wb::WBContextUI *_context_ui;
@@ -110,79 +127,105 @@ private:
   bool _close_done;
 
   void activate_command(const std::string &command);
-public:
-  ::ActionList & action_list();
-private:
-  ::ActionList _action_list;
-private:
-  void register_default_actions();
+  void rename_autosave_files(int from, int to);
 
 public:
   db_mgmt_RdbmsRef rdbms();
 
 public:
-  Sql_editor::Ref sql_editor();
+
+  Sql_editor::Ref active_sql_editor();
   Sql_editor::Ref sql_editor(int index);
-  std::string sql_editor_path(int index) { return _sql_editors[index].filename; }
+  int sql_editor_index(Sql_editor::Ref);
+  std::string sql_editor_path(int index) { return _sql_editors[index]->filename; }
   std::string sql_editor_caption(int index=-1);
-  bool sql_editor_dirty(int index) { return _sql_editors[index].dirty && !_sql_editors[index].is_scratch; }
-  bool sql_editor_is_scratch(int index) { return _sql_editors[index].is_scratch; }
+  void sql_editor_caption(int new_index, std::string caption);
+  bool sql_editor_dirty(int index) { return _sql_editors[index]->dirty && !_sql_editors[index]->is_scratch; }
+  bool sql_editor_is_scratch(int index) { return _sql_editors[index]->is_scratch; }
+  bool sql_editor_start_collapsed(int index) { return _sql_editors[index]->start_collapsed; }
   void sql_editor_dirty(int index, bool flag);
   bool sql_editor_will_close(int index);
-  long long active_recordset_for_sql_editor(int index);
+  bool sql_editor_reorder(Sql_editor::Ref, int new_index);
+  void sql_editor_open_file(int index, const std::string &file_path, const std::string &encoding= "");
   int sql_editor_index_for_recordset(long long rset);
+  RecordsetsRef sql_editor_recordsets(const int index);
+  boost::shared_ptr<mforms::ToolBar> sql_editor_toolbar(int index) { return _sql_editors[index]->toolbar; }
+
 private:
   struct Sql_editor_info {
+    typedef boost::shared_ptr<Sql_editor_info> Ref;
+    
+    boost::shared_ptr<mforms::ToolBar> toolbar;
     std::string filename;
     std::string autosave_filename;
     std::string orig_encoding;
     std::string caption;
     Sql_editor::Ref editor;
-    std::vector<long long> resultsets;
-    long long active_resultset;
+    RecordsetsRef recordsets;
+    Recordset::Ref active_recordset;
+    GMutex *recordset_mutex;
+    int rs_sequence;
     bool dirty;
     bool is_scratch;
+    bool start_collapsed;
+    bool busy;
+    
+    Sql_editor_info() : rs_sequence(0), dirty(false), is_scratch(false), start_collapsed(false), busy(false) {}
   };
-  typedef std::vector<Sql_editor_info> Sql_editors;
+  typedef std::vector<Sql_editor_info::Ref> Sql_editors;
   Sql_editors _sql_editors;
   int _sql_editors_serial;
   int _scratch_editors_serial;
   GMutex *_sql_editors_mutex;
+  
+  boost::shared_ptr<mforms::ToolBar> setup_editor_toolbar();
+  void set_editor_tool_items_enbled(const std::string &name, bool flag);
+  void set_editor_tool_items_checked(const std::string &name, bool flag);
 public:
+  void set_tool_item_checked(const std::string &name, bool flag);
+
   boost::signals2::signal<void (Sql_editor::Ref, bool)> sql_editor_list_changed;
 
-  int add_sql_editor(bool scratch = false); // returns index of the added sql_editor
+  int run_sql_in_scratch_tab(const std::string &sql, bool reuse_if_possible, bool start_collapsed);
+  int add_sql_editor(bool scratch = false, bool start_collapsed = false); // returns index of the added sql_editor
   void remove_sql_editor(int index);
-  void add_empty_recordset_for_sql_editor(int editor);
   int sql_editor_count();
-  int active_sql_editor_index() const { return _active_sql_editor_index; }
-  void active_sql_editor_index(int val) { _active_sql_editor_index= val; }
+  int active_sql_editor_index() { return _active_sql_editor_index; }
+  void active_sql_editor_index(int val);
 private:
   int _active_sql_editor_index;
   int _updating_sql_editor;
+  
+  int count_connection_editors(std::string conn_name);
 
+  Sql_editor::Ref active_sql_editor_or_new_scratch();
 private:
   void on_sql_editor_text_change();
   void on_sql_editor_text_selection_change();
-  void set_sql_editor_text(const std::string &sql, bool run_sql);
+  void set_sql_editor_text(const std::string &sql);
 public:
   std::string caption() const;
+  std::string get_title(bool new_editor);
 
 private:
   std::map<std::string, std::string> _connection_details;
 
-  grt::StringRef do_connect(grt::GRT *grt);
+  grt::StringRef do_connect(grt::GRT *grt, sql::TunnelConnection *tunnel, sql::Authentication::Ref &auth, sql::AuthenticationError *&autherr_ptr);
   grt::StringRef do_disconnect(grt::GRT *grt);
 public:
   void connect();
   bool connected() const;
   void finish_startup();
+  void title_changed();
   void cancel_query();
   void reset();
   void commit();
   void rollback();
   bool auto_commit();
   void auto_commit(bool value);
+  void toggle_autocommit();
+  
+  void run_sql();
 private:
   void do_commit();
 public:  
@@ -195,7 +238,7 @@ private:
   std::string _sql_mode;
 
 private:
-  void create_connection(sql::Dbc_connection_handler::Ref &dbc_conn, db_mgmt_ConnectionRef db_mgmt_conn, bool autocommit_mode);
+  void create_connection(sql::Dbc_connection_handler::Ref &dbc_conn, db_mgmt_ConnectionRef db_mgmt_conn, sql::TunnelConnection *tunnel, sql::Authentication::Ref auth, bool autocommit_mode);
   void init_connection(sql::Connection* dbc_conn_ref, const db_mgmt_ConnectionRef& connectionProperties, sql::Dbc_connection_handler::Ref& dbc_conn);
   void close_connection(sql::Dbc_connection_handler::Ref &dbc_conn);
   bec::GMutexLock ensure_valid_dbc_connection(sql::Dbc_connection_handler::Ref &dbc_conn, GMutex *dbc_conn_mutex);
@@ -217,15 +260,14 @@ private:
   sql::Dbc_connection_handler::Ref _usr_dbc_conn;
   GMutex *_usr_dbc_conn_mutex;
 
+  sql::Authentication::Ref _dbc_auth;
 public:
-  typedef std::vector<Recordset::Ref> Recordsets;
-
   void exec_sql(std::string &sql, Sql_editor::Ref editor, bool sync, bool wrap_with_non_std_delimiter= false, bool dont_add_limit_clause= false);
-  void exec_sql_retaining_editor_contents(const std::string &sql_script, bool sync, bool dont_add_limit_clause= false);
+  void exec_sql_retaining_editor_contents(const std::string &sql_script, Sql_editor::Ref editor, bool sync, bool dont_add_limit_clause= false);
 
-  Recordsets exec_sql_returning_results(const std::string &sql_script, bool dont_add_limit_clause);
+  RecordsetsRef exec_sql_returning_results(const std::string &sql_script, bool dont_add_limit_clause);
   
-  bool explain_sql(bool only_check_if_applicable);
+  void explain_sql();
   void explain_current_statement();
   bool is_running_query();
 private:
@@ -235,7 +277,7 @@ private:
     Dont_add_limit_clause = (1<<2),
     Show_warnings = (1<<3)
   };
-  grt::StringRef do_exec_sql(grt::GRT *grt, Ptr self_ptr, std::string &sql, Sql_editor::Ref editor, ExecFlags flags, Recordsets *result_list);
+  grt::StringRef do_exec_sql(grt::GRT *grt, Ptr self_ptr, std::string &sql, Sql_editor::Ref editor, ExecFlags flags, RecordsetsRef result_list);
   void do_explain_sql(const std::string &sql);
 public:
   GrtThreadedTask::Ref exec_sql_task;
@@ -250,32 +292,26 @@ private:
   bool _continue_on_error;
 
 public:
-  int recordset_count();
-  Recordset::Ref recordset(int index);
-  void recordsets_are_pinned_by_default(bool value);
-  bool recordsets_are_pinned_by_default() { return _recordsets_are_pinned_by_default; }
-  void active_recordset(Recordset::Ref value);
-  Recordset::Ref active_recordset() { return _active_recordset; }
-private:
-  Recordsets _recordsets;
-  GMutex *_recordsets_mutex;
-  bool _recordsets_are_pinned_by_default;
-  Recordset::Ref _active_recordset;
+  int recordset_count(int editor);
+  Recordset::Ref recordset(int editor, int index);
+  void active_recordset(int editor, Recordset::Ref value);
+  Recordset::Ref active_recordset(int editor);
+  bool recordset_reorder(int editor, Recordset::Ref value, int new_index);
 public:
-  boost::signals2::signal<void (Recordset::Ref, bool)> recordset_list_changed;
+  boost::signals2::signal<void (int, Recordset::Ref, bool)> recordset_list_changed;
+  //delme sigc::signal<int, long long> close_recordset_ui;
 private:
   void on_close_recordset(Recordset::Ptr rs_ptr);
-  int _rs_sequence;
 private:
   void apply_changes_to_recordset(Recordset::Ptr rs_ptr);
   bool run_data_changes_commit_wizard(Recordset::Ptr rs_ptr);
   void apply_data_changes_commit(std::string &sql_script_text, Recordset::Ptr rs_ptr);
-  void save_active_recordset_edits();
-  void discard_active_recordset_edits();
   void recall_recordset_query(Recordset::Ptr rs_ptr);
+  void update_editor_title_schema(const std::string& schema);
 
-  void show_export_recordset(Recordset::Ptr rs_ptr);
 public:
+  void show_export_recordset(int editor_index, Recordset::Ptr rs_ptr);
+  
   bool can_close();
   bool can_close_(bool interactive);
 
@@ -286,9 +322,10 @@ public:
 
 public:
   void new_sql_script_file();
-  void new_sql_scratch_area();
-  void open_sql_script_file(const std::string &file_path, bool run_immediately, bool run_only_skip_editor = false,
-                            const std::string &encoding = "");
+  void new_sql_scratch_area(bool start_collapsed = false);
+  void open_file(const std::string &path, bool in_new_tab);
+  void open_file(const std::string &path = "") { open_file(path, true); }
+  void save_file();
   bool save_sql_script_file(const std::string &file_path, int editor_index);
   void revert_sql_script_file();
 
@@ -296,7 +333,6 @@ public:
   boost::signals2::signal<int (int)> sql_editor_new_ui;
 
 public:
-  void get_schemata(std::list<std::string> &schemata);
   void active_schema(const std::string &value);
   std::string active_schema() const;
   int active_schema_index() const;
@@ -305,28 +341,55 @@ private:
 
 public:
   wb::LiveSchemaTree *get_schema_tree();
+  wb::LiveSchemaTree *get_base_schema_tree();
+  wb::LiveSchemaTree *get_filtered_schema_tree();
   void request_refresh_schema_tree();
   
 private:
-  std::list<std::string> fetch_live_schema_list();
-  bool fetch_live_schema_contents(bec::NodeId node, const std::string &schema_name, const wb::LiveSchemaTree::SchemaContentArrivedSlot &arrived_slot);
-  grt::StringRef do_fetch_live_schema_contents(grt::GRT *grt, Ptr self_ptr, bec::NodeId node, std::string schema_name, wb::LiveSchemaTree::SchemaContentArrivedSlot arrived_slot);
-  void notify_of_fetched_schema_contents(Ptr self_ptr, bec::NodeId node, db_SchemaRef schema, bool schemata_tree_needs_refresh, bool live_physical_overview_schema_node_needs_refresh, bool live_physical_overview_needs_refresh);
-  bool fetch_object_details(wb::LiveSchemaTree::ObjectType obj_type, const std::string &schema_name, const std::string &obj_name, const wb::LiveSchemaTree::ObjectDetailsArrivedSlot &arrived_slot);
+  // LiveSchemaTree::FetchDelegate
+  virtual std::list<std::string> fetch_schema_list();
+  virtual bool fetch_schema_contents(bec::NodeId node, const std::string &schema_name, bool include_column_data, const wb::LiveSchemaTree::SchemaContentArrivedSlot &arrived_slot);
+  virtual bool fetch_object_details(wb::LiveSchemaTree::ObjectType obj_type, const std::string &schema_name, const std::string &obj_name, short flags, const wb::LiveSchemaTree::ObjectDetailsArrivedSlot &arrived_slot);
+
 private:
-  wb::LiveSchemaTree _schema_tree;
+  // LiveSchemaTree::Delegate
+  virtual void tree_refresh() { refresh_schema_tree(true); }
+  virtual void tree_activate_objects(const std::string&, const std::vector<wb::LiveSchemaTree::ChangeRecord>& changes);
+  virtual void tree_alter_objects(const std::vector<wb::LiveSchemaTree::ChangeRecord>& changes);
+  virtual void tree_drop_objects(const std::vector<wb::LiveSchemaTree::ChangeRecord>& changes);
+  virtual void tree_create_object(wb::LiveSchemaTree::ObjectType type, const std::string &schema_name, const std::string &obj_name);  
+  
+private:
+  grt::StringRef do_fetch_live_schema_contents(grt::GRT *grt, Ptr self_ptr, bec::NodeId node, const std::string &schema_name, bool include_column_data, wb::LiveSchemaTree::SchemaContentArrivedSlot arrived_slot);
+  void notify_of_fetched_schema_contents(Ptr self_ptr, bec::NodeId node);
+  void fetch_column_data(const std::string &schema_name, const std::string &obj_name, wb::LiveSchemaTree::ViewNode& view_node);
+  void fetch_trigger_data(const std::string &schema_name, const std::string &obj_name, wb::LiveSchemaTree::TableNode& table_node);
+  void fetch_index_data(const std::string &schema_name, const std::string &obj_name, wb::LiveSchemaTree::TableNode& table_node);
+  void fetch_foreign_key_data(const std::string &schema_name, const std::string &obj_name, wb::LiveSchemaTree::TableNode& table_node);
+
+  bool is_editable_select(const std::string& statement, std::string &schema_name, std::string& table_name, std::string& error);
+  void table_details_arrived(const wb::LiveSchemaTree::SchemaNode &schema_node, const wb::LiveSchemaTree::ObjectNode *obj_node, wb::LiveSchemaTree::ObjectType obj_type, wb::LiveSchemaTree::TableNode *table_node);
+private:
+  wb::LiveSchemaTree *_schema_tree;
+  wb::LiveSchemaTree _base_schema_tree;
+  wb::LiveSchemaTree _filtered_schema_tree;
   GrtThreadedTask::Ref live_schema_fetch_task;
   GMutex *_schema_contents_mutex;
 
   mforms::TaskSidebar* _side_bar;
-  mforms::Splitter _side_splitter;
-  mforms::TextBox _info_box;
+  mforms::Splitter *_side_splitter;
+  mforms::TabView *_info_tabview;
+  mforms::HyperText *_object_info;
+  mforms::HyperText *_session_info;
+  
+  mforms::View* _side_palette_host;
+  QuerySidePalette* _side_palette;
 private:
   void do_alter_live_object(wb::LiveSchemaTree::ObjectType type, const std::string &schema_name, const std::string &obj_name);
-  void do_create_live_object(wb::LiveSchemaTree::ObjectType type, const std::string &schema_name, const std::string &obj_name);
-  void do_drop_live_object(wb::LiveSchemaTree::ObjectType type, const std::string schema_name, const std::string obj_name);
+  void simple_drop_live_object(wb::LiveSchemaTree::ObjectType type, const std::string schema_name, const std::string obj_name);
+  
   std::string get_object_ddl_script(wb::LiveSchemaTree::ObjectType type, const std::string &schema_name, const std::string &obj_name);
-  void apply_object_alter_script(std::string &alter_script, bec::DBObjectEditorBE* obj_editor);
+  void apply_object_alter_script(std::string &alter_script, bec::DBObjectEditorBE* obj_editor, RowId log_id);
   void refresh_live_object_in_editor(bec::DBObjectEditorBE* obj_editor, bool using_old_name);
   void refresh_live_object_in_overview(wb::LiveSchemaTree::ObjectType type, const std::string schema_name, const std::string old_obj_name, const std::string new_obj_name);
   
@@ -340,7 +403,7 @@ public:
   void schema_object_activated(const std::string &action, wb::LiveSchemaTree::ObjectType type, const std::string &schema, const std::string &name);
   bool apply_changes_to_object(bec::DBObjectEditorBE* obj_editor, bool dry_run);
   virtual void notify_of_ui_form_creation(bec::UIForm *form);
-  bool run_live_object_alteration_wizard(std::string &alter_script, bec::DBObjectEditorBE* obj_editor);
+  bool run_live_object_alteration_wizard(const std::string &alter_script, bec::DBObjectEditorBE* obj_editor, RowId log_id, const std::string &log_context);
 public:
   void create_live_table_stubs(bec::DBObjectEditorBE *table_editor);
   bool expand_live_table_stub(bec::DBObjectEditorBE *table_editor, const std::string &schema_name, const std::string &obj_name);
@@ -353,21 +416,21 @@ private:
   Batch_exec_progress_cb on_sql_script_run_progress;
   Batch_exec_stat_cb on_sql_script_run_statistics;
 
+  
+  int sql_script_apply_error(long long, const std::string&, const std::string&, std::string&);
+  int sql_script_apply_progress(float);
+  int sql_script_stats(long, long);
 public:
-  wb::LivePhysicalOverviewBE * live_physical_overview();
   bool activate_live_object(GrtObjectRef object);
   bool create_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name);
   bool drop_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name);
-  void refresh_physical_overview(bool sync);
-  bool is_physical_overview_enabled() const;
+  void refresh_schema_tree(bool hard_refresh);
 private:
-  grt::StringRef do_refresh_physical_overview(grt::GRT *grt, Ptr self_ptr, bool hard_refresh);
-  void do_refresh_physical_overview(bool hard_refresh);
-  void do_refresh_physical_overview(const std::list<std::string>& schema_list, bool hard_refresh);
+  grt::StringRef do_refresh_schema_tree_safe(grt::GRT *grt, Ptr self_ptr, bool hard_refresh);
+  void do_refresh_schema_tree(bool hard_refresh);
+  void do_refresh_schema_tree_ui(const std::list<std::string>& schema_list, bool hard_refresh);
 private:
-  wb::LivePhysicalOverviewBE * _live_physical_overview;
-  bool _is_physical_overview_enabled;
-  bool _is_refreshing_physical_overview;
+  bool _is_refreshing_schema_tree;
   GrtThreadedTask::Ref live_schemata_refresh_task;
 
 public:
@@ -380,9 +443,12 @@ public:
 protected:
   DbSqlEditorLog::Ref _log;
   DbSqlEditorHistory::Ref _history;
+  std::string _title;
 
 public:
   void get_log_event_details(int log_event_no, int &log_event_type_code, std::string &log_event_time, std::string &log_event_action, std::string &log_event_message);
+  
+  std::string get_text_for_actions(const std::list<int> &indexes, bool query_only);
 protected:
   RowId add_log_message(int msg_type, const std::string &msg, const std::string &context, const std::string &duration);
   void set_log_message(RowId log_message_index, int msg_type, const std::string &msg, const std::string &context, const std::string &duration);
@@ -406,17 +472,20 @@ private:
   bool call_recordset_popup_menu_item(const std::string &item, const std::vector<int> &rows,
                                       int column, Recordset::Ptr rs_ptr);
 
-  void main_form_changed(bec::UIForm *form);
-  void setup_sidebar();
-  void sidebar_command(const std::string& command);
+  virtual void handle_notification(const std::string &name, void *sender, base::NotificationInfo &info);
+  void setup_sidebars();
   
   void schema_row_selected();
+  void side_bar_filter_changed(const std::string& filter);
+  
 public:
+  void toolbar_command(const std::string& command);
+
   bool save_snippet(const std::string &text);
 
-  //mforms::TaskSidebar* get_sidebar();
-  mforms::View *get_sidebar() { return &_side_splitter; }
+  mforms::View *get_sidebar();
+  mforms::View *get_side_palette();
 };
 
 
-#endif /* _DB_SQL_EDITOR_BE_H_ */
+#endif /* _WB_SQL_EDITOR_FORM_H_ */
