@@ -115,16 +115,18 @@ StarterPopup::StarterPopup(WBContext* context)
 
   // Action menu.
   _action_menu= mforms::manage(new mforms::Menu());
-  _action_menu->set_handler(sigc::mem_fun(this, &StarterPopup::handle_command));
+  _action_menu->set_handler(boost::bind(&StarterPopup::handle_command, this, _1));
   _action_menu->add_item(_("Start Plugin"), "start_plugin");
   _action_menu->add_item(_("Start Plugin in External Browser"), "start_plugin_external");
   _action_menu->add_separator();
+  /* Bug 12613922
   _action_menu->add_item(_("Find Plugin in Web Repository"), "find_plugin");
-  _action_menu->add_item(_("Goto Author's Website"), "author_home");
+  */
+  _action_menu->add_item(_("Go to Author's Website"), "author_home");
   _action_menu->add_separator();
 
   _placing_menu= mforms::manage(new mforms::Menu());
-  _placing_menu->set_handler(sigc::mem_fun(this, &StarterPopup::handle_command));
+  _placing_menu->set_handler(boost::bind(&StarterPopup::handle_command, this, _1));
   _placing_menu->add_item(_("Set at Position 1"), "set_1");
   _placing_menu->add_item(_("Set at Position 2"), "set_2");
   _placing_menu->add_item(_("Set at Position 3"), "set_3");
