@@ -68,10 +68,10 @@ GRTManager::GRTManager(bool threaded, bool verbose)
 : _has_unsaved_changes(false), _threaded(threaded), _verbose(verbose)
 {
   _globals_tree_soft_lock_count= 0;
+  _current_idle_serial = 0;
   
   init_all();
 
-  _idle_signal = new boost::signals2::signal<void ()>();
   _idle_mutex= g_mutex_new();
   _timer_mutex= g_mutex_new();
   _disp_map_mutex= g_mutex_new();
@@ -104,8 +104,6 @@ GRTManager::GRTManager(bool threaded, bool verbose)
   _structs_tree= 0;
   _modules_tree= 0;
   _messages_list= new MessageListStorage(this);
-
-  _log_file= 0;
 }
 
 
@@ -213,16 +211,11 @@ bool GRTManager::in_main_thread()
 
 GRTManager::~GRTManager()
 {
-  delete _idle_signal;
-
   {
     GStaticMutexLock _lock(_instance_mutex);
     _instances.erase(_grt);
   }  
 
-  if (_log_file)
-    fclose(_log_file);
-
   _grt->set_message_handler(MessageSlot());
 
   _dispatcher->shutdown();
@@ -337,18 +330,18 @@ ModulesTreeBE *GRTManager::get_shared_modules_tree()
 }
 
 
-void GRTManager::set_message_callback(const boost::function<void (const grt::Message&)> &cb)
+void GRTManager::set_message_callback(const boost::function<void (grt::Message)> &cb)
 {
   _message_cb= cb;
 }
 
-void GRTManager::set_progress_callback(const boost::function<bool (const std::string&,const std::string&,float)> &cb)
+void GRTManager::set_progress_callback(const boost::function<bool (std::string,std::string,float)> &cb)
 {
   _progress_cb= cb;
 }
 
 
-void GRTManager::push_output_callback(const boost::function<bool (const std::string&)> &slot)
+void GRTManager::push_output_callback(const boost::function<bool (std::string)> &slot)
 {
   GMutexLock lock(_tmp_output_cb_stack_mutex);
   _tmp_output_cb_stack.push_back(slot);
@@ -465,7 +458,7 @@ void GRTManager::execute_grt_task(const std::string &title,
                                   const boost::function<void (grt::ValueRef)> &finished_cb,
                                   bool show_progress)
 {
-  GRTTask *task= new GRTTask(title, _dispatcher.get(), function);
+  GRTTask *task(new GRTTask(title, _dispatcher.get(), function));
 
   if (show_progress)
   {
@@ -610,22 +603,22 @@ void GRTManager::perform_idle_tasks()
 
   if (!_idle_blocked)
   {
-    boost::signals2::signal<void ()> *idle_signal = new boost::signals2::signal<void ()>();
-    g_mutex_lock(_idle_mutex);
-    std::swap(idle_signal, _idle_signal);
-    g_mutex_unlock(_idle_mutex);
-
+    int emited_group = _current_idle_serial;
+    ++_current_idle_serial;
     // emit everything
-    (*idle_signal)();
-    delete idle_signal;
+    _idle_signal();
+
+    _idle_signal.disconnect(emited_group);
   }
 }
 
 
 boost::signals2::connection GRTManager::run_once_when_idle(const boost::function<void ()> &slot)
 {
+  if (!slot)
+    throw std::invalid_argument("Adding null slot for idle");
   GMutexLock lock(_idle_mutex);
-  return _idle_signal->connect(slot);
+  return _idle_signal.connect(_current_idle_serial, slot);
 }
 
 
@@ -1089,36 +1082,6 @@ void GRTManager::cleanup_tmp_dir()
   (void) base_rmdir_recursively(get_tmp_dir().c_str());
 }
 
-
-void GRTManager::set_use_log_file(bool value)
-{
-  if (value)
-  {
-    std::string log_filename;
-    log_filename.append(get_tmp_dir()).append("wb.log");
-    _log_file= base_fopen(log_filename.c_str(), "ab+");
-  }
-}
-
-
-void GRTManager::add_log_file_entry(const char *text, size_t size)
-{
-  if (_log_file)
-  {
-    fwrite(text, size, 1, _log_file);
-    fwrite("\n", 1, 1, _log_file);
-    fflush(_log_file);
-  }
-}
-
-
-void GRTManager::add_log_file_text(const char *text)
-{
-  if (_log_file)
-    fprintf(_log_file, "%s\n", text);
-}
-
-
 void GRTManager::push_status_text(const std::string &message)
 {
   _status_text_slot(message);
@@ -1138,7 +1101,7 @@ void GRTManager::pop_status_text()
 }
 
 
-void GRTManager::set_status_slot(const boost::function<void (const std::string&)> &slot)
+void GRTManager::set_status_slot(const boost::function<void (std::string)> &slot)
 {
   _status_text_slot= slot;
 }
