@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,6 +26,7 @@
 
 //XXX should remove this mysql specific code from here --alfredo
 #include <mysql.h> // needed for enum_field_types
+#include <boost/scoped_array.hpp>
 
 #include "db_object_helpers.h"
 #include <grtpp_undo_manager.h>
@@ -33,7 +34,7 @@
 #include "grt/common.h"
 #include "grt/parse_utils.h"
 #include "grts/structs.workbench.physical.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #ifndef _WIN32
 #include <string.h>
@@ -753,10 +754,43 @@ db_mysql_StorageEngineRef TableHelper::get_engine_by_name(grt::GRT *grt, const s
   return db_mysql_StorageEngineRef();
 }
 
+/**
+ * Returns a pointer to the position of the first line break or the first character after the
+ * maximum count, whichever comes first. If no line break was found and the string is shorter
+ * than the maximum length the result points to the terminating 0.
+ */
+gchar* find_comment_split(const std::string &comment,const size_t &db_comment_len)
+{
+  size_t res;
+  // XXX: check for Unicode line breaks! especially asian languages don't use the ANSI new line.
+  gchar* pointer_to_linebreak = g_utf8_strrchr(comment.c_str(), 0, '\n');
 
+  // We need the number of characters which the string part includes, so convert to a char count.
+  if (pointer_to_linebreak != NULL)
+    res = g_utf8_pointer_to_offset(comment.c_str(), pointer_to_linebreak);
+  else
+    res = g_utf8_strlen(comment.c_str(), -1);
 
+  if (res > db_comment_len)
+    res = db_comment_len;
 
+  // Finally convert the offset back into a pointer.
+  return g_utf8_offset_to_pointer(comment.c_str(), res);
+};
 
+std::string TableHelper::get_sync_comment(const std::string &comment)
+{
+  //TODO add comment size depending on server version
+  gchar* split_position = find_comment_split(comment, 60);
+  
+  return std::string(comment.c_str(), split_position - comment.c_str());
+};
+
+std::string TableHelper::get_document_comment(const std::string &comment)
+{
+  //TODO add comment size depending on server version
+  return std::string(find_comment_split(comment, 60));
+};
 
 void ColumnHelper::copy_column(const db_ColumnRef &from, db_ColumnRef &to)
 {
