@@ -83,8 +83,8 @@ void workbench_physical_Connection::ImplData::set_in_view(bool flag)
   if (flag)
   {
     if (!_line && diagram.is_valid() && !_realize_conn.connected())
-      _realize_conn= *scoped_connect(diagram->get_data()->signal_object_realized(),
-        boost::bind(&ImplData::object_realized, this, _1)).get();
+        _realize_conn= diagram->get_data()->signal_object_realized()->connect(
+        boost::bind(&ImplData::object_realized, this, _1));
   }
   else
     _realize_conn.disconnect();
@@ -117,18 +117,18 @@ void workbench_physical_Connection::ImplData::set_foreign_key(const db_ForeignKe
   update_connected_tables();
 
   if (!_line && owner_valid && !_realize_conn.connected())
-    _realize_conn= *scoped_connect(model_DiagramRef::cast_from(self()->owner())->get_data()->signal_object_realized(),
-    boost::bind(&ImplData::object_realized, this, _1)).get();
+    _realize_conn= model_DiagramRef::cast_from(self()->owner())->get_data()->signal_object_realized()->connect(
+    boost::bind(&ImplData::object_realized, this, _1));
 
   _fk_member_changed_conn.disconnect();
   _fk_changed_conn.disconnect();
   
   if (fk.is_valid())
   {
-    _fk_member_changed_conn= *scoped_connect(fk->signal_changed(),boost::bind(&ImplData::fk_member_changed, this, _1, _2)).get();
+      _fk_member_changed_conn= fk->signal_changed()->connect(boost::bind(&ImplData::fk_member_changed, this, _1, _2));
     
     if (fk->owner().is_valid())
-      _fk_changed_conn= *scoped_connect(fk->owner()->signal_foreignKeyChanged(),boost::bind(&ImplData::fk_changed, this, _1)).get();
+        _fk_changed_conn= fk->owner()->signal_foreignKeyChanged()->connect(boost::bind(&ImplData::fk_changed, this, _1));
   }
 }
 
@@ -281,7 +281,7 @@ void workbench_physical_Connection::ImplData::fk_member_changed(const std::strin
   {
     _fk_changed_conn.disconnect();
     if (self()->foreignKey()->owner().is_valid())
-      _fk_changed_conn= *scoped_connect(self()->foreignKey()->owner()->signal_foreignKeyChanged(),boost::bind(&ImplData::fk_changed, this ,_1)).get();
+        _fk_changed_conn= self()->foreignKey()->owner()->signal_foreignKeyChanged()->connect(boost::bind(&ImplData::fk_changed, this ,_1));
   }
 }
 
@@ -452,13 +452,13 @@ bool workbench_physical_Connection::ImplData::realize()
 
     if (workbench_physical_TableFigureRef::cast_from(self()->_startFigure)->table() == self()->_foreignKey->owner())
     {
-      _table_changed_conn= *scoped_connect(db_TableRef::cast_from(self()->_foreignKey->owner())->signal_refreshDisplay(),
-        boost::bind(&workbench_physical_Connection::ImplData::table_changed, this, _1)).get();
+        _table_changed_conn= db_TableRef::cast_from(self()->_foreignKey->owner())->signal_refreshDisplay()->connect(
+        boost::bind(&workbench_physical_Connection::ImplData::table_changed, this, _1));
     }
     else
     {
-      _table_changed_conn= *scoped_connect(self()->_foreignKey->referencedTable()->signal_refreshDisplay(),
-        boost::bind(&workbench_physical_Connection::ImplData::table_changed, this, _1)).get();
+        _table_changed_conn= self()->_foreignKey->referencedTable()->signal_refreshDisplay()->connect(
+        boost::bind(&workbench_physical_Connection::ImplData::table_changed, this, _1));
     }
 
     _line->set_start_figure(start_item);
@@ -504,8 +504,7 @@ void workbench_physical_Connection::ImplData::unrealize()
     if (_highlighting)
       set_highlighted(false);
     
-    if (_table_changed_conn.connected())
-      _table_changed_conn.disconnect();
+    _table_changed_conn.disconnect();
     
     super::unrealize();
   }
