@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,36 +29,34 @@ namespace MySQL {
 namespace GUI {
 namespace Workbench {
 
-
-using namespace System;
-using namespace System::Runtime::InteropServices;
-using namespace System::Collections::Generic;
 using namespace MySQL::Grt;
 using namespace MySQL::Grt::Db;
 using namespace MySQL::Grt::Db::Sql;
 
-
-public ref class DbSqlEditorHistory
+public ref class DbSqlEditorHistoryWrapper
 {
 public:
   typedef ManagedRef<::DbSqlEditorHistory> ^ Ref;
-  DbSqlEditorHistory(Ref ref);
+
+  DbSqlEditorHistoryWrapper(Ref ref);
   Ref ref() { return _ref; }
 private:
   Ref _ref;
-private:
-  ~DbSqlEditorHistory();
+
+  ~DbSqlEditorHistoryWrapper();
 
 public:
-  VarGridModel ^ entries_model() { return _entries_model; }
-  VarGridModel ^ details_model() { return _details_model; }
+  VarGridModelWrapper ^ entries_model() { return _entries_model; }
+  VarGridModelWrapper ^ details_model() { return _details_model; }
 private:
-  VarGridModel ^_entries_model;
-  VarGridModel ^_details_model;
+  VarGridModelWrapper ^_entries_model;
+  VarGridModelWrapper ^_details_model;
 
 public:
   void current_entry(int index) { _ref->current_entry(index);   }
   int  current_entry()          { return _ref->current_entry(); }
+
+  System::Windows::Forms::ContextMenuStrip^ get_details_context_menu();
 };
 
 
