@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,9 +25,8 @@ namespace MySQL {
 namespace Grt {
 namespace Db {
 
-
+// TODO: move this to the cpp file.
 using namespace System;
-using namespace System::Runtime::InteropServices;
 using namespace System::Collections::Generic;
 using namespace MySQL::Grt;
 
