@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,10 +33,12 @@
 #include "common/document_properties_form.h"
 
 #include "GrtTemplates.h"
-#include "SigCSlot.h"
+#include "DelegateWrapper.h"
 #include "FindDialogBE.h"
 #include "Overview.h"
-#include "ModelViewForm.h"
+#include "ModelDiagramFormWrapper.h"
+
+#include "LogWrapper.h"
 
 #if WBContext_VERSION > 5
 # error outdated wrapper Wb.h
@@ -145,14 +147,13 @@ public ref class WbOptions
   String^ run_at_startup_lang;
   bool force_sw_rendering;
   bool force_opengl_rendering;
-  bool use_log_file;
   bool quit_when_done;
 
 public:
   WbOptions(String^ basedir, String^ plugin_search_path, String^ struct_search_path,
     String^ module_search_path, String^ library_search_path, String^ user_data_dir, String^ open_at_startup, 
     String^ open_at_startup_type, String^ run_at_startup, String^ run_at_startup_lang,
-    bool force_sw_rendering, bool force_opengl_rendering, bool use_log_file, bool quit_when_done) 
+    bool force_sw_rendering, bool force_opengl_rendering, bool quit_when_done) 
     : basedir(basedir),
     plugin_search_path(plugin_search_path),
     struct_search_path(struct_search_path),
@@ -165,9 +166,16 @@ public:
     run_at_startup_lang(run_at_startup_lang),
     force_sw_rendering(force_sw_rendering),
     force_opengl_rendering(force_opengl_rendering),
-    use_log_file(use_log_file),
     quit_when_done(quit_when_done)
   {
+    Logger::LogDebug(DOMAIN_WB_CONTEXT_WRAPPER, 1, String::Format("Creating options wrapper "
+      "with these values:\n\tbase dir: {0}\n\tplugin path: {1}\n\tstruct path: {2}\n\tmodule path: {3}\n\t"
+      "library path: {4}\n\tuser data dir: {5}\n\topen at start: {6}\n\topen type: {7}\n\trun at startup: {8}\n\t"
+      "run type: {9}\n\tForce SW rendering: {10}\n\tForce OpenGL: {11}\n\tquit when done: {12}\n",
+      basedir, plugin_search_path, struct_search_path, module_search_path, library_search_path,
+      user_data_dir, open_at_startup, open_at_startup_type, run_at_startup, run_at_startup_lang,
+      force_sw_rendering, force_opengl_rendering, quit_when_done
+    ));
   }
 
   ::wb::WBOptions *get_unmanaged_object()
@@ -185,7 +193,6 @@ public:
     opt->run_language= NativeToCppString(run_at_startup_lang);
     opt->force_sw_rendering= force_sw_rendering;
     opt->force_opengl_rendering= force_opengl_rendering;
-    opt->use_log_file= use_log_file;
     opt->quit_when_done= quit_when_done;
     return opt;
   }
@@ -243,11 +250,28 @@ private:
   void create_main_form_view_wrapper(const std::string& view_name, boost::shared_ptr<bec::UIForm> form_be);
 
 public:
-  WbContext(bool verbose) : inner(new ::wb::WBContextUI(verbose)), manager(nullptr)
-  { }
+  WbContext(bool verbose)
+  {
+    inner = new ::wb::WBContextUI(verbose);
+    manager = nullptr;
+    physical_overview = nullptr;
+    catalog_tree = nullptr;
+    history_tree = nullptr;
+    usertypes_tree = nullptr;
+
+    Logger::LogDebug(DOMAIN_WB_CONTEXT_WRAPPER, 1, "Creating WbContext\n");
+  }
 
   ~WbContext()
   {
+    delete catalog_tree;
+    delete history_tree;
+    delete physical_overview;
+    delete inner;
+    delete manager;
+    delete usertypes_tree;
+
+    Logger::LogDebug(DOMAIN_WB_CONTEXT_WRAPPER, 1, "Destroying WbContext\n");
   }
 
   ::wb::WBContextUI *get_unmanaged_object() 
@@ -262,9 +286,14 @@ public:
       options->get_unmanaged_object());
   }
 
-  bool use_opengl()
+  bool opengl_rendering_enforced()
   {
-    return inner->get_wb()->use_opengl();
+    return inner->get_wb()->opengl_rendering_enforced();
+  }
+
+  bool software_rendering_enforced()
+  {
+    return inner->get_wb()->software_rendering_enforced();
   }
 
   bool is_busy()
@@ -301,53 +330,18 @@ public:
   }
 
   MenuStrip^ menu_for_form(MySQL::Grt::UIForm^ form);
+  void validate_menu_for_form(MySQL::Grt::UIForm^ form);
   ToolStrip^ toolbar_for_form(MySQL::Grt::UIForm^ form);
 
-  String^ get_title()
-  { return CppStringToNative(inner->get_title()); }
-
-  bool has_unsaved_changes()
-  { return inner->get_wb()->has_unsaved_changes(); }
-
-  void open_document(String^ file)
-  {
-    inner->get_wb()->open_document(NativeToCppString(file));
-  }
-
-  bool save_changes()
-  { return inner->get_wb()->save_changes(); }
-
-  void flush_idle_tasks()
-  {
-    try 
-    {
-      inner->get_wb()->flush_idle_tasks(); 
-    }
-    catch(std::exception *ex)
-    {
-      throw gcnew BackendException(ex);
-    }
-    catch(std::exception &ex)
-    {
-      throw gcnew BackendException(&ex);
-    }
-    catch(...)
-    {
-      throw; // Don't wrap the exception into something completely useless. The original exception
-             // often is converted automatically into a proper managed exception.
-    }
-  }
-
-  double delay_for_next_timer()
-  {
-    return inner->get_wb()->get_grt_manager()->delay_for_next_timeout();
-  }
-
-  void flush_timers()
-  {
-    return inner->get_wb()->get_grt_manager()->flush_timers();
-  }
-
+  void focus_search_box(MySQL::Grt::UIForm^ form);
+  String^ get_search_string(MySQL::Grt::UIForm^ form);
+  String^ get_title();
+  bool has_unsaved_changes();
+  void open_document(String^ file);
+  bool save_changes();
+  void flush_idle_tasks();
+  double delay_for_next_timer();
+  void flush_timers();
 
   // ----- Edit menu handling
   void validate_edit_menu()
@@ -534,153 +528,30 @@ public:
         this->search_delegate->get_slot());
   }
   
+  // Plugins/command handling.
+  void add_frontend_commands(List<String^>^ commands);
+  void remove_frontend_commands(List<String^>^ commands);
+  void activate_command(String^ name);
+  virtual FindDialogBE^ get_find_dialog();
 
-
-  // plugins
-  void add_frontend_commands(List<String^>^ commands)
-  {
-    inner->get_command_ui()->add_frontend_commands(NativeToCppStringList2(commands));
-  }
-
-  void activate_command(String^ name)
-  {
-    std::string command= NativeToCppString(name);
-    inner->get_command_ui()->activate_command(command);
-  }
-
-  virtual FindDialogBE^ get_find_dialog()
-  {
-    return gcnew FindDialogBE(inner->get_find_dialog());
-  }
-
-  // Overview
-
-  Overview^ get_physical_overview()
-  {
-    if (physical_overview == nullptr)
-      physical_overview= gcnew Overview(inner->get_physical_overview());
-    return physical_overview;
-  }
-
-  ::MySQL::Grt::GrtValueTree^ get_catalog_tree()
-  {
-    if (catalog_tree == nullptr)
-      catalog_tree= gcnew ::MySQL::Grt::GrtValueTree(inner->get_catalog_tree());
-
-    return catalog_tree;
-  }
-
-  ::MySQL::Grt::TreeModel^ get_history_tree()
-  {
-    if (history_tree == nullptr)
-      history_tree= gcnew ::MySQL::Grt::TreeModel(inner->get_history_tree());
-    
-    return history_tree;
-  }
-
-  ::MySQL::Grt::ListModel^ get_usertypes_tree()
-  {
-    if (usertypes_tree == nullptr)
-      usertypes_tree= gcnew ::MySQL::Grt::ListModel(inner->get_usertypes_tree());
-    
-    return usertypes_tree;
-  } 
-
-  ::MySQL::Grt::GrtValueInspector^ get_inspector_for_selection(UIForm^ form, [Out] List<String^>^ %items)
-  {
-    std::vector<std::string> vitems;
-    bec::ValueInspectorBE *insp= 
-      inner->create_inspector_for_selection(form->get_unmanaged_object(), vitems);
-    if (!insp)
-      insp= inner->create_inspector_for_selection(vitems);
-    if (insp)
-    {
-      items= CppStringListToNative(vitems);
-      return gcnew ::MySQL::Grt::GrtValueInspector(insp, true);
-    }
-    return nullptr;
-  }
-
-  bool are_lists_equal(GrtValue^ v1, GrtValue^ v2)
-  {
-    if ((nullptr == v1) || (nullptr == v2))
-      return (v1 == v2);
-    ::grt::ValueRef v1_(v1->get_unmanaged_object());
-    ::grt::ValueRef v2_(v2->get_unmanaged_object());
-    ::grt::ObjectListRef l1_= ::grt::ObjectListRef::cast_from(v1_);
-    ::grt::ObjectListRef l2_= ::grt::ObjectListRef::cast_from(v2_);
-
-    return ::grt::compare_list_contents(l1_, l2_);
-  }
-
-  String^ get_description_for_selection(UIForm^ form, [Out] GrtValue^ %activeObjList, [Out] List<String^>^ %items)
-  {
-    std::vector<std::string> vitems;
-    ::grt::ListRef<GrtObject> activeObjList_;
-    std::string val= 
-      inner->get_description_for_selection(form->get_unmanaged_object(), activeObjList_, vitems);
-    activeObjList= gcnew GrtValue(activeObjList_);
-    items= CppStringListToNative(vitems);
-    return CppStringToNative(val);
-  }
-
-  String^ get_description_for_selection([Out] GrtValue^ %activeObjList, [Out] List<String^>^ %items)
-  {
-    std::vector<std::string> vitems;
-    ::grt::ListRef<GrtObject> activeObjList_;
-    std::string val= inner->get_description_for_selection(activeObjList_, vitems);
-    activeObjList= gcnew GrtValue(activeObjList_);
-    items= CppStringListToNative(vitems);
-    return CppStringToNative(val);
-  }
-
-  void set_description_for_selection(GrtValue^ activeObjList, String^ val)
-  {
-    std::string val_= NativeToCppString(val);
-    ::grt::ListRef<GrtObject> activeObjList_(::grt::ListRef<GrtObject>::cast_from(activeObjList->get_unmanaged_object()));
-    inner->set_description_for_selection(activeObjList_, val_);
-  }
-
-  ::MySQL::Workbench::ModelViewForm^ get_diagram_form_for_diagram(String^ id)
-  {
-    return gcnew ModelViewForm(inner->get_wb()->get_model_context()->get_diagram_form_for_diagram_id(NativeToCppString(id)));
-  }
-
-
-  void set_active_form(UIForm^ uiform)
-  {
-    if (uiform == nullptr)
-      inner->set_active_form(0);
-    else
-      inner->set_active_form(uiform->get_unmanaged_object());
-  }
-
-  void set_active_form_from_appview(MySQL::Forms::AppViewDockContent^ form)
-  {
-    if (form == nullptr)
-      inner->set_active_form(0);
-    else
-      inner->set_active_form(form->get_unmanaged_object());
-  }
-
-  String ^get_active_context()
-  { return CppStringToNative(inner->get_active_context()); }
-
-
-  void show_output()
-  {
-    inner->show_output();
-  }
-
-  void close_gui_plugin(IntPtr handle)
-  {
-    inner->get_wb()->close_gui_plugin((NativeHandle)handle.ToPointer());
-  }
-  
-  void execute_plugin(String^ name)
-  {
-    inner->get_wb()->execute_plugin(NativeToCppStringRaw(name));
-  }
+  // Overview.
+  Overview^ get_physical_overview();
+  ::MySQL::Grt::GrtValueTree^ get_catalog_tree();
+  ::MySQL::Grt::TreeModel^ get_history_tree();
+  ::MySQL::Grt::ListModel^ get_usertypes_tree();
+  ::MySQL::Grt::GrtValueInspector^ get_inspector_for_selection(UIForm^ form, [Out] List<String^>^ %items);
+  bool are_lists_equal(GrtValue^ v1, GrtValue^ v2);
+  String^ get_description_for_selection(UIForm^ form, [Out] GrtValue^ %activeObjList, [Out] List<String^>^ %items);
+  String^ get_description_for_selection([Out] GrtValue^ %activeObjList, [Out] List<String^>^ %items);
+  void set_description_for_selection(GrtValue^ activeObjList, String^ val);
+  ::MySQL::Workbench::ModelDiagramFormWrapper^ get_diagram_form_for_diagram(String^ id);
+  void set_active_form(UIForm^ uiform);
+  void set_active_form_from_appview(MySQL::Forms::AppViewDockContent^ form);
+  String ^get_active_context();
+  void show_output();
+  void close_gui_plugin(IntPtr handle);  
+  void execute_plugin(String^ name);
+  void report_bug(String^ errorInfo);
 
   // State support.
   String^ read_state(String^ name, String^ domain, String^ default_value);
@@ -730,7 +601,9 @@ public:
   {
     inner->get_wb()->close_document_finish();
 
-    // release overview object
+    // Explicitly delete the overview object to avoid garbage collection to kick in after
+    // our internal (non-managed) objects are gone already.
+    delete physical_overview;
     physical_overview = nullptr;
   }
 
