@@ -1,3 +1,22 @@
+/* 
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 #ifndef _STDAFX_H_
 #define _STDAFX_H_
 
@@ -13,7 +32,6 @@
 #include <list>
 #include <vector>
 
-
 #include <string>
 #include <map>
 #include <set>
@@ -26,7 +44,6 @@
 #undef nil
 #define nil empty
 #endif
-
 #include <stdio.h>
 
 #define _USE_MATH_DEFINES
@@ -59,11 +76,8 @@
 #include <boost/weak_ptr.hpp>
 
 #include "GrtTemplates.h"
-#include "SigCSlot.h"
-#include "db_sql_editor_history_wr.h"
+#include "DelegateWrapper.h"
 
 #include "mforms/task_sidebar.h"
-#include "db_sql_editor_wr.h"
-#include "wb_sql_editor_snippets_wr.h"
 
 #endif // _STDAFX_H_
