@@ -23,14 +23,13 @@
 #include "grt/editor_base.h"
 #include "sql_editor_wr.h"
 #include "GrtTemplates.h"
-#include "SigCSlot.h"
+#include "DelegateWrapper.h"
 #include "UIForm.h"
 #include "GrtManager.h"
 
 #ifdef _MSC_VER
 using namespace MySQL::Grt;
 using namespace System;
-using namespace System::Runtime::InteropServices;
 using namespace System::Collections::Generic;
 #endif
 
@@ -96,7 +95,7 @@ public:
       title, MessageBoxButtons::OK, MessageBoxIcon::Error);
   }
 
-  Db::Sql::Sql_editor ^ get_sql_editor() { return Ref2Ptr<::Sql_editor, Db::Sql::Sql_editor>(get_unmanaged_object()->get_sql_editor()); }
+  Db::Sql::SqlEditorWrapper^ get_sql_editor() { return Ref2Ptr<::Sql_editor, Db::Sql::SqlEditorWrapper>(get_unmanaged_object()->get_sql_editor()); }
 
   void show_validation_error(String ^title, String ^reason)
   {
@@ -108,45 +107,6 @@ public:
   {
     return get_unmanaged_object()->should_close_on_delete_of(NativeToCppString(oid));
   }
-
-public:
-  typedef DelegateSlot3<
-    int, int,
-    long long, Int64,
-    std::string, String^,
-    std::string, String^> LiveObjectChangeErrorDelegate;
-  void set_live_object_change_error_cb(LiveObjectChangeErrorDelegate::ManagedDelegate ^cb)
-  {
-    on_live_object_change_error_delegate= gcnew LiveObjectChangeErrorDelegate(cb);
-    get_unmanaged_object()->on_live_object_change_error= on_live_object_change_error_delegate->get_slot();
-  }
-private:
-  LiveObjectChangeErrorDelegate^ on_live_object_change_error_delegate;
-
-public:
-  typedef DelegateSlot1<
-    int, int,
-    float, float> LiveObjectChangeProgressDelegate;
-  void set_live_object_change_progress_cb(LiveObjectChangeProgressDelegate::ManagedDelegate ^cb)
-  {
-    on_live_object_change_progress_delegate= gcnew LiveObjectChangeProgressDelegate(cb);
-    get_unmanaged_object()->on_live_object_change_progress= on_live_object_change_progress_delegate->get_slot();
-  }
-private:
-  LiveObjectChangeProgressDelegate^ on_live_object_change_progress_delegate;
-
-public:
-  typedef DelegateSlot2<
-    int, int,
-    int, int,
-    int, int> LiveObjectChangeStatisticsDelegate;
-  void set_live_object_change_statistics_cb(LiveObjectChangeStatisticsDelegate::ManagedDelegate ^cb)
-  {
-    on_live_object_change_statistics_delegate= gcnew LiveObjectChangeStatisticsDelegate(cb);
-    get_unmanaged_object()->on_live_object_change_statistics= on_live_object_change_statistics_delegate->get_slot();
-  }
-private:
-  LiveObjectChangeStatisticsDelegate^ on_live_object_change_statistics_delegate;
 };
 
 
