@@ -155,6 +155,22 @@ static std::vector<int> IntListToCppVector(List<int>^ input)
 
 //--------------------------------------------------------------------------------------------------
 
+static Dictionary<String^, String^>^ CppStringMapToDictionary(const std::map<std::string, std::string>& input)
+{
+  typedef const std::map<std::string, std::string> SourceContainerType;
+  typedef Dictionary<String^, String^> TargetContainerType;
+
+  TargetContainerType^ result = gcnew TargetContainerType(static_cast<int>(input.size()));
+  SourceContainerType::const_iterator e = input.end();
+
+  for(SourceContainerType::const_iterator iterator = input.begin(); iterator != input.end(); ++iterator)
+    result->Add(CppStringToNativeRaw(iterator->first), CppStringToNativeRaw(iterator->second));
+
+  return result;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 static MySQL::Drawing::Color NativeToColor(System::Drawing::Color^ color)
 {
   return MySQL::Drawing::Color(color->R / 255.0, color->G / 255.0, color->B / 255.0, color->A / 255.0);
