@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,9 +21,7 @@
 #define __EXCEPTIONS_H__
 
 #ifdef _MSC_VER
-//#include  <vcclr.h> // .net interop helpers
 using namespace System;
-using namespace System::Runtime::InteropServices;
 using namespace System::Collections::Generic;
 #endif
 
