@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,6 +24,7 @@
 #import "GRTIconCache.h"
 #import "MCPPUtilities.h"
 #import "WBMiniToolbar.h"
+#import "MVerticalLayoutView.h"
 
 static int onRefresh(MResultsetViewer *self);
 static void updateToolbar(MResultsetViewer *self);
@@ -42,7 +43,7 @@ static NSImage *descendingSortIndicator= nil;
   
   for (int index= 0, count= (*mData)->get_column_count(); index < count; ++index)
   {
-    std::string label= (*mData)->get_column_caption(index);
+    std::string label= base::sanitize_utf8((*mData)->get_column_caption(index));
     //bec::GridModel::ColumnType type= (*mData)->get_column_type(index);
     NSTableColumn *column= [[[NSTableColumn alloc] initWithIdentifier: [NSString stringWithFormat:@"%i", index]] autorelease];
 
@@ -78,42 +79,10 @@ static NSImage *descendingSortIndicator= nil;
   [super dealloc];
 }
 
-
+/*
 static int processTaskMessage(int msgType, const std::string &message, const std::string &detail,
                               MResultsetViewer *self)
 {
-  /*
-  NSRect rect= [[self->mTableView enclosingScrollView] frame];
-  NSColor *color= [NSColor blackColor];
-  
-  if (message.empty())
-  {
-    [self->mMessagesText setHidden: YES];
-    
-    rect.origin.y= 20;
-    rect.size.height= NSHeight([self->mView frame]) - 20;
-  }
-  else
-  {
-    [self->mMessagesText setHidden: NO];
-    
-    rect.origin.y= 50;
-    rect.size.height= NSHeight([self->mView frame]) - 50;
-  }
-  
-  [[self->mTableView enclosingScrollView] setFrame: rect];
-    
-  switch (msgType)
-  {
-    case grt::ErrorMsg:
-      color= [NSColor redColor];
-      break;
-  }
-  [self->mMessagesText setAttributedStringValue: 
-   [[[NSAttributedString alloc] initWithString:[NSString stringWithUTF8String: message.c_str()]
-                                    attributes:[NSDictionary dictionaryWithObject:color 
-                                                                           forKey:NSForegroundColorAttributeName]] autorelease]];
-   */
   if (msgType == grt::ErrorMsg)
     self->mGotError = YES;
   
@@ -130,7 +99,7 @@ static int processTaskMessage(int msgType, const std::string &message, const std
 //  (*self->mData)->grtm()->replace_status_text(message);
   
   return 0;
-}
+}*/
 
 - (void)clickedTable:(id)sender
 {
@@ -170,11 +139,15 @@ static int processTaskMessage(int msgType, const std::string &message, const std
     
     (*mData)->refresh_ui_cb= boost::bind(onRefresh, self);
     mSigConns.push_back((*mData)->refresh_ui_status_bar_signal.connect(boost::bind(updateToolbar, self)));
-    (*mData)->task->msg_cb(boost::bind(processTaskMessage, _1, _2, _3, self));
+    //(*mData)->task->msg_cb(boost::bind(processTaskMessage, _1, _2, _3, self));
     
     [mTableView selectionChangedActionTarget:self];
     [mTableView setSelectionChangedAction:@selector(handleNSTableViewSelectionIsChangingNotification:)];
+    
+    [mView setExpandSubviewsByDefault: YES]; // makes the resultset part expand inside the VerticalLayouter
 
+    [mView tile];
+    
     onRefresh(self);
   }
   return self;
@@ -195,7 +168,10 @@ static int processTaskMessage(int msgType, const std::string &message, const std
   return mView;
 }
 
-
+- (MGridView*)gridView
+{
+  return mTableView;
+}
 
 - (void)updateToolbar
 {
@@ -405,14 +381,14 @@ static void updateToolbar(MResultsetViewer *self)
         NSLog(@"unhandled toolbar action %s", action.c_str());
     }
   }
-  
+/*  
   if (mErrorMessage)
   {
     NSRunAlertPanel(@"Error", @"%@", @"OK", nil, nil, mErrorMessage);
     [mErrorMessage release];
     mErrorMessage= nil;
     mGotError= NO;
-  }
+  }*/
   [self updateToolbar];
 }
 
@@ -520,12 +496,17 @@ static int onRefresh(MResultsetViewer *self)
     [aCell setIsNull: (*mData)->is_field_null(rowIndex, columnIndex)];
     [aCell setIsBlob: (*mData)->get_column_type(columnIndex) == bec::GridModel::BlobType];
     
-    if ( (columnIndex+1 == selectedColumnIndex) && (rowIndex == selectedRowIndex) ) {
+    if ( (columnIndex+1 == selectedColumnIndex) && (rowIndex == selectedRowIndex) ) 
+    {
       [aCell setDrawsBackground: YES];
-      [aCell setBackgroundColor: [NSColor selectedTextBackgroundColor]];// selectedMenuItemColor]];
+      if ([[aTableView window] firstResponder] != aTableView)
+        [aCell setBackgroundColor: [NSColor lightGrayColor]];
+      else
+        [aCell setBackgroundColor: [NSColor alternateSelectedControlColor]];// selectedMenuItemColor]];
      // [aCell setTextColor: [NSColor whiteColor]];
     }
-    else {
+    else 
+    {
       [aCell setDrawsBackground: NO];
       [aCell setTextColor: [NSColor blackColor]];
     }
@@ -564,40 +545,36 @@ static int onRefresh(MResultsetViewer *self)
 - (void)saveEdits:(id)sender
 {
   [mStatusText setStringValue: @"Applying changes..."];
-  (*mData)->apply_changes();
-  if (!mErrorMessage || !mGotError)
-    [mStatusText setStringValue: mErrorMessage ?: @"Changes applied."];
+  std::string messages;
+  if ((*mData)->apply_changes_and_gather_messages(messages))
+    [mStatusText setStringValue: messages.empty() ? [NSString stringWithCPPString: messages]: @"Changes applied."];
   else
   {
     [mStatusText setStringValue: @"Apply failed."];
-    NSRunAlertPanel(@"Apply Changes Failed", @"%@", @"OK", nil, nil, mErrorMessage);
+    NSRunAlertPanel(@"Apply Changes Failed", @"%@", @"OK", nil, nil, [NSString stringWithCPPString: messages]);
   }
-  [mErrorMessage release];
-  mErrorMessage= nil;
-  mGotError= NO;
 }
 
 
 - (void)discardEdits:(id)sender
 {
   [mStatusText setStringValue: @"Discarding changes..."];
-  (*mData)->rollback();
-  if (!mErrorMessage || !mGotError)
-    [mStatusText setStringValue: mErrorMessage ?: @"Changes discarded."];
+  std::string messages;
+  (*mData)->rollback_and_gather_messages(messages);
+  if (messages.empty())
+    [mStatusText setStringValue: @"Changes discarded."];
   else
   {
     [mStatusText setStringValue: @"Discard failed."];
-    NSRunAlertPanel(@"Discard Changes Failed", @"%@", @"OK", nil, nil, mErrorMessage);
+    NSRunAlertPanel(@"Discard Changes Failed", @"%@", @"OK", nil, nil, [NSString stringWithCPPString: messages]);
   }
-  [mErrorMessage release];
-  mErrorMessage= nil;
-  mGotError= NO;
 }
 
 
 - (void)close
 {
   (*mData)->close();
+  /*
   if (mErrorMessage && mGotError)
   {
     [mStatusText setStringValue: @"Apply failed."];
@@ -605,7 +582,7 @@ static int onRefresh(MResultsetViewer *self)
   }
   [mErrorMessage release];
   mErrorMessage= nil;
-  mGotError= NO;
+  mGotError= NO;*/
 }
 
 - (void) tableView: (NSTableView *) tableView
