@@ -20,12 +20,12 @@
 #import "WBModelSidebarController.h"
 
 @class WBMainController;
-@class WBQuerySidebarController;
 @class WBBasePanel;
 @class WBModelOverviewPanel;
 @class WBSplitView;
 @class WBSplitViewUnbrokenizerDelegate;
 @class MContainerView;
+@class MTabSwitcher;
 
 @class WBPluginEditorBase;
 
@@ -36,17 +36,13 @@
 }
 @end
 
-
+class MacNotificationObserver;
 
 @interface WBMainWindow : NSWindowController
 {
   IBOutlet NSTextField *statusBarText;
-
   IBOutlet NSTabView *topTabView;
-
-  IBOutlet NSMenu *contextMenu;
-  
-  IBOutlet NSView *tabSwitcher;
+  IBOutlet MTabSwitcher *tabSwitcher;
   
   NSMutableDictionary *_panels;
   
@@ -55,6 +51,8 @@
   NSMutableArray *_closedTopTabs;
   NSMutableArray *_closedBottomTabs;
   
+  NSImage *_homeIcon;
+  
   WBMainController *_mainController;
   
   NSMenu *_defaultMainMenu;
@@ -66,6 +64,8 @@
   wb::WBContextUI *_wbui;
   
   NSTimeInterval _lastClick;
+  
+  MacNotificationObserver *_backendObserver;
 }
 
 - (void)load;
@@ -76,7 +76,8 @@
 - (NSTabViewItem*)addTopPanel:(WBBasePanel*)panel;
 - (NSTabViewItem*)addTopPanelAndSwitch:(WBBasePanel*)panel;
 - (void)addBottomPanel:(WBBasePanel*)panel;
-- (WBBasePanel*) findPanelForPluginType: (Class) type;
+- (WBBasePanel*)findPanelForPluginType: (Class) type;
+- (WBBasePanel*)findMainPanelForUIForm: (bec::UIForm*)form;
 - (void)activatePanel: (WBBasePanel*) panel;
 - (void)setTitle:(NSString*)title
         forPanel:(WBBasePanel*)panel;
@@ -85,6 +86,7 @@
 
 - (WBBasePanel*)selectedTopPanel;
 
+- (IBAction)handleMenuAction:(id)sender;
 - (IBAction)showMySQLOverview:(id)sender;
 
 - (void)focusSearchField:(id)sender;
@@ -119,7 +121,7 @@
 - (WBBasePanel*)activePanel;
 - (WBBasePanel*)selectedMainPanel;
 
-- (void) forwardCommandToPanels: (const std::string) command;
+- (void)forwardCommandToPanels: (const std::string) command;
 
 - (void)firstResponderChanged: (NSResponder*)responder;
 
