@@ -34,8 +34,8 @@ namespace wb {
   
   wb::WBContextUI *_wbui;
   
-  sigc::connection *_historyRefresh;
-  sigc::connection *_userTypesRefresh;
+  boost::signals2::connection _historyRefresh;
+  boost::signals2::connection _userTypesRefresh;
 }
 
 - (IBAction)activateCatalogItem:(id)sender;
