@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -116,9 +116,9 @@ namespace MySQL.Utilities
     // Drawing resources.
     private Brush fontDarkBrush = new SolidBrush(Color.FromArgb(30, 30, 30));
 
-    private Font headerFont = ControlUtilities.getFont("Trebuchet MS", 11, FontStyle.Bold);
-    private Font tabHeaderCaptionFont = ControlUtilities.getFont("Trebuchet MS", 11, FontStyle.Bold);
-    private Font tabHeaderDescriptionFont = ControlUtilities.getFont("Trebuchet MS", 7);
+    private Font headerFont = ControlUtilities.GetFont("Trebuchet MS", 11, FontStyle.Bold);
+    private Font tabHeaderCaptionFont = ControlUtilities.GetFont("Trebuchet MS", 11, FontStyle.Bold);
+    private Font tabHeaderDescriptionFont = ControlUtilities.GetFont("Trebuchet MS", 7);
 
     // Custom buttons. They are painted in the header, right aligned, left to the +/- buttons,
     // from left to right (index 0 is at the left side).
