@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,16 +20,13 @@
 using System;
 using System.Collections.Generic;
 using System.ComponentModel;
-using System.ComponentModel.Design;
 using System.Drawing;
-using System.Drawing.Design;
 using System.Drawing.Drawing2D;
 using System.Runtime.InteropServices;
 using System.Windows.Forms;
 
 using MySQL.Utilities;
 using MySQL.Utilities.SysUtils;
-using System.Drawing.Text;
 
 namespace MySQL.Controls
 {
@@ -66,6 +63,7 @@ namespace MySQL.Controls
       internal TabPage page;
       internal bool isValid;
       internal CloseButtonVisiblity closeButtonVisibility; // For individual close buttons.
+      internal bool isBusy; // Show a busy indicator is to be shown for that tab (in place of the close button).
       internal Rectangle tabArea;
       internal Rectangle buttonArea;
 
@@ -83,9 +81,9 @@ namespace MySQL.Controls
 		private System.ComponentModel.Container components = null;
 
     private ImageList leftRightImages = null;
-    private Bitmap darkCloseButton;
-    private Bitmap lightCloseButton;
-    private Size buttonSize;
+    private Bitmap darkCloseButton = null;
+    private Bitmap lightCloseButton = null;
+    private Dictionary<string, Bitmap> busyIndicators = new Dictionary<string, Bitmap>();
     private SubClass scroller = null;
     private Form activationTracker = null;
 
@@ -95,9 +93,10 @@ namespace MySQL.Controls
     private Color topStyleFocusedTextColor = Color.FromArgb(0xff, 0xcc, 0xd3, 0xdf);
     private Color topNormalFocusedTabGradientStart = Color.FromArgb(0xff, 0xff, 0xe9, 0xb6);
 
+    private int glowSize = 12; // The size of the glow behind text on glass tabs.
+
     private bool showCloseButton = true; // Central flag for close buttons. Can be overridden by each page.
     private bool showFocusState = true;
-    private bool canCloseLastTab = false;
     private bool hideWhenEmpty = false;
 
     // Painting/Layouting
@@ -122,8 +121,8 @@ namespace MySQL.Controls
 
     // Mouse handling.
     private int lastTabHit = -1;
+    private Point lastClickPosition;
     private bool buttonHit = false;
-    private bool inWheelHandling = false;
 
     #region Construction and destruction
 
@@ -144,7 +143,7 @@ namespace MySQL.Controls
 			leftRightImages = new ImageList();
 
 			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FlatTabControl));
-			Bitmap icons = ((System.Drawing.Bitmap)(resources.GetObject("TabIcons.bmp")));
+			Bitmap icons = ((System.Drawing.Bitmap)(resources.GetObject("TabIcons")));
 			if (icons != null)
 			{
 				icons.MakeTransparent(Color.White);
@@ -154,8 +153,9 @@ namespace MySQL.Controls
       // Both buttons really should have the same size.
       darkCloseButton = ((System.Drawing.Bitmap)(resources.GetObject("tab_close_dark")));
       lightCloseButton = ((System.Drawing.Bitmap)(resources.GetObject("tab_close_light")));
-      
-      buttonSize = new Size(darkCloseButton.Width, darkCloseButton.Height);
+
+      CanReorderTabs = false;
+      AllowDrop = false;
     }
 
     protected override void OnCreateControl()
@@ -178,7 +178,7 @@ namespace MySQL.Controls
 		{
 			if (disposing)
 			{
-				if(components != null)
+				if (components != null)
 				{
 					components.Dispose();
 				}
@@ -186,6 +186,13 @@ namespace MySQL.Controls
 				leftRightImages.Dispose();
         darkCloseButton.Dispose();
         lightCloseButton.Dispose();
+
+        foreach (Bitmap image in busyIndicators.Values)
+        {
+          // Must be the same delegate we used to start the animation with!
+          ImageAnimator.StopAnimate(image, new EventHandler(BusyAnimationStep));
+          // image.Dispose(); Images are references to our resource bundle and need to disposal.
+        }
       }
 			base.Dispose(disposing);
 		}
@@ -284,30 +291,6 @@ namespace MySQL.Controls
 
           break;
         }
-
-        case WM.MOUSEWHEEL:
-          if (!inWheelHandling)
-          {
-            inWheelHandling = true;
-
-            // Forward mouse wheel messages to the control under the mouse pointer.
-            // Only handle messages if the mouse pointer is in the form's client area.
-            // Also check we don't have any other window (e.g. dialogs) in front.
-            Win32.POINT pos = new Win32.POINT(m.LParam.ToInt32() & 0xffff, m.LParam.ToInt32() >> 16);
-            IntPtr targetWindow = Win32.WindowFromPoint(pos);
-            if (targetWindow != IntPtr.Zero)
-            {
-              Control target = Control.FromHandle(targetWindow);
-              if ((targetWindow != m.HWnd) && (target != null) && (target != this))
-              {
-                Win32.SendMessage(targetWindow, (uint) m.Msg, m.WParam, m.LParam);
-                m.Result = IntPtr.Zero;
-              }
-            }
-            inWheelHandling = false;
-          }
-          break;
-
         default:
           base.WndProc(ref m);
 
@@ -500,37 +483,40 @@ namespace MySQL.Controls
           g.DrawLine(grayPen, Margin.Left - 0.5f, Margin.Top + ItemSize.Height - 1, rightBorder, Margin.Top + ItemSize.Height - 1);
 
         // Fade out the gray line on the left hand side of the active tab (if it is not the first one).
-        RectangleF area = GetTabRect(SelectedIndex);
-        area.X += 0.5f;
-        if (SelectedIndex > 0)
+        if (SelectedIndex >= 0)
         {
+          RectangleF area = GetTabRect(SelectedIndex);
+          area.X += 0.5f;
+          if (SelectedIndex > 0)
+          {
+            using (LinearGradientBrush gradientBrush = new LinearGradientBrush(
+              new RectangleF(area.Left - 2, 0, 4, 1),
+              Color.FromArgb(0xff, 0x92, 0x9b, 0xa2), Color.FromArgb(0xff, 0xd6, 0xdc, 0xe5),
+              LinearGradientMode.Horizontal))
+            {
+              using (Pen pen = new Pen(gradientBrush))
+                g.DrawLine(pen, area.Left - 2, area.Bottom - 1, area.Left + 3, area.Bottom - 1);
+            }
+          }
+
+
+          // The color to erase the separator line is what is the final color of the tab gradient.
+          using (Pen lightPen = new Pen(Color.FromArgb(0xff, 0xd6, 0xdc, 0xe5)))
+          {
+            g.DrawLine(lightPen, area.Left, Margin.Top + ItemSize.Height - 1, area.Right - 2,
+              Margin.Top + ItemSize.Height - 1);
+          }
+
+          // Fade out the gray line on the right hand side of the active tab.
           using (LinearGradientBrush gradientBrush = new LinearGradientBrush(
-            new RectangleF(area.Left - 2, 0, 4, 1),
-            Color.FromArgb(0xff, 0x92, 0x9b, 0xa2), Color.FromArgb(0xff, 0xd6, 0xdc, 0xe5),
+            new RectangleF(area.Right - 2, 0, 3, 1),
+            Color.FromArgb(0xff, 0xd6, 0xdc, 0xe5), Color.FromArgb(0xff, 0x92, 0x9b, 0xa2),
             LinearGradientMode.Horizontal))
           {
             using (Pen pen = new Pen(gradientBrush))
-              g.DrawLine(pen, area.Left - 2, area.Bottom - 1, area.Left + 3, area.Bottom - 1);
+              g.DrawLine(pen, area.Right - 2, area.Bottom - 1, area.Right + 1, area.Bottom - 1);
           }
         }
-
-
-        // The color to erase the separator line is what is the final color of the tab gradient.
-        using (Pen lightPen = new Pen(Color.FromArgb(0xff, 0xd6, 0xdc, 0xe5)))
-        {
-          g.DrawLine(lightPen, area.Left, Margin.Top + ItemSize.Height - 1, area.Right - 2,
-            Margin.Top + ItemSize.Height - 1);
-        }
-
-        // Fade out the gray line on the right hand side of the active tab.
-        using (LinearGradientBrush gradientBrush = new LinearGradientBrush(
-          new RectangleF(area.Right - 2, 0, 3, 1),
-          Color.FromArgb(0xff, 0xd6, 0xdc, 0xe5), Color.FromArgb(0xff, 0x92, 0x9b, 0xa2), 
-          LinearGradientMode.Horizontal))
-        {
-          using (Pen pen = new Pen(gradientBrush))
-            g.DrawLine(pen, area.Right - 2, area.Bottom - 1, area.Right + 1, area.Bottom - 1);
-        }
       }
 		}
 
@@ -538,18 +524,23 @@ namespace MySQL.Controls
     {
 	    RectangleF bounds = GetTabRect(index);
 
-      // For high quality drawing with anti aliased lines we need to specify line and gradient coordinates
-      // which lie between two pixels (not exactly *on* one, so adjust by a half pixel offset for these cases.
-      bounds.X -= 0.5f;
-      if (tabStyle != TabStyleType.TopTransparent)
-        bounds.Y -= 0.5f;
-      
       TabPage page = TabPages[index];
-
       bool isSelected = (SelectedIndex == index);
+
+      // For high quality drawing with antialiased lines we need to specify line and gradient coordinates
+      // which lie between two pixels (not exactly *on* one, so adjust by a half pixel offset for these cases.
+      if (tabStyle != TabStyleType.TopTransparent)
+      {
+        bounds.X -= 0.5f;
+        bounds.Y -= 0.5f;
+      }
+      else
+        if (isSelected)
+          bounds.X -= 0.5f;
+
       if (isSelected || tabStyle == TabStyleType.TopTransparent)
       {
-        GraphicsPath outline = GetTabOutline(ref bounds, index == 0);
+        GraphicsPath outline = GetTabOutline(bounds, index == 0);
 
         // Draw interior first.
         GraphicsPath interior = (GraphicsPath) outline.Clone();
@@ -607,25 +598,32 @@ namespace MySQL.Controls
         }
       }
 
-      bounds.Offset(itemPadding.Left, 0);
-      
-      // Tab icon.
+      bool drawingOnGlass = (tabStyle == TabStyleType.TopTransparent &&
+                             !isSelected &&
+                             ControlUtilities.IsCompositionEnabled());
+
+      // Tab icon. Compute position here but draw after the text (to draw it over the text glow
+      // if necessary).
+      bool drawImage = false;
+      Point imagePosition = new Point(0, 0);
       if ((page.ImageIndex >= 0) && (ImageList != null) && (ImageList.Images[page.ImageIndex] != null))
       {
+        drawImage = true;
         int iconSpacing = 4;
 
-        Point position = new Point((int) bounds.X, (int) bounds.Y);
+        imagePosition = new Point((int) bounds.X + itemPadding.Left, (int) bounds.Y);
 				
         int adjustment = ImageList.ImageSize.Width + iconSpacing;
 
-        position.Y += 1 + ((int)bounds.Height - ImageList.ImageSize.Height) / 2;
+        imagePosition.Y += 1 + ((int)bounds.Height - ImageList.ImageSize.Height) / 2;
         bounds.X += adjustment;
         bounds.Width -= adjustment;
-        
-        ImageList.Draw(g, position, page.ImageIndex);
       }
 
-			// Tab text.
+      bool buttonSpaceNeeded = IsCloseButtonVisible(index) || (layoutInfo[index].isBusy);
+      Rectangle buttonRect = GetTabButtonRect(index);
+      
+      // Tab text.
       if (page.Text.Length > 0)
       {
         using (StringFormat stringFormat = new StringFormat())
@@ -635,6 +633,10 @@ namespace MySQL.Controls
           stringFormat.FormatFlags = StringFormatFlags.NoWrap;
           stringFormat.LineAlignment = StringAlignment.Center;
 
+          // Create format flags variant of the StringFormat for aero rendering.
+          TextFormatFlags formatFlags = TextFormatFlags.EndEllipsis | TextFormatFlags.VerticalCenter |
+            TextFormatFlags.SingleLine;
+
           Brush brush;
           if (tabStyle == TabStyleType.TopTransparent)
           {
@@ -661,9 +663,23 @@ namespace MySQL.Controls
               brush = new SolidBrush(Color.White);
 
           int buttonPart = 0;
-          if (IsCloseButtonVisible(index))
-            buttonPart = buttonSpacing + buttonSize.Width;
-          bounds.Width -= itemPadding.Horizontal + buttonPart;
+          if (buttonSpaceNeeded)
+            buttonPart = buttonSpacing + buttonRect.Width;
+
+          // When computing the left padding take the necessary glow around text into account, so we
+          // don't add too much extra space.
+          if (drawingOnGlass)
+          {
+            int leftSpace = itemPadding.Left > glowSize ? itemPadding.Left - glowSize : 0;
+            int rightSpace = itemPadding.Right > glowSize ? itemPadding.Right - glowSize : 0;
+            bounds.X += leftSpace;
+            bounds.Width -= buttonPart + leftSpace + rightSpace;
+          }
+          else
+          {
+            bounds.X += itemPadding.Left;
+            bounds.Width -= itemPadding.Horizontal + buttonPart;
+          }
           bounds.Height -= itemPadding.Vertical;
 
           // For writing text on a (semi) transparent surface (e.g. when using Aero)
@@ -677,27 +693,62 @@ namespace MySQL.Controls
           }
           else
           {
-            using (GraphicsPath textPath = new GraphicsPath())
+            if (ControlUtilities.IsCompositionEnabled())
             {
-              float emSize = g.DpiY * Font.SizeInPoints / 72;
-              if (tabStyle == TabStyleType.TopTransparent)
-                bounds.Y--; // Account for outline border.
-              textPath.AddString(page.Text, Font.FontFamily, (int)Font.Style, emSize, bounds, stringFormat);
-              g.FillPath(brush, textPath);
+              Rectangle intBounds = Rectangle.Ceiling(bounds);
+              Win32.DrawTextOnGlass(g, page.Text, Font, intBounds, Color.Black, formatFlags, true);
             }
+            else
+              using (GraphicsPath textPath = new GraphicsPath())
+              {
+                float emSize = g.DpiY * Font.SizeInPoints / 72;
+                if (tabStyle == TabStyleType.TopTransparent)
+                  bounds.Y--; // Account for outline border.
+                textPath.AddString(page.Text, Font.FontFamily, (int)Font.Style, emSize, bounds, stringFormat);
+                g.FillPath(brush, textPath);
+              }
           }
           brush.Dispose();
         }
       }
 
-      // Finally the close button if this tab is active (or we use the TopTransparent style).
-      if ((isSelected || tabStyle == TabStyleType.TopTransparent) && IsCloseButtonVisible(index))
+      if (drawImage)
+        ImageList.Draw(g, imagePosition, page.ImageIndex);
+
+      // Finally the close button if this tab is active (or we use the TopTransparent style or
+      // the tab is marked as busy).
+      if ((isSelected || tabStyle == TabStyleType.TopTransparent || layoutInfo[index].isBusy) &&
+        buttonSpaceNeeded)
       {
-        Rectangle buttonRect = CloseButtonRect(index);
-        if (drawFocused || tabStyle == TabStyleType.TopTransparent)
-          g.DrawImage(darkCloseButton, buttonRect);
+        if (layoutInfo[index].isBusy)
+        {
+          Bitmap indicator;
+          switch (tabStyle)
+          {
+            case TabStyleType.TopNormal:
+              if (isSelected)
+              {
+                if (drawFocused)
+                  indicator = GetBusyIndicatorForStyle("yellow");
+                else
+                  indicator = GetBusyIndicatorForStyle("blue");
+              }
+              else
+                indicator = GetBusyIndicatorForStyle("darkblue");
+              break;
+
+            default:
+              indicator = GetBusyIndicatorForStyle("white");
+              break;
+          }
+          ImageAnimator.UpdateFrames(indicator);
+          g.DrawImageUnscaled(indicator, buttonRect);
+        }
         else
-          g.DrawImage(lightCloseButton, buttonRect);
+          if (drawFocused || tabStyle == TabStyleType.TopTransparent)
+            g.DrawImage(darkCloseButton, buttonRect);
+          else
+            g.DrawImage(lightCloseButton, buttonRect);
       }
 		}
 
@@ -706,7 +757,7 @@ namespace MySQL.Controls
     /// </summary>
     /// <param name="bounds">The bounding rectangle for the tab.</param>
     /// <returns>The outline of the tab as path, ready to be filled.</returns>
-    private GraphicsPath GetTabOutline(ref RectangleF recBounds, bool isFirst)
+    private GraphicsPath GetTabOutline(RectangleF recBounds, bool isFirst)
     {
       GraphicsPath path = new GraphicsPath();
       float cornerSize = 6.5f;
@@ -878,7 +929,7 @@ namespace MySQL.Controls
     /// </summary>
     /// <param name="index"></param>
     /// <returns></returns>
-    public Rectangle CloseButtonRect(int index)
+    public Rectangle GetTabButtonRect(int index)
     {
       ValidateTab(index);
 
@@ -887,6 +938,36 @@ namespace MySQL.Controls
       return area;
     }
 
+    /// <summary>
+    /// Checks if the busy animation for the given style has been loaded already. If not
+    /// the image is loaded and the animation for it started.
+    /// </summary>
+    /// <returns>The image registered as busy animation.</returns>
+    internal Bitmap GetBusyIndicatorForStyle(string type)
+    {
+      if (!busyIndicators.ContainsKey(type))
+      {
+        System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FlatTabControl));
+        switch (type)
+        {
+          case "blue":
+            busyIndicators[type] = ((System.Drawing.Bitmap)(resources.GetObject("busy-indicator-blue")));
+            break;
+          case "darkblue":
+            busyIndicators[type] = ((System.Drawing.Bitmap)(resources.GetObject("busy-indicator-darkblue")));
+            break;
+          case "yellow":
+            busyIndicators[type] = ((System.Drawing.Bitmap)(resources.GetObject("busy-indicator-yellow")));
+            break;
+          default:
+            busyIndicators[type] = ((System.Drawing.Bitmap)(resources.GetObject("busy-indicator-white")));
+            break;
+        }
+        ImageAnimator.Animate(busyIndicators[type], new EventHandler(BusyAnimationStep));
+      }
+      return busyIndicators[type];
+    }
+
 		internal void DrawScroller(Graphics g)
 		{
 			if ((leftRightImages == null) || (leftRightImages.Images.Count != 4))
@@ -976,7 +1057,7 @@ namespace MySQL.Controls
         if (s == "msctls_updown32")
         {
           scroller = new SubClass(pWnd, true);
-          scroller.SubClassedWndProc += new SubClass.SubClassWndProcEventHandler(ScrollerWndProc); //LLL not deleted anywhere
+          scroller.SubClassedWndProc += new SubClass.SubClassWndProcEventHandler(ScrollerWndProc);
           break;
         }
 
@@ -1004,9 +1085,9 @@ namespace MySQL.Controls
           Win32.ShowWindow(scroller.Handle, (int)SW.SHOWNOACTIVATE);
 
           // The scroll range is the number of steps (scaled pixels) we need to scroll over the full tab space.
-          int scrollRange = (int)Math.Ceiling((totalTabWidth - availableWidth) / (float)scrollScaleFactor) +
-            (int)((scrollerSpacing / (float)scrollScaleFactor));
-          Win32.SendMessage(scroller.Handle, (int)UDM.SETRANGE32, IntPtr.Zero, new IntPtr(scrollRange));
+          int scrollRange = (int) Math.Ceiling((totalTabWidth - availableWidth) / (float) scrollScaleFactor) +
+            (int) ((scrollerSpacing / (float) scrollScaleFactor));
+          Win32.SendMessage(scroller.Handle, (int) UDM.SETRANGE32, IntPtr.Zero, new IntPtr(scrollRange));
 
           // Make sure the scroll position is still ok before setting it in the scroller.
           // Always scroll so that the right border stays close to the scroller.
@@ -1021,7 +1102,7 @@ namespace MySQL.Controls
           acceleration.nSec = 0;
           IntPtr parameter = Marshal.AllocHGlobal(Marshal.SizeOf(acceleration));
           Marshal.StructureToPtr(acceleration, parameter, true);
-          Win32.SendMessage(scroller.Handle, (uint)UDM.SETACCEL, new IntPtr(1), parameter);
+          Win32.SendMessage(scroller.Handle, (uint) UDM.SETACCEL, new IntPtr(1), parameter);
           Marshal.FreeHGlobal(parameter);
         }
         else
@@ -1112,6 +1193,8 @@ namespace MySQL.Controls
       {
         TabPage page = e.Control as TabPage;
         page.TextChanged += new EventHandler(PageTextChanged);
+        if (tabStyle == TabStyleType.BottomNormal)
+          page.BackColor = Color.White;
         AdjustLayoutInfo(e.Control as TabPage);
         UpdateScroller();
 
@@ -1160,15 +1243,11 @@ namespace MySQL.Controls
           lastTabHit = TabIndexFromPosition(e.Location);
           if (lastTabHit > -1)
           {
+            lastClickPosition = e.Location;
             if (SelectedIndex != lastTabHit)
-            {
-              SuspendLayout();
               SelectedIndex = lastTabHit;
-              ResumeLayout(true);
-            }
             else
-              if (CloseButtonRect(lastTabHit).Contains(e.Location))
-                buttonHit = true;
+              buttonHit = GetTabButtonRect(lastTabHit).Contains(e.Location);
           }
           break;
 
@@ -1185,8 +1264,8 @@ namespace MySQL.Controls
         case MouseButtons.Left:
           if (lastTabHit > -1)
           {
-            if (CloseButtonRect(lastTabHit).Contains(e.Location) && buttonHit &&
-              (TabCount > 1 || canCloseLastTab))
+            if (buttonHit && GetTabButtonRect(lastTabHit).Contains(e.Location) &&
+              (TabCount > 1 || CanCloseLastTab))
             {
               // Close tab if the application agrees.
               TabPage page = TabPages[lastTabHit];
@@ -1201,6 +1280,87 @@ namespace MySQL.Controls
       }
     }
 
+    protected override void OnMouseMove(MouseEventArgs e)
+    {
+      switch (e.Button)
+      {
+        case MouseButtons.Left:
+          if (lastTabHit > -1 && !buttonHit && CanReorderTabs &&
+            (Math.Abs(lastClickPosition.X - e.Location.X) > 5 || Math.Abs(lastClickPosition.Y - e.Location.Y) > 5))
+          {
+            TabPage page = TabPages[lastTabHit];
+            if (DoDragDrop(page, DragDropEffects.Move) == DragDropEffects.Move)
+            {
+              int newIndex = TabPages.IndexOf(page);
+              OnTabMoved(new TabMovedEventArgs(lastTabHit, newIndex));
+            }
+          }
+          else
+            base.OnMouseMove(e);
+          break;
+
+        default:
+          base.OnMouseMove(e);
+          break;
+      }
+    }
+
+    protected override void OnDragOver(DragEventArgs e)
+    {
+      // See if a tab page is being dragged.
+      if (e.Data.GetData(typeof(TabPage)) == null)
+      {
+        base.OnDragOver(e);
+        return;
+      }
+
+      TabPage dragTab = (TabPage) e.Data.GetData(typeof(TabPage));
+      int dragTabIndex = TabPages.IndexOf(dragTab);
+      if (dragTabIndex < 0)
+      {
+        // Tab page doesn't belong to this view. Ignore it.
+        e.Effect = DragDropEffects.None;
+        return;
+      }
+
+      // Hover over a tab?
+      int hoverIndex = TabIndexFromPosition(PointToClient(new Point(e.X, e.Y)));
+      if (hoverIndex < 0)
+      {
+        e.Effect = DragDropEffects.None;
+        return;
+      }
+
+      e.Effect = DragDropEffects.Move;
+
+      // Do this check after we set the drop effect so we don't get a block cursor
+      // when we already moved a page and still hover with the mouse over that.
+      if (dragTabIndex == hoverIndex)
+        return;
+
+      Rectangle dragTabRect = GetTabRect(dragTabIndex);
+      Rectangle hoverTabRect = GetTabRect(hoverIndex);
+
+      if (dragTabRect.Width < hoverTabRect.Width)
+      {
+        Point tcLocation = PointToScreen(Location);
+
+        if (dragTabIndex < hoverIndex)
+        {
+          if ((e.X - tcLocation.X) > ((hoverTabRect.X + hoverTabRect.Width) - dragTabRect.Width))
+            MovePage(dragTab, hoverIndex);
+        }
+        else
+          if (dragTabIndex > hoverIndex)
+          {
+            if ((e.X - tcLocation.X) < (hoverTabRect.X + dragTabRect.Width))
+              MovePage(dragTab, hoverIndex);
+          }
+      }
+      else
+        MovePage(dragTab, hoverIndex);
+    }
+
     protected override void OnEnter(EventArgs e)
     {
       base.OnEnter(e);
@@ -1213,6 +1373,17 @@ namespace MySQL.Controls
       Invalidate();
     }
 
+    protected override void OnSizeChanged(EventArgs e)
+    {
+      if (IsHandleCreated)
+      {
+        this.BeginInvoke((MethodInvoker) delegate
+        {
+          base.OnSizeChanged(e);
+        });
+      }
+    }
+
     void PageTextChanged(object sender, EventArgs e)
     {
       AdjustLayoutInfo(sender as TabPage);
@@ -1232,6 +1403,20 @@ namespace MySQL.Controls
         TabClosed(this, args);
     }
 
+    public event EventHandler<TabMovingEventArgs> TabMoving;
+    protected internal void OnTabMoving(TabMovingEventArgs args)
+    {
+      if (TabMoving != null)
+        TabMoving(this, args);
+    }
+
+    public event EventHandler<TabMovedEventArgs> TabMoved;
+    protected internal void OnTabMoved(TabMovedEventArgs args)
+    {
+      if (TabMoved != null)
+        TabMoved(this, args);
+    }
+
     #endregion
 
     #region Component Designer generated code
@@ -1345,7 +1530,19 @@ namespace MySQL.Controls
           }
 
           int buttonPart = 0;
-          if (IsCloseButtonVisible(currentIndex))
+          bool buttonSpaceNeeded = IsCloseButtonVisible(currentIndex) || layoutInfo[currentIndex].isBusy;
+          Size buttonSize;
+          if (layoutInfo[index].isBusy)
+          {
+            // Assuming here all busy indicator images are of the same size.
+            // Not a strong limitation actually, since all tabs are of same height too.
+            System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FlatTabControl));
+            Bitmap image = ((System.Drawing.Bitmap)(resources.GetObject("busy-indicator-blue")));
+            buttonSize = image.Size;
+          }
+          else
+            buttonSize = new Size(darkCloseButton.Width, darkCloseButton.Height);
+          if (buttonSpaceNeeded)
             buttonPart = buttonSpacing + buttonSize.Width;
           int tabWidth = itemPadding.Horizontal + textWidth + buttonPart;
           if ((info.page.ImageIndex >= 0) && (ImageList != null) &&
@@ -1354,7 +1551,7 @@ namespace MySQL.Controls
             tabWidth += ImageList.ImageSize.Width;
             
             // Add 4 px spacing if there is something after the icon.
-            if (IsCloseButtonVisible(currentIndex) || textWidth > 0)
+            if (buttonSpaceNeeded || textWidth > 0)
               tabWidth += 4;
           }
 
@@ -1365,7 +1562,7 @@ namespace MySQL.Controls
           info.tabArea = new Rectangle(offsetX, offsetY, tabWidth, ItemSize.Height);
           offsetX = info.tabArea.Right;
 
-          if (IsCloseButtonVisible(currentIndex))
+          if (buttonSpaceNeeded)
           {
             int buttonOffset = offsetY + (info.tabArea.Height - buttonSize.Height) / 2;
             info.buttonArea = new Rectangle(offsetX - itemPadding.Right - buttonSize.Width, buttonOffset,
@@ -1390,18 +1587,52 @@ namespace MySQL.Controls
       layoutInfo[index].closeButtonVisibility = visibility;
     }
 
+    /// <summary>
+    /// Mark individual tabs as busy causing them to show a busy indicator instead of the close button.
+    /// </summary>
+    public void SetBusy(int index, bool busy)
+    {
+      if (layoutInfo[index].isBusy != busy)
+      {
+        if (!busy)
+        {
+          layoutInfo[index].isBusy = false;
+          AdjustLayoutInfo(TabPages[index]);
+        }
+        else
+        {
+          AdjustLayoutInfo(TabPages[index]);
+          layoutInfo[index].isBusy = true;
+        }
+        Invalidate();
+      }
+    }
+
+    private void BusyAnimationStep(object sender, EventArgs args)
+    {
+      foreach (TabInfo info in layoutInfo)
+        if (info.isBusy)
+        {
+          Rectangle area = info.tabArea;
+          area.Offset(scrollOffset, 0);
+          Invalidate(area, false);
+        }
+    }
+
     /// <summary>
     /// Does the actual work to close a tab page, sending out appropriate events and updating
     /// internal structures. Returns true if the page was actually closed.
     /// </summary>
     public bool CloseTabPage(TabPage page)
     {
-      if (TabCount > 1 || canCloseLastTab)
+      if (TabCount > 1 || CanCloseLastTab)
       {
         TabClosingEventArgs args = new TabClosingEventArgs(page, true);
         OnTabClosing(args);
         if (args.canClose)
         {
+          SetBusy(TabPages.IndexOf(page), false);
+
           // Remove this tab and select its predecessor if possible.
           if (SelectedIndex > 0)
             SelectedIndex = SelectedIndex - 1;
@@ -1481,6 +1712,44 @@ namespace MySQL.Controls
       return result;
     }
 
+    /// <summary>
+    /// Moves the given page to the new index.
+    /// </summary>
+    private void MovePage(TabPage page, int newIndex)
+    {
+      int oldIndex = TabPages.IndexOf(page);
+
+      TabMovingEventArgs args = new TabMovingEventArgs(oldIndex, newIndex);
+      OnTabMoving(args);
+      if (args.Cancel)
+        return;
+
+      bool selectTab = (oldIndex == SelectedIndex);
+
+      // The simple approach here would be to remove the page and insert it at the new location.
+      // However this will produce heavy flickering, so we move the pages one by one instead.
+      if (oldIndex < newIndex)
+      {
+        for (int index = oldIndex; index < newIndex; index++)
+          TabPages[index] = TabPages[index + 1];
+        TabPages[newIndex] = page;
+        AdjustLayoutInfo(TabPages[oldIndex]);
+      }
+      else
+      {
+        for (int index = oldIndex; index > newIndex; index--)
+          TabPages[index] = TabPages[index - 1];
+        TabPages[newIndex] = page;
+        AdjustLayoutInfo(page);
+      }
+
+      // Keep the tab selected which was selected before.
+      if (selectTab)
+        SelectedIndex = newIndex;
+
+      Invalidate();
+    }
+
     // In order to simplify general handling of page content (which is very often just a container
     // with the actual content) we introduce here the concept of documents.
     // A document is the content of a tab. When using this concept you should only have one control on each
@@ -1539,9 +1808,13 @@ namespace MySQL.Controls
     /// </summary>
     public int IndexFromDocument(ITabDocument document)
     {
+      int i = 0;
       foreach (TabPage page in TabPages)
+      {
         if (DocumentFromHierarchy(page) == document)
-          return page.TabIndex;
+          return i;
+        i++;
+      }
 
       return -1;
     }
@@ -1745,11 +2018,7 @@ namespace MySQL.Controls
     }
 
     [Browsable(true)]
-    public bool CanCloseLastTab
-    {
-      get { return canCloseLastTab; }
-      set { canCloseLastTab = value; }
-    }
+    public bool CanCloseLastTab { get; set; }
 
     [Browsable(true)]
     public bool HideWhenEmpty
@@ -1786,9 +2055,11 @@ namespace MySQL.Controls
       }
     }
 
+    [Browsable(true)]
+    public bool CanReorderTabs { get; set; }
+
     // With the switch to .NET 4.0 client profile we have to separate runtime
     // and design time code.
-//    [Editor(typeof(FlatTabControlCollectionEditor), typeof(UITypeEditor))]
     [Editor("MySQL.Controls.FlatTabControlCollectionEditor", "UITypeEditor")]
     public new TabPageCollection TabPages
     {
@@ -1828,6 +2099,11 @@ namespace MySQL.Controls
       TopLevel = false;
     }
 
+    new virtual public void Show()
+    {
+      base.Show();
+    }
+
     new public void Activate()
     {
       Show();
@@ -1922,18 +2198,10 @@ namespace MySQL.Controls
           host.ToolTipText = value;
       }
     }
-
-    public int Index
-    {
-      get
-      {
-        if (host != null)
-          return host.TabIndex;
-        return -1;
-      }
-    }
   }
 
+  #region Event argument classes
+
   public class TabClosingEventArgs : EventArgs
   {
     public TabPage page;
@@ -1956,4 +2224,30 @@ namespace MySQL.Controls
     }
   }
 
+  public class TabMovedEventArgs : EventArgs
+  {
+    public int FromIndex, ToIndex;
+
+    public TabMovedEventArgs(int from, int to)
+    {
+      FromIndex = from;
+      ToIndex = to;
+    }
+  }
+
+  public class TabMovingEventArgs : EventArgs
+  {
+    public int FromIndex, ToIndex;
+    public bool Cancel;
+
+    public TabMovingEventArgs(int from, int to)
+    {
+      FromIndex = from;
+      ToIndex = to;
+      Cancel = false;
+    }
+  }
+
+  #endregion
+
 }
