@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,7 +26,6 @@ using System.Windows.Forms;
 using MySQL.Grt;
 using MySQL.Utilities.Properties;
 
-
 namespace MySQL.Controls
 {
   public class CustomDataGridViewTextBoxColumn : DataGridViewTextBoxColumn
@@ -35,41 +34,74 @@ namespace MySQL.Controls
     public AdditionalColumnWidthDelegate AdditionalColumnWidth;
     public override int GetPreferredWidth(DataGridViewAutoSizeColumnMode autoSizeColumnMode, bool fixedHeight)
     {
-      int prefferedWidth = base.GetPreferredWidth(autoSizeColumnMode, fixedHeight);
+      int preferredWidth = base.GetPreferredWidth(autoSizeColumnMode, fixedHeight);
       if (AdditionalColumnWidth != null)
-        prefferedWidth += AdditionalColumnWidth(this);
-      return prefferedWidth;
+        preferredWidth += AdditionalColumnWidth(this);
+      return preferredWidth;
+    }
+
+    private void InitializeComponent()
+    {
+
     }
   }
 
   public class GridView : DataGridView
   {
-    private GridModel model;
     private bool refreshing = false;
     private Bitmap fieldNullBitmap;
     private Bitmap fieldBlobBitmap;
     private bool wrapMode = false;
     private bool autoScroll = false;
     private bool allowAutoResizeColumns = true;
-    // defines the grid color
-    private Color gridColor = Color.FromArgb(240, 240, 240);
-    // defines the alternating row color
-    private Color gridAlternateRowColor = Color.FromArgb(248, 248, 248);
+
+    // Row main and alternate colors.
+    private Color gridColor = Color.FromArgb(255, 255, 255);
+    private Color gridAlternateRowColor = Color.FromArgb(237, 243, 253);
+    private bool busyAnimationActive = false;
+    private Bitmap mainBusyIcon = Resources.busy_indicator_white;
+    private Bitmap alternateBusyIcon = Resources.busy_indicator_lightblue;
 
     public CustomDataGridViewTextBoxColumn.AdditionalColumnWidthDelegate AdditionalColumnWidth;
 
+    // Message types (marked in the first column). Values taken from logging backend.
+    private enum MessageType
+    {
+      ErrorMsg,
+      WarningMsg,
+      NoteMsg,
+      OKMsg,
+      EditMsg,
+      BusyMsg
+    };
+
     protected override void Dispose(bool disposing)
     {
-      if (disposing && (model != null))
+      if (disposing && (Model != null))
       {
-        model.Dispose();
-        model = null;
+        Model.Dispose();
+        Model = null;
       }
+
+      if (busyAnimationActive)
+      {
+        ImageAnimator.StopAnimate(mainBusyIcon, new EventHandler(BusyAnimationStep));
+        ImageAnimator.StopAnimate(alternateBusyIcon, new EventHandler(BusyAnimationStep));
+      }
+
+      // Workaround for bug in DataGridView. When being in edit mode while the grid is disposed
+      // DataGridView tries to access a not existing edit control internally, which crashes.
+      CancelEdit();
+      if (Columns.Count > 0)
+        Columns.Clear();
+
       base.Dispose(disposing);
     }
 
     public GridView(GridModel model)
     {
+      ShowBusyAnimation = false;
+
       fieldNullBitmap = Resources.field_overlay_null;
       fieldBlobBitmap = Resources.field_overlay_blob;
 
@@ -92,29 +124,25 @@ namespace MySQL.Controls
 			GridColor = gridColor;
     }
 
-
-    public GridModel Model
-    {
-      set { model = value; }
-      get { return model; }
-    }
-
+    public GridModel Model { get; set; }
 
     public void ProcessModelChange()
     {
-      SuspendLayout();
+      if (Disposing || IsDisposed)
+        return;
+
       refreshing = true;
 
-      ReadOnly = model.is_readonly();
+      ReadOnly = Model.is_readonly();
 
-      CustomDataGridViewTextBoxColumn[] columns = new CustomDataGridViewTextBoxColumn[model.get_column_count()];
-      for (int i= 0; i < model.get_column_count(); i++)
+      CustomDataGridViewTextBoxColumn[] columns = new CustomDataGridViewTextBoxColumn[Model.get_column_count()];
+      for (int i= 0; i < Model.get_column_count(); i++)
       {
         CustomDataGridViewTextBoxColumn column= new CustomDataGridViewTextBoxColumn();
         column.AdditionalColumnWidth = AdditionalColumnWidth;
-        column.HeaderText = model.get_column_caption(i);
+        column.HeaderText = Model.get_column_caption(i);
         Type columnValueType;
-        switch (model.get_column_type(i))
+        switch (Model.get_column_type(i))
         {
           case GridModel.ColumnType.DatetimeType:
             columnValueType = typeof(DateTime);//! needs corresponding set_field or conversion to string. see PushValue
@@ -158,7 +186,7 @@ namespace MySQL.Controls
       {
         List<int> indexes;
         List<int> orders;
-        model.sort_columns(out indexes, out orders);
+        Model.sort_columns(out indexes, out orders);
         int l = 0;
         foreach (int n in indexes)
         {
@@ -170,46 +198,60 @@ namespace MySQL.Controls
 
       refreshing = false;
       ProcessModelRowsChange();
-      ResumeLayout();
     }
 
 
     public int ProcessModelRowsChange()
     {
-      SuspendLayout();
+      if (Disposing || IsDisposed)
+        return 0;
+
       refreshing = true;
 
-      int firstDisplayedScrollingRowIndex = AutoScroll ? -1 : 0;
-      if (AutoScroll && (RowCount > 0))
-      {
-        if (Rows[RowCount-1].Displayed)
-          firstDisplayedScrollingRowIndex = FirstDisplayedScrollingRowIndex;
-        else
-          firstDisplayedScrollingRowIndex = -1;
-      }
-
-      RowCount = model.count();
+      RowCount = Model.count();
       if (allowAutoResizeColumns)
         AutoResizeColumns(DataGridViewAutoSizeColumnsMode.DisplayedCells, true);
       if (wrapMode)
         AutoResizeRows(DataGridViewAutoSizeRowsMode.DisplayedCells, true);
 
-      if (0 != firstDisplayedScrollingRowIndex)
+      if (ShowBusyAnimation)
       {
-        if (FirstDisplayedScrollingRowIndex >= RowCount)
-          firstDisplayedScrollingRowIndex = -1;
-        if ((-1 == firstDisplayedScrollingRowIndex) && (RowCount > 0))
+        // Start or stop the busy indications depending on the type of messages we have.
+        bool hasBusyMessage = false;
+        for (int i = 0; i < Model.count(); i++)
         {
-          firstDisplayedScrollingRowIndex = RowCount - 1;
-          FirstDisplayedScrollingRowIndex = firstDisplayedScrollingRowIndex;
+          int type;
+          Model.get_field(new NodeId(i), 0, out type);
+          hasBusyMessage = ((MessageType)type == MessageType.BusyMsg);
+          if (hasBusyMessage)
+            break;
+        }
+
+        lock (this)
+        {
+          if (hasBusyMessage != busyAnimationActive)
+            if (hasBusyMessage)
+            {
+              busyAnimationActive = ImageAnimator.CanAnimate(mainBusyIcon);
+              ImageAnimator.Animate(mainBusyIcon, new EventHandler(BusyAnimationStep));
+              ImageAnimator.Animate(alternateBusyIcon, new EventHandler(BusyAnimationStep));
+            }
+            else
+            {
+              busyAnimationActive = false;
+              ImageAnimator.StopAnimate(mainBusyIcon, new EventHandler(BusyAnimationStep));
+              ImageAnimator.StopAnimate(alternateBusyIcon, new EventHandler(BusyAnimationStep));
+            }
         }
       }
 
-      ClearSelection();
+      if (AutoScroll && (RowCount > 0))
+        CurrentCell = this[0, Rows.Count - 1];
 
+      ClearSelection();
       Refresh();
       refreshing = false;
-      ResumeLayout();
+
       return 0;
     }
 
@@ -230,6 +272,7 @@ namespace MySQL.Controls
         VerticalScrollBar.Value = VerticalScrollBar.Maximum;
     }
 
+    #region Properties
 
     public bool AllowAutoResizeColumns
     {
@@ -254,13 +297,40 @@ namespace MySQL.Controls
       }
     }
 
-
     public bool AutoScroll
     {
       get { return autoScroll; }
       set { autoScroll = value; }
     }
 
+    /// <summary>
+    /// Determines if anything can be copied from the grid to the clipboard.
+    /// </summary>
+    public bool CanCopy
+    {
+      get { return SelectedCells.Count > 0; }
+    }
+
+    /// <summary>
+    /// Causes the grid to interpret a record as message. In that case the value in the first
+    /// column is taken as message type. If that value indicates a busy message (identified by the
+    /// MessageType enum) then a busy indicator is shown, instead of an icon.
+    /// </summary>
+    public bool ShowBusyAnimation { get; set; }
+
+    #endregion
+
+    #region Message handling
+
+    private void BusyAnimationStep(object sender, EventArgs args)
+    {
+      if (IsDisposed || Columns.Count == 0)
+        return;
+
+      Rectangle indicatorColumnRect = new Rectangle(0, 0, Columns[0].Width, ClientSize.Height);
+      Invalidate(indicatorColumnRect, false);
+    }
+
     private void OnRowPrePaint(Object sender, DataGridViewRowPrePaintEventArgs e)
     {
       AutoResizeRow(e.RowIndex);
@@ -271,30 +341,71 @@ namespace MySQL.Controls
     {
       base.OnCellPainting(e);
 
-      if ((0 > e.ColumnIndex) || (0 > e.RowIndex))
+      if (e.ColumnIndex < 0 || e.RowIndex < 0)
         return;
 
-      int iconId = model.get_field_icon(new NodeId(e.RowIndex), e.ColumnIndex, IconSize.Icon16);
-      Image icon = GrtIconManager.get_instance().get_icon(iconId);
-      if (null != icon)
+      NodeId nodeId = new NodeId(e.RowIndex);
+      Bitmap icon = null;
+      Rectangle rect = e.CellBounds;
+
+      if (e.ColumnIndex == 0)
       {
-        Rectangle rect = e.CellBounds;
-        rect.Height = icon.Height;
-        rect.Width = icon.Width;
-        switch (e.CellStyle.Alignment)
+        // Check if we have to draw a busy indicator.
+        int type;
+        Model.get_field(nodeId, 0, out type);
+        if ((MessageType) type == MessageType.BusyMsg)
         {
-          case DataGridViewContentAlignment.TopRight:
-          case DataGridViewContentAlignment.MiddleRight:
-          case DataGridViewContentAlignment.BottomRight:
-            rect.X += e.CellBounds.Width - rect.Width;
-            rect.X -= 1; // cell border width (required only for right alignment)
-            break;
-          case DataGridViewContentAlignment.TopCenter:
-          case DataGridViewContentAlignment.MiddleCenter:
-          case DataGridViewContentAlignment.BottomCenter:
-            rect.X += (e.CellBounds.Width - rect.Width) / 2;
-            break;
+          icon = (e.RowIndex % 2 == 1) ? Resources.busy_indicator_lightblue : Resources.busy_indicator_white;
+          ImageAnimator.UpdateFrames();
+          rect.X += (rect.Width - icon.Width) / 2;
+          rect.Y += (rect.Height - icon.Height) / 2;
+          rect.Size = icon.Size;
         }
+      }
+
+      if (icon == null)
+      {
+        int iconId = Model.get_field_icon(nodeId, e.ColumnIndex, IconSize.Icon16);
+        icon = GrtIconManager.get_instance().get_icon(iconId);
+        if (icon != null)
+        {
+          rect.Size = icon.Size;
+
+          // Horizontal alignment.
+          switch (e.CellStyle.Alignment)
+          {
+            case DataGridViewContentAlignment.TopRight:
+            case DataGridViewContentAlignment.MiddleRight:
+            case DataGridViewContentAlignment.BottomRight:
+              rect.X += e.CellBounds.Width - rect.Width;
+              rect.X -= 1; // cell border width (required only for right alignment)
+              break;
+            case DataGridViewContentAlignment.TopCenter:
+            case DataGridViewContentAlignment.MiddleCenter:
+            case DataGridViewContentAlignment.BottomCenter:
+              rect.X += (e.CellBounds.Width - rect.Width) / 2;
+              break;
+          }
+
+          // Vertical alignment.
+          switch (e.CellStyle.Alignment)
+          {
+            case DataGridViewContentAlignment.MiddleLeft:
+            case DataGridViewContentAlignment.MiddleCenter:
+            case DataGridViewContentAlignment.MiddleRight:
+              rect.Y += (e.CellBounds.Height - rect.Height) / 2;
+              break;
+            case DataGridViewContentAlignment.BottomLeft:
+            case DataGridViewContentAlignment.BottomCenter:
+            case DataGridViewContentAlignment.BottomRight:
+              rect.Y += e.CellBounds.Height - rect.Height;
+              break;
+          }
+        }
+      }
+
+      if (icon != null)
+      {
         e.PaintBackground(e.CellBounds, true);
         e.Graphics.DrawImageUnscaledAndClipped(icon, rect);
         e.Handled = true;
@@ -322,10 +433,10 @@ namespace MySQL.Controls
     {
       base.OnRowsRemoved(e);
 
-      if (refreshing || model == null)
+      if (refreshing || Model == null)
         return;
       for (int i = 0; i < e.RowCount; i++)
-        model.delete_node(new NodeId(e.RowIndex));
+        Model.delete_node(new NodeId(e.RowIndex));
     }
 
     protected override void OnNewRowNeeded(DataGridViewRowEventArgs e)
@@ -340,28 +451,32 @@ namespace MySQL.Controls
     {
       base.OnCancelRowEdit(e);
 
-      e.Response = (model.count() < RowCount);
+      if (Model != null)
+        e.Response = (Model.count() < RowCount);
     }
 
     protected override void OnCellBeginEdit(DataGridViewCellCancelEventArgs e)
     {
       base.OnCellBeginEdit(e);
 
-      switch (model.get_column_type(e.ColumnIndex))
+      switch (Model.get_column_type(e.ColumnIndex))
       {
         case GridModel.ColumnType.BlobType:
           e.Cancel = true;
           break;
       }
       if (!e.Cancel)
-        model.set_edited_field(e.RowIndex, e.ColumnIndex);
+        Model.set_edited_field(e.RowIndex, e.ColumnIndex);
     }
 
     protected override void OnCellEndEdit(DataGridViewCellEventArgs e)
     {
       base.OnCellEndEdit(e);
 
-      model.set_edited_field(-1, -1);
+      if (Model == null)
+        return;
+
+      Model.set_edited_field(-1, -1);
     }
 
     protected override void OnCellValuePushed(DataGridViewCellValueEventArgs e)
@@ -375,7 +490,7 @@ namespace MySQL.Controls
       {
         if (typeof(string) != Columns[e.ColumnIndex].ValueType)
         {
-          model.set_field_null(new NodeId(e.RowIndex), e.ColumnIndex);
+          Model.set_field_null(new NodeId(e.RowIndex), e.ColumnIndex);
         }
         else
           e.Value = "";
@@ -385,13 +500,13 @@ namespace MySQL.Controls
       {
         Type t = e.Value.GetType();
         if (typeof(string) == t)
-          model.set_field(new NodeId(e.RowIndex), e.ColumnIndex, (string)e.Value);
+          Model.set_field(new NodeId(e.RowIndex), e.ColumnIndex, (string)e.Value);
         else if (typeof(double) == t)
-          model.set_field(new NodeId(e.RowIndex), e.ColumnIndex, (double)e.Value);
+          Model.set_field(new NodeId(e.RowIndex), e.ColumnIndex, (double)e.Value);
         else if (typeof(int) == t)
-          model.set_field(new NodeId(e.RowIndex), e.ColumnIndex, (int)e.Value);
+          Model.set_field(new NodeId(e.RowIndex), e.ColumnIndex, (int)e.Value);
         else if (typeof(long) == t)
-          model.set_field(new NodeId(e.RowIndex), e.ColumnIndex, (long)e.Value);
+          Model.set_field(new NodeId(e.RowIndex), e.ColumnIndex, (long)e.Value);
       }
     }
 
@@ -399,9 +514,12 @@ namespace MySQL.Controls
     {
       base.OnCellValueNeeded(e);
 
+      if (Model == null)
+        return;
+
       String value;
-      model.get_field_repr(new NodeId(e.RowIndex), e.ColumnIndex, out value);
-      e.Value = value;
+      Model.get_field_repr(new NodeId(e.RowIndex), e.ColumnIndex, out value);
+      e.Value = value.Replace('\n', ' ');
     }
 
     protected override void OnCellToolTipTextNeeded(DataGridViewCellToolTipTextNeededEventArgs e)
@@ -412,7 +530,7 @@ namespace MySQL.Controls
         return;
 
       String value;
-      model.get_field_repr(new NodeId(e.RowIndex), e.ColumnIndex, out value);
+      Model.get_field_repr(new NodeId(e.RowIndex), e.ColumnIndex, out value);
 
       DataGridViewCell cell = Rows[e.RowIndex].Cells[e.ColumnIndex];
       if ((cell.Size.Width < cell.PreferredSize.Width)
@@ -480,6 +598,8 @@ namespace MySQL.Controls
       return base.ProcessCmdKey(ref msg, keyData);
     }
 
+    #endregion
+
     public List<NodeId> SelectedNodes()
     {
       List<DataGridViewRow> selectedRows = GetSelectedRows();
@@ -540,12 +660,5 @@ namespace MySQL.Controls
       Clipboard.SetDataObject(data);
     }
 
-    /// <summary>
-    /// Determines if anything can be copied from the grid to the clipboard.
-    /// </summary>
-    public bool CanCopy()
-    {
-      return SelectedCells.Count > 0;
-    }
   }
 }
