@@ -1,5 +1,5 @@
 ﻿/* 
- * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2011 Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,6 +34,8 @@ namespace MySQL.Utilities
     // the we need to split that into a singleton message popup and a base HUD window.
     static readonly HUDForm instance = new HUDForm();
 
+    public delegate bool CancelDelegate();
+
     private Size baseSize;
     private float borderSize = 3;
     private float shadowSize = 20;
@@ -42,7 +44,9 @@ namespace MySQL.Utilities
     private float cornerSize = 20;
     private int animationSteps = 4;
     private bool useAnimation = false;
+    private RectangleF cancelButtonBounds;
 
+    private CancelDelegate cancelDelegate = null;
     private Bitmap contentBitmap;
 
     static HUDForm()
@@ -55,17 +59,35 @@ namespace MySQL.Utilities
       InitializeComponent();
       baseSize = Size;
       SetStyle(ControlStyles.Selectable, false);
+      UpdateStyles();
+
+      cancelButtonPicture.UseWaitCursor = false;
+      cancelButtonPicture.Cursor = Cursors.AppStarting;
     }
 
-    public void Show(String title, String message, bool animated)
+    public static DialogResult ShowModal(String title, String message, bool animated, CancelDelegate deleg)
     {
-      label1.Text = title;
-      label2.Text = message;
+      instance.label1.Text = title;
+      instance.label2.Text = message;
+      instance.cancelDelegate = deleg;
 
-      Reshow(animated);
+      instance.Reshow(animated, true);
+
+      // Need to set Visible back to false to avoid an exception in ShowDialog about an already
+      // visible window. Doesn't usually have a visual effect.
+      instance.Visible = false;
+      return instance.ShowDialog();
     }
 
-    public void Reshow(bool animated)
+    public static void Show(String title, String message, bool animated)
+    {
+      instance.label1.Text = title;
+      instance.label2.Text = message;
+
+      instance.Reshow(animated, false);
+    }
+
+    public void Reshow(bool animated, bool modal)
     {
       Form mainForm = Application.OpenForms["MainForm"];
       if (mainForm == null)
@@ -76,7 +98,7 @@ namespace MySQL.Utilities
       // The (transparent) HUD will cover the entire application window.
       Bounds = mainForm.Bounds;
 
-      PrepareBitmap();
+      PrepareBitmap(modal);
 
       // Don't use animations in a terminal session (remote desktop).
       useAnimation = animated && !SystemInformation.TerminalServerSession;
@@ -94,15 +116,14 @@ namespace MySQL.Utilities
       }
     }
 
-    public new void Hide()
+    /// <summary>
+    /// Hides the HUD form and releases any used bitmap resource.
+    /// </summary>
+    private void DoHide()
     {
-      Form mainForm = Application.OpenForms["MainForm"];
-      if (mainForm != null)
-        mainForm.Update();
-
       if (useAnimation && contentBitmap != null)
       {
-        for (float i = animationSteps; i >= 0;  i--)
+        for (float i = animationSteps; i >= 0; i--)
           ControlUtilities.SetBitmap(this, contentBitmap, (int)(i / animationSteps * 255));
       }
       base.Hide();
@@ -112,8 +133,35 @@ namespace MySQL.Utilities
         contentBitmap.Dispose();
         contentBitmap = null;
       }
+      cancelDelegate = null;
     }
 
+    public static new void Hide()
+    {
+      Form mainForm = Application.OpenForms["MainForm"];
+      if (mainForm != null)
+      {
+        if (mainForm.InvokeRequired)
+          mainForm.Invoke((Action)(() => mainForm.Update()));
+        else
+          mainForm.Update();
+      }
+
+      if (instance.InvokeRequired)
+        instance.Invoke((Action)(() => instance.DoHide()));
+      else
+        instance.DoHide();
+    }
+
+    #region Properties
+
+    public static bool IsVisible
+    {
+      get { return instance.Visible; }
+    }
+
+    #endregion
+
     #region Drawing
 
     protected GraphicsPath GetPath()
@@ -137,7 +185,7 @@ namespace MySQL.Utilities
     /// Prepares the bitmap used to draw the window. Layered windows (like this one) use a bitmap for their
     /// content, including alpha channel.
     /// </summary>
-    protected void PrepareBitmap()
+    protected void PrepareBitmap(bool showCancelButton)
     {
       if (contentBitmap != null)
         contentBitmap.Dispose();
@@ -238,6 +286,16 @@ namespace MySQL.Utilities
 
         using (Brush textBrush = new SolidBrush(label2.ForeColor))
           g.DrawString(label2.Text, label2.Font, textBrush, textBounds, format);
+
+        if (showCancelButton)
+        {
+          Point buttonLocation = cancelButtonPicture.Location;
+          buttonLocation.Offset((int)innerBounds.Left, (int)innerBounds.Top);
+          cancelButtonBounds = new RectangleF(buttonLocation, cancelButtonPicture.Size);
+          g.DrawImageUnscaled(cancelButtonPicture.Image, buttonLocation);
+        }
+        else
+          cancelButtonBounds = new RectangleF();
       }
     }
 
@@ -276,9 +334,34 @@ namespace MySQL.Utilities
       switch ((WM) m.Msg)
       {
         case WM.LBUTTONDOWN:
-          Hide();
-          m.Result = IntPtr.Zero;
-          return;
+          {
+            int x = m.LParam.ToInt32() & 0xFFFF;
+            int y = m.LParam.ToInt32() >> 16;
+            if (cancelButtonBounds.Contains(x, y) && cancelDelegate != null && cancelDelegate())
+            {
+              Hide();
+              DialogResult = DialogResult.Cancel;
+            }
+            break;
+          }
+        case WM.SETCURSOR:
+          {
+            Point position = PointToClient(MousePosition);
+            if (cancelButtonBounds.Contains(position))
+            {
+              // Check before set. Windows XP will otherwise cause an endless recursion.
+              if (Cursor.Current != Cursors.AppStarting)
+                Cursor.Current = Cursors.AppStarting;
+            }
+            else
+            {
+              if (Cursor.Current != Cursors.WaitCursor)
+                Cursor.Current = Cursors.WaitCursor;
+            }
+
+            m.Result = new IntPtr(1);
+            return;
+          }
         case WM.MOUSEACTIVATE:
           m.Result = new IntPtr(Win32.MA_NOACTIVATE);
           return;
@@ -296,6 +379,12 @@ namespace MySQL.Utilities
       base.WndProc(ref m);
     }
 
+    public static void Finished()
+    {
+      Hide();
+      instance.DialogResult = DialogResult.OK;
+    }
+
     #endregion
 
   }
