@@ -328,8 +328,7 @@ namespace MySQL.GUI.Workbench
           // Returns the current bounds of the main window, encoded in a string (we only have one return type: string).
           // This can be used to center messages over the application window, regardless on which screen/monitor
           // the app windows currently is.
-          int[] values= { Left, Top, Width, Height };
-          result = String.Format("{0} {1} {2} {3} ", values);
+          result = String.Format("{0} {1} {2} {3} ", Left, Top, Width, Height);
           break;
       }
       return result;
