@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,6 @@
  * 02110-1301  USA
  */
 
-
 using System;
 using System.Collections.Generic;
 using System.Drawing;
@@ -25,12 +24,12 @@ using System.IO;
 using System.Windows.Forms;
 
 using MySQL.Controls;
+using MySQL.Forms;
 using MySQL.Grt;
 using MySQL.GUI.Mdc;
 using MySQL.GUI.Workbench.Plugins;
-using MySQL.Forms;
-using MySQL.Workbench;
 using MySQL.Utilities.SysUtils;
+using MySQL.Workbench;
 
 namespace MySQL.GUI.Workbench
 {
@@ -41,7 +40,7 @@ namespace MySQL.GUI.Workbench
     private WbContext wbContext;
     private WorkbenchDocumentClosing onWorkbenchDocumentClosing = null;
     private SortedDictionary<String, Cursor> cursors = new SortedDictionary<String, Cursor>();
-    private MySQL.Workbench.ModelViewForm formBE= null;
+    private MySQL.Workbench.ModelDiagramFormWrapper formBE = null;
 
     private ModelCatalogForm modelCatalogForm;
     private ModelLayerForm modelLayerForm;
@@ -59,24 +58,21 @@ namespace MySQL.GUI.Workbench
 
     #region Constructors
 
-    public ModelDiagramForm(WbContext WbContext, String id, IntPtr payload)
+    public ModelDiagramForm(WbContext context, String id, IntPtr payload)
     {
       InitializeComponent();
       
-      wbContext= WbContext;
+      wbContext = context;
 
-      if (wbContext.use_opengl())
-        InitializeGLCanvas(id, payload);
-      else
-        InitializeGDICanvas(id, payload);
+      CreateCanvas(id, payload);
 
-      openGlCanvasViewer.CanvasPanel.MouseMove += new MouseEventHandler(CanvasPanel_MouseMove);
-      openGlCanvasViewer.CanvasPanel.MouseDown += new MouseEventHandler(CanvasPanel_MouseDown);
-      openGlCanvasViewer.CanvasPanel.MouseUp += new MouseEventHandler(CanvasPanel_MouseUp);
-      openGlCanvasViewer.CanvasPanel.MouseDoubleClick += new MouseEventHandler(CanvasPanel_MouseDoubleClick);
-      openGlCanvasViewer.CanvasPanel.KeyDown += new KeyEventHandler(CanvasPanel_KeyDown);
-      openGlCanvasViewer.CanvasPanel.KeyUp += new KeyEventHandler(CanvasPanel_KeyUp);
-      openGlCanvasViewer.CanvasPanel.MouseLeave += new EventHandler(CanvasPanel_MouseLeave);
+      canvasViewer.CanvasPanel.MouseMove += new MouseEventHandler(CanvasPanel_MouseMove);
+      canvasViewer.CanvasPanel.MouseDown += new MouseEventHandler(CanvasPanel_MouseDown);
+      canvasViewer.CanvasPanel.MouseUp += new MouseEventHandler(CanvasPanel_MouseUp);
+      canvasViewer.CanvasPanel.MouseDoubleClick += new MouseEventHandler(CanvasPanel_MouseDoubleClick);
+      canvasViewer.CanvasPanel.KeyDown += new KeyEventHandler(CanvasPanel_KeyDown);
+      canvasViewer.CanvasPanel.KeyUp += new KeyEventHandler(CanvasPanel_KeyUp);
+      canvasViewer.CanvasPanel.MouseLeave += new EventHandler(CanvasPanel_MouseLeave);
 
       // Sidebar windows.
       modelNavigator = new ModelNavigatorForm(this);
@@ -105,6 +101,20 @@ namespace MySQL.GUI.Workbench
       FocusCanvasControl();
     }
 
+    private void Destroy()
+    {
+      formBE.Dispose();
+
+      // Explicitly close all editors to ensure plugins are unregistered from the backend.
+      foreach (ITabDocument content in bottomTabControl.Documents)
+        CloseTabDocument(content);
+
+      canvasViewer.FinalizeCanvas();
+
+      if (onWorkbenchDocumentClosing != null)
+        onWorkbenchDocumentClosing(this, new FormClosingEventArgs(CloseReason.FormOwnerClosing, false));
+    }
+
     #endregion
     
     #region IWorkbenchDocument Interface
@@ -115,12 +125,12 @@ namespace MySQL.GUI.Workbench
       set { onWorkbenchDocumentClosing = value; }
     }
     
-    public UIForm BackendClass
+    public UIForm BackendForm
     {
       get { return formBE; }
     }
 
-    public ModelViewForm View
+    public ModelDiagramFormWrapper DiagramWrapper
     {
         get { return formBE; }
     }
@@ -143,8 +153,9 @@ namespace MySQL.GUI.Workbench
             modelObjectDescriptionForm.UpdateForView(formBE);
           }
           break;
-        case RefreshType.RefreshSchema:
         case RefreshType.RefreshSchemaList:
+        case RefreshType.RefreshSchema:
+        case RefreshType.RefreshSchemaNoReload:
           modelCatalogForm.UpdateCatalogTree();
           break;
 
@@ -228,7 +239,7 @@ namespace MySQL.GUI.Workbench
 
     public BaseWindowsCanvasView Canvas
     {
-      get { return (BaseWindowsCanvasView)openGlCanvasViewer.Canvas; }
+      get { return (BaseWindowsCanvasView)canvasViewer.Canvas; }
     }
 
     public double Zoom
@@ -256,11 +267,11 @@ namespace MySQL.GUI.Workbench
       base.WndProc(ref m);
     }
 
-    private void openGlCanvasViewer_DragEnter(object sender, DragEventArgs e)
+    private void canvasViewer_DragEnter(object sender, DragEventArgs e)
     {
       bool dragObjectIsMyClass =
-                (e.Data.GetDataPresent(typeof(GrtValue)) == true) ||
-                (e.Data.GetDataPresent(typeof(List<GrtValue>)) == true);
+                (e.Data.GetDataPresent(typeof(GrtValue))) ||
+                (e.Data.GetDataPresent(typeof(List<GrtValue>)));
       if (dragObjectIsMyClass)
       {
         GrtValue val = (GrtValue)e.Data.GetData(typeof(GrtValue));
@@ -288,7 +299,7 @@ namespace MySQL.GUI.Workbench
       }
     }
     
-    private void openGlCanvasViewer_DragDrop(object sender, DragEventArgs e)
+    private void canvasViewer_DragDrop(object sender, DragEventArgs e)
     {
       GrtValue val = (GrtValue)e.Data.GetData(typeof(GrtValue));
       Point p = (sender as Control).PointToClient(new Point(e.X, e.Y));
@@ -312,18 +323,6 @@ namespace MySQL.GUI.Workbench
       }
     }
 
-    private void Destroy()
-    {
-      // Explicitly close all editors to ensure plugins are unregistered from the backend.
-      foreach (ITabDocument content in bottomTabControl.Documents)
-        CloseTabDocument(content);
-
-      openGlCanvasViewer.FinalizeCanvas();
-      
-      if (onWorkbenchDocumentClosing != null)
-        onWorkbenchDocumentClosing(this, new FormClosingEventArgs(CloseReason.FormOwnerClosing, false));
-    }
-
     void CanvasPanel_MouseUp(object sender, MouseEventArgs e)
     {
       Point p = e.Location;
@@ -368,7 +367,7 @@ namespace MySQL.GUI.Workbench
     {
       ITabDocument document = (sender as FlatTabControl).DocumentFromPage(e.page);
       if (document is IWorkbenchDocument)
-        e.canClose = (document as IWorkbenchDocument).BackendClass.can_close();
+        e.canClose = (document as IWorkbenchDocument).BackendForm.can_close();
       else
         if (document is MySQL.Forms.AppViewDockContent)
         {
@@ -392,11 +391,11 @@ namespace MySQL.GUI.Workbench
       {
         Plugins.DockablePlugin plugin = document as Plugins.DockablePlugin;
         wbContext.close_gui_plugin(plugin.GetFixedPtr());
-        plugin.ReleaseHandle();
+        plugin.Dispose();
       }
       else
         if (document is IWorkbenchDocument)
-          (document as IWorkbenchDocument).BackendClass.close();
+          (document as IWorkbenchDocument).BackendForm.close();
         else
           if (document is MySQL.Forms.AppViewDockContent)
           {
@@ -521,21 +520,11 @@ namespace MySQL.GUI.Workbench
       }
     }
 
-    public BaseWindowsCanvasView InitializeGLCanvas(String id, IntPtr payload)
+    public BaseWindowsCanvasView CreateCanvas(String id, IntPtr payload)
     {
-      BaseWindowsCanvasView canvas = openGlCanvasViewer.InitializeGLCanvas(this, payload, false);
-
+      BaseWindowsCanvasView canvas = canvasViewer.CreateCanvasView(this, payload, false,
+        wbContext.software_rendering_enforced(), wbContext.opengl_rendering_enforced());
       formBE = wbContext.get_diagram_form_for_diagram(id);
-
-      return canvas;
-    }
-
-    public BaseWindowsCanvasView InitializeGDICanvas(String id, IntPtr payload)
-    {
-      BaseWindowsCanvasView canvas = openGlCanvasViewer.InitializeGDICanvas(this, payload, false);
-
-      formBE = wbContext.get_diagram_form_for_diagram(id);
-
       return canvas;
     }
 
@@ -548,7 +537,7 @@ namespace MySQL.GUI.Workbench
     {
       formBE.update_options_toolbar();
       optionsToolStrip.Bounds = new Rectangle(toolsToolStrip.Width, diagramPanel.Padding.Top,
-        openGlCanvasViewer.ClientSize.Width - toolsToolStrip.Width, 25);
+        canvasViewer.ClientSize.Width - toolsToolStrip.Width, 25);
       optionsToolStrip.Visible = optionsToolStrip.Items.Count > 0;
       SetCursor(formBE.get_tool_cursor());
     }
@@ -558,13 +547,13 @@ namespace MySQL.GUI.Workbench
       // if no cursor is specified, use default arrow
       if (CursorFileName.Equals(""))
       {
-        openGlCanvasViewer.Cursor = System.Windows.Forms.Cursors.Default;
+        canvasViewer.Cursor = System.Windows.Forms.Cursors.Default;
         return;
       }
 
       // Check if cursor already cached
       if (cursors.ContainsKey(CursorFileName))
-        openGlCanvasViewer.Cursor = cursors[CursorFileName];
+        canvasViewer.Cursor = cursors[CursorFileName];
       else
       {
         // Load cursor
@@ -578,18 +567,18 @@ namespace MySQL.GUI.Workbench
           {
             // Add cursor to cache
             cursors.Add(CursorFileName, c);
-            openGlCanvasViewer.Cursor = c;
+            canvasViewer.Cursor = c;
           }
         }
 
         if (c == null)
-          openGlCanvasViewer.Cursor = System.Windows.Forms.Cursors.Default;
+          canvasViewer.Cursor = System.Windows.Forms.Cursors.Default;
       }
     }
 
     public void FocusCanvasControl()
     {
-      ActiveControl = openGlCanvasViewer.CanvasPanel;
+      ActiveControl = canvasViewer.CanvasPanel;
     }
 
     private void ShowModelNavigator()
@@ -690,4 +679,4 @@ namespace MySQL.GUI.Workbench
     #endregion
 
   }
-}
\ No newline at end of file
+}
