@@ -53,7 +53,7 @@ namespace MySQL.GUI.Workbench
       this.model = model;
       this.manager = manager;
 
-      layerTreeBE = model.View.get_layer_tree();
+      layerTreeBE = model.DiagramWrapper.get_layer_tree();
       layerTreeModel = new SimpleGrtTreeModel(layerTreeView, layerTreeBE, nodeStateIcon, false);
       layerTreeModel.AddColumn(nameNodeControl, 0, false);
       layerTreeModel.AddColumn(selectedNodeControl, 11, false);
