@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,17 +21,17 @@ using System;
 using System.Collections;
 using System.Collections.Generic;
 using System.Drawing;
+using System.IO;
 using System.Windows.Forms;
-using MySQL.Controls;
 
+using MySQL.Controls;
+using MySQL.Forms;
 using MySQL.Grt;
 using MySQL.GUI.Workbench.Plugins;
 using MySQL.GUI.Workbench.Properties;
 using MySQL.Utilities;
 using MySQL.Utilities.SysUtils;
-using MySQL.Forms;
 using MySQL.Workbench;
-using System.IO;
 
 namespace MySQL.GUI.Workbench
 {
@@ -185,7 +185,7 @@ namespace MySQL.GUI.Workbench
       set { ; }
     }
 
-    public UIForm BackendClass
+    public UIForm BackendForm
     {
       get { return wbOverview.get_uiform(); }
     }
@@ -289,7 +289,7 @@ namespace MySQL.GUI.Workbench
     {
       ITabDocument document = (sender as FlatTabControl).DocumentFromPage(e.page);
       if (document is IWorkbenchDocument)
-        e.canClose = (document as IWorkbenchDocument).BackendClass.can_close();
+        e.canClose = (document as IWorkbenchDocument).BackendForm.can_close();
       else
         if (document is MySQL.Forms.AppViewDockContent)
         {
@@ -308,13 +308,11 @@ namespace MySQL.GUI.Workbench
       {
         Plugins.DockablePlugin plugin = document as Plugins.DockablePlugin;
         wbContext.close_gui_plugin(plugin.GetFixedPtr());
-        plugin.ReleaseHandle();
-        plugin.Close();
         plugin.Dispose();
       }
       else
         if (document is IWorkbenchDocument)
-          (document as IWorkbenchDocument).BackendClass.close();
+          (document as IWorkbenchDocument).BackendForm.close();
         else
           if (document is MySQL.Forms.AppViewDockContent)
           {
@@ -1274,6 +1272,21 @@ namespace MySQL.GUI.Workbench
               list.SelectedItems[0].BeginEdit();
           }
         }
+        else if (name == "drop")
+        {
+          List <NodeId> node_ids = new List<NodeId>();
+
+          if (item.Owner.Tag is ListView)
+          {
+            ListView list = item.Owner.Tag as ListView;
+            foreach (ListViewItem list_item in list.SelectedItems)
+              node_ids.Add((list_item.Tag as Identifier).id);
+          }
+          else
+            node_ids.Add((item.Tag as Identifier).id);
+
+          wbOverview.activate_popup_item_for_nodes(name, node_ids);
+        }
         else
           wbOverview.activate_popup_item_for_nodes(name, new List<NodeId>() { (item.Tag as Identifier).id });
       }
@@ -1347,18 +1360,22 @@ namespace MySQL.GUI.Workbench
 
       if (listView != null && e.Button == MouseButtons.Right)
       {
-        NodeId itemNodeId = null;
         List<MySQL.Grt.MenuItem> items = new List<MySQL.Grt.MenuItem>();
+        List<NodeId> item_nodes = new List<NodeId>();
 
-        if (listView.SelectedItems.Count == 1)
-          itemNodeId = (listView.SelectedItems[0].Tag as Identifier).id;
-        else
-          if (listView.SelectedItems.Count == 0)
-            itemNodeId = (listView.Tag as Identifier).id; // nodeid for the group represented by the listview
-
-        if (itemNodeId != null)
+        if (listView.SelectedItems.Count == 0)
         {
-          items = wbOverview.get_popup_items_for_nodes(new List<NodeId>() { itemNodeId });
+          item_nodes.Add((listView.Tag as Identifier).id); // nodeid for the group represented by the listview
+        }
+        else
+        {
+          foreach (ListViewItem item in listView.SelectedItems)
+            item_nodes.Add((item.Tag as Identifier).id);
+        }
+
+        if (item_nodes.Count != 0)
+        {
+          items = wbOverview.get_popup_items_for_nodes(item_nodes);
           if (items.Count > 0)
           {
             System.Windows.Forms.ContextMenuStrip menu;
@@ -1366,7 +1383,7 @@ namespace MySQL.GUI.Workbench
             menu = workbenchMenuManager.ShowContextMenu(listView, items, e.X, e.Y, new EventHandler(PopupActionHandler));
             menu.Tag = listView;
 
-            SetMenuItemsTag(menu.Items, CreateIdentifier(itemNodeId));
+            SetMenuItemsTag(menu.Items, CreateIdentifier(item_nodes[0]));
           }
         }
       }
