@@ -18,19 +18,20 @@
  */
 
 using System;
+using System.IO;
 using System.Reflection;
 using System.Threading;
 using System.Windows.Forms;
-using System.IO;
+using System.Collections;
+using System.Collections.Generic;
+
 using Microsoft.Win32;
 
+using MySQL.Forms;
 using MySQL.Grt;
 using MySQL.Utilities;
 using MySQL.Utilities.SysUtils;
 using MySQL.Workbench;
-using MySQL.Forms;
-
-using SingleInstanceApplication;
 
 namespace MySQL.GUI.Workbench
 {
@@ -69,99 +70,7 @@ namespace MySQL.GUI.Workbench
     [STAThread]
     static void Main(string[] Args)
     {
-      Application.EnableVisualStyles();
-      Application.SetCompatibleTextRenderingDefault(false);
-
-      // Hook into the exception handling to establish our own handling.
-      AppDomain currentDomain = AppDomain.CurrentDomain; // CLR
-      currentDomain.UnhandledException += new UnhandledExceptionEventHandler(OnUnhandledException);
-
-      Application.ThreadException += // Windows Forms
-         new System.Threading.ThreadExceptionEventHandler(OnGuiUnhandledException);
-
-      // Read some early values which cannot be stored in the preferences (since they are loaded
-      // later) from registry.
-      bool singleInstance = false;
-      string lastVersion = "";
-      string currentVersion = GetApplicationMetaInfo(ApplicationMetaInfo.Version);
-      RegistryKey wbKey = Registry.CurrentUser;
-      try
-      {
-        wbKey = wbKey.OpenSubKey(@"Software\Oracle\MySQL Workbench", false);
-        if (wbKey != null)
-        {
-          singleInstance = wbKey.GetValue("SingleInstance", 0).ToString() == "1";
-          lastVersion = wbKey.GetValue("LastStartedAs", "").ToString();
-        }
-        else
-          Registry.CurrentUser.CreateSubKey(@"Software\Oracle\MySQL Workbench");
-      }
-      catch (Exception)
-      {
-        // Ignore errors.
-      }
-      finally
-      {
-        if (wbKey != null)
-          wbKey.Close();
-      }
-
-      // First check if this is the first instance of Workbench (if enabled).
-      // The setting for single-instance is stored in the registry as it is Windows-only
-      // and loading of the application settings happens later.
-      if (singleInstance)
-      {
-        if (!ApplicationInstanceManager.CreateSingleInstance(
-          Assembly.GetExecutingAssembly().GetName().Name, Args, SingleInstanceCallback))
-          return;
-      }
-
-      // Give the main thread a proper name, so we can later check for it when needed.
-      Thread.CurrentThread.Name = "mainthread";
-
-      // Get the base directory of the executable and change the working dir to it
-      // TODO: the working dir should be untouched and the app should work
-      // from any basedir
-      System.Reflection.Assembly asm = System.Reflection.Assembly.GetEntryAssembly();
-      string baseDir = System.IO.Path.GetDirectoryName(asm.Location);
-      string workdir = System.IO.Directory.GetCurrentDirectory();
-      System.IO.Directory.SetCurrentDirectory(baseDir);
-
-      // Next check if this is the first start of a new version of WB. In this case remove all
-      // compiled python files. They will be automatically recreated and can produce problems
-      // under certain circumstances.
-      if (currentVersion != lastVersion)
-        RemoveCompiledPythonFiles(baseDir);
-
-      // Some people don't have c:\windows\system32 in PATH, so we need to set it here
-      // for WBA to find the needed commands
-      String systemdir = Environment.GetFolderPath(Environment.SpecialFolder.System);
-      String syspath = Environment.GetEnvironmentVariable("PATH");
-
-      String []paths= syspath.Split(new char[]{';'});
-      syspath = "";
-      // strip all python related dirs from PATH to avoid conflicts with other Python installations
-      foreach (String path in paths)
-      {
-        if (!path.ToLower().Contains("python"))
-          syspath = syspath+";"+path;
-      }
-      Environment.SetEnvironmentVariable("PATH", systemdir + syspath);
-
-      // Clear PYTHONPATH environment variable, as we do not need it but our python impl
-      // seriously gets confused with it.
-      Environment.SetEnvironmentVariable("PYTHONPATH", workdir + "\\python\\Lib;" + workdir + "\\python\\DLLs;" + workdir + "\\python");
-      Environment.SetEnvironmentVariable("PYTHONHOME", workdir + "\\python");
-
-      // Initialize forms stuff.
-      MySQL.Forms.Manager formsManager = new MySQL.Forms.Manager();
-
-      #region Command line argument parsing and handling
-
-      // Connect application to console to have proper output there if requested.
-      Win32.RedirectConsole();
-
-      // Command line parsing
+      // Start with command line parsing.
       Arguments CommandLine = new Arguments();
 
       CommandLine.AddOptionName("-verbose", true);
@@ -182,22 +91,144 @@ namespace MySQL.GUI.Workbench
       CommandLine.AddOptionName("-run-python", false);
       CommandLine.AddOptionName("-run-lua", false);
       CommandLine.AddOptionName("-quit-when-done", true);
+      CommandLine.AddOptionName("-log-level", false);
 
       CommandLine.Parse(Args);
 
       // Command line handling
-      if (ProcessCommandLineArguments(CommandLine))
+      bool exitApplication = ProcessCommandLineArguments(CommandLine);
+
+      // Initialize the logger. Enabled log levels are by default error, warning and info
+      // (+ debug1 and 2 in debug builds).
+      string userDir = System.IO.Path.Combine(System.IO.Path.Combine(
+        Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),
+        "MySQL"), "Workbench");
+      Logger.InitLogger(userDir);
+#if DEBUG
+      string logLevel = "debug2";
+#else
+      string logLevel = "info";
+#endif
+      string environmentLevel = Environment.GetEnvironmentVariable("WB_LOG_LEVEL");
+      if (environmentLevel != null)
+        logLevel = environmentLevel;
+      if (CommandLine["log-level"] != null)
+        logLevel = CommandLine["log-level"];
+
+      Logger.ActiveLevel = logLevel;
+      PrintInitialLogInfo();
+
+      if (exitApplication)
       {
-        Win32.ReleaseConsole();
+        Logger.LogInfo("Workbench", "Command line params told us to shut down.\n");
         return;
       }
 
-      #endregion
+      Application.EnableVisualStyles();
+      Application.SetCompatibleTextRenderingDefault(false);
+
+      // Hook into the exception handling to establish our own handling.
+      AppDomain currentDomain = AppDomain.CurrentDomain; // CLR
+      currentDomain.UnhandledException += new UnhandledExceptionEventHandler(OnUnhandledException);
+
+      Application.ThreadException += // Windows Forms
+         new System.Threading.ThreadExceptionEventHandler(OnGuiUnhandledException);
+
+      // Read some early values which cannot be stored in the preferences (since they are loaded
+      // later) from registry.
+      bool singleInstance = false;
+      string lastVersion = "";
+      string currentVersion = GetApplicationMetaInfo(ApplicationMetaInfo.Version);
+      Logger.LogInfo("Workbench", "Current version given by meta info is: " + currentVersion + '\n');
+      RegistryKey wbKey = Registry.CurrentUser;
+      try
+      {
+        wbKey = wbKey.OpenSubKey(@"Software\Oracle\MySQL Workbench", false);
+        if (wbKey != null)
+        {
+          singleInstance = wbKey.GetValue("SingleInstance", 0).ToString() == "1";
+          lastVersion = wbKey.GetValue("LastStartedAs", "").ToString();
+        }
+        else
+          Registry.CurrentUser.CreateSubKey(@"Software\Oracle\MySQL Workbench");
+      }
+      catch (Exception e)
+      {
+        Logger.LogError("Workbench", "Error while checking single instance reg key: " + e.Message + '\n');
+      }
+      finally
+      {
+        if (wbKey != null)
+          wbKey.Close();
+      }
+
+      // First check if this is the first instance of Workbench (if enabled).
+      // The setting for single-instance is stored in the registry as it is Windows-only
+      // and loading of the application settings happens later.
+      if (singleInstance)
+      {
+        if (!ApplicationInstanceManager.CreateSingleInstance(
+          Assembly.GetExecutingAssembly().GetName().Name, Args, SingleInstanceCallback))
+        {
+          Logger.LogInfo("Workbench", "Exiting as another instance of WB is already running.\n");
+          return;
+        }
+      }
+
+      // Give the main thread a proper name, so we can later check for it when needed.
+      Thread.CurrentThread.Name = "mainthread";
+
+      // Get the base directory of the executable and change the working dir to it
+      // TODO: the working dir should be untouched and the app should work
+      // from any basedir
+      System.Reflection.Assembly asm = System.Reflection.Assembly.GetEntryAssembly();
+      string baseDir = System.IO.Path.GetDirectoryName(asm.Location);
+      string workdir = System.IO.Directory.GetCurrentDirectory();
+      System.IO.Directory.SetCurrentDirectory(baseDir);
+
+      // Next check if this is the first start of a new version of WB. In this case remove all
+      // compiled python files. They will be automatically recreated and can produce problems
+      // under certain circumstances.
+      if (currentVersion != lastVersion)
+      {
+        Logger.LogInfo("Workbench", "This is the first start of a new version. Doing some clean up.\n");
+        List<string> failed = new List<string>();
+        RemoveCompiledPythonFiles(baseDir, failed);
+
+        // TODO: decide if we wanna ask the user to remove those files manually or just ignore them.
+      }
+
+      // Some people don't have c:\windows\system32 in PATH, so we need to set it here
+      // for WBA to find the needed commands
+      String systemFolder = Environment.GetFolderPath(Environment.SpecialFolder.System);
+      String cleanedPath = Environment.GetEnvironmentVariable("PATH");
+
+      String []paths= cleanedPath.Split(new char[]{';'});
+      cleanedPath = "";
+      // Strip all python related dirs from PATH to avoid conflicts with other Python installations.
+      foreach (String path in paths)
+      {
+        if (!path.ToLower().Contains("python"))
+          cleanedPath = cleanedPath+";"+path;
+      }
+      Environment.SetEnvironmentVariable("PATH", systemFolder + cleanedPath);
+      Logger.LogInfo("Workbench", "Setting PATH to: " + systemFolder + cleanedPath + '\n');
+
+      // Clear PYTHONPATH environment variable, as we do not need it but our python impl
+      // seriously gets confused with it.
+      Environment.SetEnvironmentVariable("PYTHONPATH", workdir + "\\python\\Lib;" + workdir + "\\python\\DLLs;" + workdir + "\\python");
+      Environment.SetEnvironmentVariable("PYTHONHOME", workdir + "\\python");
+
+      // Initialize forms stuff.
+      MySQL.Forms.Manager formsManager = new MySQL.Forms.Manager();
+
+      // Connect application to console to have proper output there if requested.
+      Win32.RedirectConsole();
 
       #region Runtime path check
 
       // Currently WB has trouble running from a path containing non-ASCII characters.
-      // Actually, our third party libraries have (namely lua, python, ctemplate, yassl etc.), 
+      // Actually, our third party libraries have (namely lua, python, ctemplate), 
       // as they don't consider Unicode file names (encoded as UTF-8) which leads to file-not-found
       // errors. Refuse to work in such a path for now.
       foreach (Char c in baseDir)
@@ -222,6 +253,7 @@ namespace MySQL.GUI.Workbench
 
         if (DateTime.Now.Subtract(fileDate).TotalDays > 45)
         {
+          Logger.LogInfo("Workbench", "Found an old WB pre release. Showing warning.\n");
           if (MessageBox.Show("This version of MySQL Workbench is older than 45 days and most probably outdated. "
             + Environment.NewLine
             + "It is recommended to upgrade to a newer version if available. "
@@ -297,9 +329,6 @@ namespace MySQL.GUI.Workbench
           new AppImpl.StringAppCommandAppViewStringDelegate(mainForm.ApplicationCommand));
 
 
-        /// mainForm.PrintOutputText
-        /// mainForm.ShowOutputForm
-
         string openAtStartupType = "";
         string openAtStartup = null;
 
@@ -376,17 +405,17 @@ namespace MySQL.GUI.Workbench
           "",       // struct
           baseDir + "/modules", // modules
           baseDir, // library
-          System.IO.Path.Combine(System.IO.Path.Combine(
-              Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),
-              "MySQL"), "Workbench"),
+          userDir,
           openAtStartup,
           openAtStartupType,
           runScript, runScriptLanguage,
           (CommandLine["swrendering"] != null),
           (CommandLine["openglrendering"] != null),
-          (CommandLine["log"] != null),
           (CommandLine["quit-when-done"] != null || CommandLine["quitwhendone"] != null));
 
+        // TODO: check return value and show error message.
+        // Currently the return value is always true. In case of an error an exception is raised.
+        // That should change.
         wbContext.init(formsApplication, wbOptions,
           new WbContext.VoidStrUIFormDelegate(mainForm.CreateMainFormView)
         );
@@ -404,8 +433,10 @@ namespace MySQL.GUI.Workbench
         grtManager.perform_idle_tasks();
         
         // Setup Menus
+        Logger.LogInfo("Workbench", "Setting up main UI\n");
         mainForm.SetupUI();
         mainForm.Show();
+        Logger.LogInfo("Workbench", "UI is up\n");
 
         // Tell the backend our main UI is ready. This will also load a model if it was given via command line
         // and opens the overview form for it.
@@ -419,9 +450,9 @@ namespace MySQL.GUI.Workbench
           if (wbKey != null)
             wbKey.SetValue("LastStartedAs", currentVersion);
         }
-        catch (Exception)
+        catch (Exception e)
         {
-          // Ignore errors.
+          Logger.LogError("Workbench", "Couldn't write regkey LastStartedAs: " + e.Message + '\n');
         }
         finally
         {
@@ -434,6 +465,7 @@ namespace MySQL.GUI.Workbench
         {
           try
           {
+            Logger.LogInfo("Workbench", "Running the application\n");
             Application.Run(new MySQLApplicationContext(mainForm));
           }
           catch (Exception e)
@@ -441,7 +473,9 @@ namespace MySQL.GUI.Workbench
             HandleException(e);
           }
         }
-        
+
+        Logger.LogInfo("Workbench", "Shutting down Workbench\n");
+
         // shutdown wb context
         if (wbContext != null)
         {
@@ -449,17 +483,40 @@ namespace MySQL.GUI.Workbench
             wbContext.flush_idle_tasks();
 
           wbContext.finalize();
+          wbContext.Dispose();
         }
         formsApplication.Dispose();
+        GC.Collect();
       }
       
       #endregion
 
       Win32.ReleaseConsole();
+
+      Logger.LogInfo("Workbench", "Done\n");
     }
 
-    //----------------------------------------------------------------------------------------------
-
+    /// <summary>
+    /// Prints some general info to the log file.
+    /// </summary>
+    private static void PrintInitialLogInfo()
+    {
+      Logger.LogInfo("Workbench", "Starting up Workbench\n");
+      Logger.LogInfo("Workbench", string.Format("Current environment:\n\tCommand line: {0}\n\tCurrentDirectory: {1}\n" + 
+        "\tHasShutdownStarted: {2}\n\tOSVersion: {3}\n\tSystemDirectory: {4}\n" +
+        "\tTickCount: {5}\n\tUserInteractive: {6}\n\tVersion: {7}\n\tWorkingSet: {8}\n",
+        Environment.CommandLine, Environment.CurrentDirectory, Environment.HasShutdownStarted,
+        Environment.OSVersion.ToString(), Environment.SystemDirectory,
+        Environment.TickCount, Environment.UserInteractive,
+        Environment.Version.ToString(), Environment.WorkingSet));
+      
+      IDictionary environmentVariables = Environment.GetEnvironmentVariables();
+      string variables = "";
+      foreach (DictionaryEntry entry in environmentVariables)
+        variables += string.Format("\t{0} = {1}\n", entry.Key, entry.Value);
+      Logger.LogInfo("Workbench", "Environment variables:\n" + variables);
+    }
+    
     #region Application handlers
 
     static void timer_Tick(object sender, EventArgs e)
@@ -674,7 +731,6 @@ namespace MySQL.GUI.Workbench
         CommandLine.AddOptionName("-run-lua", false);
         CommandLine.AddOptionName("-quit-when-done", true);
 
-
         Console.WriteLine("Options" + Environment.NewLine
           + "  -admin instance .... Open an admin tab to the named server instance at startup" + Environment.NewLine
           + "  -open filename ..... Open the given filename at startup" + Environment.NewLine
@@ -686,32 +742,30 @@ namespace MySQL.GUI.Workbench
           + "  -quit-when-done .... Quits Workbench once the given script finishes executing"  + Environment.NewLine
           + "  -swrendering ....... Force the canvas to use software rendering instead of OpenGL" + Environment.NewLine
           + "  -nologo ............ Do not display the splash screen" + Environment.NewLine
-          + "  -log ............... Instruction to save messages (other debug info) to file" + Environment.NewLine
+          + "  -log-level ......... Determines the amount of logging (error, warning, info, debug1/2/3)" + Environment.NewLine
           + "  -verbose (-v) ...... Print verbose output in the GRT Shell" + Environment.NewLine
           + "  -version ........... Print the version information" + Environment.NewLine
           + "  -grtversion ........ Print the GRT version information" + Environment.NewLine
           + "  -help (-h) ......... Print this output" + Environment.NewLine
         );
 
-        /*Console.WriteLine("GRT Options" + Environment.NewLine
-          + "{0}",
-          Grt.option_description);*/
-
-        return true;
-      }
-      else if (CommandLine["version"] != null)
-      {
-        Console.WriteLine("MySQL Workbench {0}.{1} {2}",
-          GetApplicationMetaInfo(ApplicationMetaInfo.Version), 
-          GetApplicationMetaInfo(ApplicationMetaInfo.Revision),
-          GetApplicationMetaInfo(ApplicationMetaInfo.Configuration));
-        return true;
-      }
-      else if (CommandLine["grtversion"] != null)
-      {
-        Console.WriteLine("GRT Environment {0}", GRT.version());
         return true;
       }
+      else
+        if (CommandLine["version"] != null)
+        {
+          Console.WriteLine("MySQL Workbench {0}.{1} {2}",
+            GetApplicationMetaInfo(ApplicationMetaInfo.Version), 
+            GetApplicationMetaInfo(ApplicationMetaInfo.Revision),
+            GetApplicationMetaInfo(ApplicationMetaInfo.Configuration));
+          return true;
+        }
+        else
+          if (CommandLine["grtversion"] != null)
+          {
+            Console.WriteLine("GRT Environment {0}", GRT.version());
+            return true;
+          }
 
       return false;
     }
@@ -756,6 +810,8 @@ namespace MySQL.GUI.Workbench
         info += "FullText = " + o.ToString();
       }
 
+      Logger.LogError("Workbench", message + "\n" + info + '\n');
+
       // Check for blocked files (Windows "security" feature).
       if (info.Contains("0x80131515"))
       {
@@ -770,19 +826,26 @@ namespace MySQL.GUI.Workbench
         Win32Api.UnblockWorkbenchFiles(System.IO.Directory.GetCurrentDirectory());
       }
       else
-        ExceptionDialog.Show(message, info);
+        ExceptionDialog.Show(message, info, wbContext);
     }
 
     /// <summary>
     /// Removes all pyc files from the given dir and its sub dirs.
     /// </summary>
-    private static void RemoveCompiledPythonFiles(string folder)
+    private static void RemoveCompiledPythonFiles(string folder, List<string> failed)
     {
       foreach (string file in Directory.GetFiles(folder, "*.pyc"))
-        File.Delete(file);
+        try
+        {
+          File.Delete(file);
+        }
+        catch (Exception)
+        {
+          failed.Add(file);
+        }
 
       foreach (string subfolder in Directory.GetDirectories(folder))
-        RemoveCompiledPythonFiles(subfolder);
+        RemoveCompiledPythonFiles(subfolder, failed);
     }
 
     #endregion
