@@ -1091,16 +1091,51 @@ namespace ScintillaNet.Configuration
 					ind.Style = ic.Style.Value;
 			}
 
-			foreach(Configuration c in configList)
-            {
-            	foreach (KeyWordConfig kwc in c.Lexing_Keywords)
-				{
-					if (kwc.Inherit.HasValue && kwc.Inherit.Value)
-						Scintilla.Lexing.Keywords[kwc.List] += kwc.Value;
-					else
-						Scintilla.Lexing.Keywords[kwc.List] = kwc.Value;
-				}
-            }
+      // 2010-09-23 ml: set lexer before setting its properties.
+      s = null;
+      foreach (Configuration c in configList)
+      {
+        if (c.Lexing_Language != null)
+          s = c.Lexing_Language;
+      }
+
+      if (s == null)
+      {
+        //	None of the configs specified a lexer. First let's see if
+        //	we have a Language-Lexer map defined:
+        if (Scintilla.Lexing.LexerLanguageMap.ContainsKey(_language))
+        {
+          s = Scintilla.Lexing.LexerLanguageMap[_language];
+        }
+        else
+        {
+          try
+          {
+            Enum.Parse(typeof(Lexer), _language, true);
+
+            //	If we made it here, the language matches one of
+            //	the lexer names, just use that.
+            s = _language;
+          }
+          catch (ArgumentException)
+          {
+            //	No match, oh well. Don't set the lexer.
+          }
+        }
+      }
+
+      Scintilla.Lexing.LexerName = s;
+
+      foreach (Configuration c in configList)
+      {
+        foreach (KeyWordConfig kwc in c.Lexing_Keywords)
+        {
+          if (kwc.Inherit.HasValue && kwc.Inherit.Value)
+            Scintilla.Lexing.Keywords[kwc.List] += kwc.Value;
+          else
+            Scintilla.Lexing.Keywords[kwc.List] = kwc.Value;
+          }
+        }
 
 			//	Hrm... unfortunately there's no way to clear
 			//	Scintilla's Lexing Properties. Guess we'll just
@@ -1109,6 +1144,9 @@ namespace ScintillaNet.Configuration
 			//	means that the "Inherit" attribute is really
 			//	meaningless. Nevertheless I'm leaving it in
 			//	just in case it ever becomes useful.
+			//	
+      // 2010-09-23 ml: This is no longer true. As soon as a new lexer is set (as we do above)
+      //                a new internal lexer state is created which starts with an empty prop set.
 			foreach (Configuration c in configList)
 			{
 				foreach (KeyValuePair<string,string> item in c.Lexing_Properties)
@@ -1163,39 +1201,6 @@ namespace ScintillaNet.Configuration
 			if (s != null)
 				Scintilla.Lexing.StreamCommentSufix = s;
 
-			s = null;
-			foreach (Configuration c in configList)
-			{
-				if (c.Lexing_Language != null)
-					s = c.Lexing_Language;
-			}
-			
-			if (s == null)
-			{
-				//	None of the configs specified a lexer. First let's see if
-				//	we have a Language-Lexer map defined:
-				if (Scintilla.Lexing.LexerLanguageMap.ContainsKey(_language))
-				{
-					s = Scintilla.Lexing.LexerLanguageMap[_language];
-				}
-				else
-				{
-					try
-					{
-						Enum.Parse(typeof(Lexer), _language, true);
-
-						//	If we made it here, the language matches one of
-						//	the lexer names, just use that.
-						s = _language;
-					}
-					catch (ArgumentException) 
-					{ 
-						//	No match, oh well. Don't set the lexer.
-					}
-				}
-			}
-			Scintilla.Lexing.LexerName = s;
-
 			LineCache? lc = null;
 			foreach (Configuration c in configList)
 			{
@@ -1733,7 +1738,7 @@ namespace ScintillaNet.Configuration
 				if (c.Styles.Bits.HasValue)
 					i = c.Styles.Bits;
 			}
-      /* ml: See Bits property for why this is outcommented.
+      /* ml: style bits are obsolete, always use the default (7).
 			if (i.HasValue)
 				Scintilla.Styles.Bits = i.Value;
       */
