@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,7 @@
  * 02110-1301  USA
  */
 
-#include <mforms/mforms.h>
+#import "MFMForms.h"
 #include "base/string_utilities.h"
 
 #import "MFCodeEditor.h"
@@ -147,7 +147,7 @@ void read_style_attribute(NSXMLElement* node, NSString* name, NSMutableDictionar
 /**
  * Loads the configuration file for the given language and fills the given style map with the loaded
  * style data. Every other setting (tab usage, keywords etc.) will be applied
- * to the editor on the way. Styles need a bit more processing so the caller will take for that.
+ * to the editor on the way. Styles need a bit more processing so the caller will take care for that.
  */
 void load_language_settings(ScintillaView* scintilla, NSString* language, NSMutableDictionary* style_map)
 {
@@ -384,6 +384,44 @@ void setup_marker(NSBundle* bundle, ScintillaView* scintilla, int marker_id, con
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Either hides the gutter entirely or shows it with certain width settings.
+ */
+void setup_gutter(ScintillaView* scintilla, bool show)
+{
+  if (show)
+  {
+    // Margin: Line number style.
+    [scintilla setColorProperty: SCI_STYLESETFORE parameter: STYLE_LINENUMBER fromHTML: @"#404040"];
+    [scintilla setColorProperty: SCI_STYLESETBACK parameter: STYLE_LINENUMBER fromHTML: @"#E0E0E0"];
+    
+    [scintilla setGeneralProperty: SCI_SETMARGINTYPEN parameter: 0 value: SC_MARGIN_NUMBER];
+    int lineNumberStyleWidth = [scintilla getGeneralProperty: SCI_TEXTWIDTH parameter: STYLE_LINENUMBER extra: (int)"_99999"];
+    [scintilla setGeneralProperty: SCI_SETMARGINWIDTHN parameter: 0 value: lineNumberStyleWidth];
+    [scintilla setGeneralProperty: SCI_SETMARGINSENSITIVEN parameter: 0 value: 1]; // Make Scintilla send a notification for clicks in this margin.
+    
+    // Margin: Markers.
+    [scintilla setGeneralProperty: SCI_SETMARGINWIDTHN parameter: 1 value: 16];
+    [scintilla setGeneralProperty: SCI_SETMARGINSENSITIVEN parameter: 1 value: 1];
+    
+    // Margin: Indicators (folders).
+    [scintilla setColorProperty: SCI_SETFOLDMARGINCOLOUR
+                      parameter: 1
+                      value: [NSColor colorWithDeviceWhite: 230.0 / 255 alpha: 1]];
+    [scintilla setGeneralProperty: SCI_SETMARGINWIDTHN parameter: 2 value: 16];
+    [scintilla setGeneralProperty: SCI_SETMARGINSENSITIVEN parameter: 2 value: 1];
+
+  }
+  else
+  {
+    [scintilla setGeneralProperty: SCI_SETMARGINWIDTHN parameter: 0 value: 0];
+    [scintilla setGeneralProperty: SCI_SETMARGINWIDTHN parameter: 1 value: 0];
+    [scintilla setGeneralProperty: SCI_SETMARGINWIDTHN parameter: 2 value: 0];
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Helper to set up the code editor with usually used settings.
  */
@@ -399,32 +437,15 @@ void setup_editor(ScintillaView* scintilla, bool use_tabs, int indentation, NSSt
   
   [scintilla setGeneralProperty: SCI_SETCARETLINEVISIBLE value: 1];
   [scintilla setColorProperty: SCI_SETCARETLINEBACK
-                    value: [NSColor colorWithDeviceRed: 248.0 / 255 green: 200.0 / 255 blue: 0 alpha: 1]];
+                    parameter: 0
+                        value: [NSColor colorWithDeviceRed: 248.0 / 255 green: 200.0 / 255 blue: 0 alpha: 1]];
   [scintilla setGeneralProperty: SCI_SETCARETLINEBACKALPHA value: 20];
 
   [scintilla setGeneralProperty: SCI_SETBACKSPACEUNINDENTS value: 1];
-  int guide_type = (language == @"python") ? SC_IV_LOOKFORWARD : SC_IV_LOOKBOTH;
-  [scintilla setGeneralProperty: SCI_SETINDENTATIONGUIDES value: guide_type];
+//  int guide_type = [language isEqualTo: @"python"] ? SC_IV_LOOKFORWARD : SC_IV_LOOKBOTH;
+//  [scintilla setGeneralProperty: SCI_SETINDENTATIONGUIDES value: guide_type];
 
-  // Margin: Line number style.
-  [scintilla setColorProperty: SCI_STYLESETFORE parameter: STYLE_LINENUMBER fromHTML: @"#404040"];
-  [scintilla setColorProperty: SCI_STYLESETBACK parameter: STYLE_LINENUMBER fromHTML: @"#E0E0E0"];
-  
-  [scintilla setGeneralProperty: SCI_SETMARGINTYPEN parameter: 0 value: SC_MARGIN_NUMBER];
-  int lineNumberStyleWidth = [scintilla getGeneralProperty: SCI_TEXTWIDTH parameter: STYLE_LINENUMBER extra: (int)"_99999"];
-	[scintilla setGeneralProperty: SCI_SETMARGINWIDTHN parameter: 0 value: lineNumberStyleWidth];
-  [scintilla setGeneralProperty: SCI_SETMARGINSENSITIVEN parameter: 0 value: 1]; // Make Scintilla send a notification for clicks in this margin.
-  
-  // Margin: Markers.
-  [scintilla setGeneralProperty: SCI_SETMARGINWIDTHN parameter: 1 value: 16];
-  [scintilla setGeneralProperty: SCI_SETMARGINSENSITIVEN parameter: 1 value: 1];
-  
-  // Margin: Indicators (folders).
-  [scintilla setColorProperty: SCI_SETFOLDMARGINCOLOUR
-                    parameter: 1
-                    value: [NSColor colorWithDeviceWhite: 230.0 / 255 alpha: 1]];
-  [scintilla setGeneralProperty: SCI_SETMARGINWIDTHN parameter: 2 value: 16];
-  [scintilla setGeneralProperty: SCI_SETMARGINSENSITIVEN parameter: 2 value: 1];
+  setup_gutter(scintilla, true);
   
   [scintilla setGeneralProperty: SCI_SETEOLMODE value: SC_EOL_LF];
 
@@ -755,6 +776,19 @@ static void ce_set_font(CodeEditor* self, const std::string& fontDescription)
   }
 }
   
+//--------------------------------------------------------------------------------------------------
+
+static void ce_show_gutter(CodeEditor* self, bool flag)
+{
+  ScintillaView *editor = self->get_data();
+  if (editor != nil)
+  {
+    setup_gutter(editor, flag);
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void cf_codeeditor_init()
 {
   ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();
@@ -771,6 +805,7 @@ void cf_codeeditor_init()
   f->_code_editor_impl.remove_markup = ce_remove_markup;
   f->_code_editor_impl.line_count = ce_line_count;
   f->_code_editor_impl.set_font = ce_set_font;
+  f->_code_editor_impl.show_gutter = ce_show_gutter;
 }
 
 @end
