@@ -48,9 +48,12 @@
 
 //--------------------------------------------------------------------------------------------------
 
-- (void)setPadding:(int)padding
+- (void)setPaddingLeft:(float)lpad right:(float)rpad top:(float)tpad bottom:(float)bpad
 {
-  mPadding= padding;
+  mLeftPadding= lpad;
+  mRightPadding= rpad;
+  mTopPadding= tpad;
+  mBottomPadding= bpad;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -144,12 +147,12 @@
   }
   else if (mDefaultBackColor != nil)
   {
-    NSView *parent = [self superview];
+    id parent = [self superview];
     // check if we're inside a NSBox or NSTabView, which draw their own background
     while (parent)
     {
       if ([parent isKindOfClass: [NSBox class]] || ([parent isKindOfClass: objc_getClass("MFTabViewImpl")] && 
-                                                    [(NSTabView*)parent tabViewType] != NSNoTabsNoBorder))
+                                                    [[parent tabView] tabViewType] != NSNoTabsNoBorder))
         break;
       parent = [parent superview];
     }
