@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -231,6 +231,11 @@
   return size;
 }
 
+- (NSSize)minimumSize
+{
+  return [self preferredSize];
+}
+
 static bool drawbox_create(::mforms::DrawBox *self)
 {
   [[[MFDrawBoxImpl alloc] initWithObject:self] autorelease];
