@@ -50,6 +50,8 @@ extern void cf_menu_init();
 extern void cf_codeeditor_init();
 extern void cf_menubar_init();
 extern void cf_toolbar_init();
+extern void cf_hypertext_init();
+extern void cf_popover_init();
 
 extern "C" {
 void mforms_cocoa_init()
@@ -85,6 +87,8 @@ void mforms_cocoa_init()
   cf_util_init();
   cf_menubar_init();
   cf_toolbar_init();
+  cf_hypertext_init();
+  cf_popover_init();
 }
   
   
