@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -98,7 +98,8 @@
 {
   id content= [[self subviews] lastObject];
   NSSize size= [self frame].size;
-  size.width= NSWidth([self frame]) - 2*mBasePadding - (mLeftPadding + mRightPadding);
+  size.width -= 2*mBasePadding + (mLeftPadding + mRightPadding);
+  size.height -= 2*mBasePadding + (mTopPadding + mBottomPadding);
   // size.height= [content minimumSize].height;
   [content setFrame: NSMakeRect(mLeftPadding+mBasePadding, mTopPadding+mBasePadding, size.width, size.height)];
 }
@@ -265,6 +266,16 @@
 }
 
 
+- (void)resizeSubviewsWithOldSize:(NSSize)oldBoundsSize
+{
+  NSRect bounds = [self bounds];
+  bounds.size.width -= mTopLeftOffset.x + mBottomRightOffset.x;
+  bounds.size.height -= mTopLeftOffset.y + mBottomRightOffset.y;
+//  bounds.origin.x = mTopLeftOffset.x;
+//  bounds.origin.y = mTopLeftOffset.y;
+  [[self contentView] setFrame: bounds];
+}
+
 - (mforms::Object*)mformsObject
 {
   return mOwner;
@@ -324,7 +335,10 @@
 
 - (void)setContentView:(NSView*)content
 {
-  [[self contentView] addSubview: content];
+  if (content)
+    [[self contentView] addSubview: content];
+  else
+    [[[[self contentView] subviews] lastObject] removeFromSuperview];
   [self subviewMinimumSizeChanged];
 }
 
