@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,10 +17,12 @@
  * 02110-1301  USA
  */
 
+
+
 #import "MFScrollPanel.h"
 
 #import "MFView.h"
-#include "mforms/mforms.h"
+#import "MFMForms.h"
 
 @interface MFClipView : NSClipView
 {
