@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +17,8 @@
  * 02110-1301  USA
  */
 
-
 #import "MFTextEntry.h"
-#include "mforms/mforms.h"
+#import "MFMForms.h"
 
 @implementation LimitedTextFieldFormatter
 
@@ -86,10 +85,15 @@
       [self setCell: [secureField cell]];
       [secureField release];
     }
-    else if (type == ::mforms::SearchEntry)
+    else if (type == ::mforms::SearchEntry || type == ::mforms::SmallSearchEntry)
     {
       NSSearchField *searchField= [[NSSearchField alloc] initWithFrame: NSMakeRect(0, 0, 30, 0)];
       [self setCell: [searchField cell]];
+      if (type == ::mforms::SmallSearchEntry)
+      {
+        [[self cell] setControlSize: NSSmallControlSize];
+        [self setFont: [NSFont systemFontOfSize: [NSFont systemFontSizeForControlSize: NSSmallControlSize]]];
+      }
       [searchField release];
     }
     mOwner= aEntry;
@@ -127,18 +131,23 @@
   return size;
 }
 
-
 - (BOOL)heightIsFixed
 {
   return YES;
 }
 
-
-- (void)setFrame:(NSRect)frame
+- (void)setFixedFrameSize:(NSSize)size
 {
-  [super setFrame:frame];
+  mFixedSize = size;
+  [super setFixedFrameSize: size];
 }
 
+- (NSSize)fixedFrameSize
+{
+  return mFixedSize;
+}
+
+
 /*
 - (void)setFrame:(NSRect)frame
 {
