@@ -23,10 +23,12 @@
 
 @interface MFTreeViewImpl : NSScrollView {
   NSOutlineView *mOutline;
+  NSMutableDictionary *mIconCache;
   id mContents;
   mforms::TreeView *mOwner;
   int mSortColumn;
   int mFreezeCount;
+  BOOL mReorderable;
   BOOL mSortColumnEnabled;
 }
 
