@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,7 +19,7 @@
 
 
 #import "MFView.h"
-#import <mforms/mforms.h>
+#import "MFMForms.h"
 #include "base/string_utilities.h"
 
 #import "MFContainerBase.h" // to get forw declaration of setFreezeRelayout:
@@ -83,17 +83,22 @@ enum ViewFlags
   return NSMakeSize(0, 0);
 }
 
+- (NSSize)fixedFrameSize
+{
+  return [self frame].size;
+}
 
 - (NSSize)preferredSize
 {
   NSSize size= [self minimumSize];
+  NSSize fsize = [self fixedFrameSize];
 
   // If a fixed size is set honour that but don't go below the
   // minimal required size.
   if ([self widthIsFixed])
-    size.width= MAX(size.width, NSWidth([self frame]));
+    size.width= MAX(size.width, fsize.width);
   if ([self heightIsFixed])
-    size.height= MAX(size.height, NSHeight([self frame]));
+    size.height= MAX(size.height, fsize.height);
   return size;
 }
 
@@ -103,9 +108,9 @@ enum ViewFlags
   NSSize size= [self minimumSizeForWidth: [self widthIsFixed] ? NSWidth([self frame]) : width];
   
   if ([self heightIsFixed])
-    size.height= NSHeight([self frame]);
+    size.height= [self fixedFrameSize].height;
   if ([self widthIsFixed])
-    size.width= NSWidth([self frame]);
+    size.width= [self fixedFrameSize].width;
   
   return size;  
 }
@@ -131,6 +136,16 @@ enum ViewFlags
   }
 }
 
+
+- (void)drawBounds:(NSRect)rect
+{
+  NSFrameRect(rect);
+  [NSBezierPath strokeLineFromPoint:NSMakePoint(NSMinX(rect), NSMinY(rect))
+                            toPoint:NSMakePoint(NSMaxX(rect), NSMaxY(rect))];
+  [NSBezierPath strokeLineFromPoint:NSMakePoint(NSMinX(rect), NSMaxY(rect))
+                            toPoint:NSMakePoint(NSMaxX(rect), NSMinY(rect))];
+}
+
 @end
 
 
@@ -194,7 +209,7 @@ static void view_set_position(::mforms::View *self, int x, int y)
   }
 }
 
-static void view_client_to_screen(::mforms::View *self, MySQL::Geometry::Point& point)
+static void view_client_to_screen(::mforms::View *self, int& x, int& y)
 {
   id view = self->get_data();
   if (view)
@@ -202,10 +217,10 @@ static void view_client_to_screen(::mforms::View *self, MySQL::Geometry::Point&
     NSPoint pointInWindowCoordinates;
     NSPoint pointInScreenCoords;
     
-    pointInWindowCoordinates = [view convertPoint: NSMakePoint(point.x, point.y) toView: nil];
+    pointInWindowCoordinates = [view convertPoint: NSMakePoint(x, y) toView: nil];
     pointInScreenCoords = [[view window] convertBaseToScreen: pointInWindowCoordinates];
-    point.x = pointInScreenCoords.x;
-    point.y = pointInScreenCoords.y;
+    x = pointInScreenCoords.x;
+    y = pointInScreenCoords.y;
   }
 }
 
@@ -388,9 +403,10 @@ static void view_set_back_image(::mforms::View *self, const std::string &path, m
 static void view_flush_events(::mforms::View *)
 {}
 
-static void view_set_padding(::mforms::View *self, const ::MySQL::Geometry::Padding& padding)
+static void view_set_padding(::mforms::View *self, int left, int top, int right, int bottom)
 {
-  // dummy
+  if ([self->get_data() respondsToSelector: @selector(setPaddingLeft:right:top:bottom:)])
+    [self->get_data() setPaddingLeft:left right:right top:top bottom:bottom];
 }
 
 void cf_view_init()
