@@ -149,14 +149,20 @@ static BOOL modalHUDRunning = NO;
 {
   if ((self= [super init]) != nil)
   {
-    if (![NSBundle loadNibNamed: @"Components" owner: self])
-      NSLog(@"Error loading Components.xib");
+    if (![NSBundle loadNibNamed: @"HUDPanel" owner: self])
+      NSLog(@"Error loading HUDPanel.xib");
 
     [hudPanel setBecomesKeyOnlyIfNeeded: YES];
   }
   return self;
 }
 
+- (void) dealloc
+{
+  [hudPanel release];
+  [super dealloc];
+}
+
 //--------------------------------------------------------------------------------------------------
 
 - (NSPanel*) hud
