@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,7 @@
 
 #include "lf_mforms.h"
 #include "lf_view.h"
+#include "base/file_utilities.h"
 
 #define RESPONSE_OK 1
 #define RESPONSE_CANCEL 0
@@ -37,6 +38,9 @@ class FileChooserImpl : public ViewImpl
   Gtk::FileChooserDialog *_dlg;
   Gtk::Widget *get_outer() const { return _dlg; }
 
+  Gtk::Table *_options_table;
+  std::map<std::string, Gtk::ComboBoxText*> _combos;
+ 
   static std::vector<std::string> split_string(const std::string &s, const std::string &sep)
   {
     std::vector<std::string> parts;
@@ -61,9 +65,9 @@ class FileChooserImpl : public ViewImpl
   
   }
   
-  static bool create(::mforms::FileChooser *self, ::mforms::FileChooserType type, const bool show_hidden)
+  static bool create(::mforms::FileChooser *self, ::mforms::Form *owner, ::mforms::FileChooserType type, const bool show_hidden)
   {
-    return new FileChooserImpl(self, type, show_hidden) != 0;
+    return new FileChooserImpl(self, owner, type, show_hidden) != 0;
   }
 
   static void set_title(::mforms::FileChooser *self, const std::string &title)
@@ -108,10 +112,69 @@ class FileChooserImpl : public ViewImpl
     return dlg ? dlg->_dlg->get_filename().raw() : "";
   }
 
+  static void set_path(FileChooser *self, const std::string &path)
+  {
+    FileChooserImpl *dlg = self->get_data<FileChooserImpl>();
+    dlg->_dlg->set_filename(path);
+
+    std::string ext = base::extension(path);
+    Gtk::ComboBoxText *combo = dlg->_combos["format"];
+    if (combo)
+    {
+      std::vector<std::string> &extensions(self->_selector_options["format"]);
+      std::vector<std::string>::const_iterator it = std::find(extensions.begin(), extensions.end(), ext.substr(1));
+      if (it != extensions.end())
+        combo->set_active(it - extensions.begin());
+    }
+  }
+
+  static void add_selector_option(FileChooser *self, const std::string &name, const std::string &label, const std::vector<std::pair<std::string,std::string> > &values)
+  {
+    FileChooserImpl *dlg = self->get_data<FileChooserImpl>();
+    int row;
+    if (!dlg->_options_table)
+    {
+      dlg->_options_table = Gtk::manage(new Gtk::Table(1, 2));
+      dlg->_options_table->set_col_spacings(4);
+      dlg->_dlg->set_extra_widget(*dlg->_options_table);
+      row = 0;
+    }
+    else
+    {
+      row = dlg->_options_table->property_n_rows().get_value();
+      dlg->_options_table->property_n_rows().set_value(row + 1);
+    }
+    Gtk::ComboBoxText *combo = 0; 
+    if (!(combo = dlg->_combos[name]))
+    {
+      combo = dlg->_combos[name] = Gtk::manage(new Gtk::ComboBoxText());
+      dlg->_options_table->attach(*Gtk::manage(new Gtk::Label(label)), 0, 1, row, row+1, Gtk::FILL, Gtk::FILL);
+      dlg->_options_table->attach(*dlg->_combos[name], 1, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL);
+      dlg->_options_table->show_all();
+    }
+    //combo->remove_all();
+    combo->clear_items();
+    for (std::vector<std::pair<std::string,std::string> >::const_iterator iter = values.begin(); iter != values.end(); ++iter)
+      combo->append_text(iter->first);
+    combo->set_active(0);
+  }
+
+  static std::string get_selector_option_value(FileChooser *self, const std::string &name)
+  {
+    FileChooserImpl *dlg = self->get_data<FileChooserImpl>();
+    if (dlg->_combos[name])
+    {
+      int i = dlg->_combos[name]->get_active_row_number();
+      if (i >= 0)
+        return self->_selector_options[name][i];
+    }
+    return "";
+  }
+
   static void set_extensions(FileChooser *self, const std::string &extensions, const std::string &default_extension)
   {
     FileChooserImpl *dlg = self->get_data<FileChooserImpl>();
-    if ( dlg )
+    if (dlg)
     {      
       // extensions format:
       // AAA Files (*.aaa)|*.aaa,BBB Files (*.bbb)
@@ -136,8 +199,8 @@ class FileChooserImpl : public ViewImpl
     }
   }
 
-  FileChooserImpl(::mforms::FileChooser *form, ::mforms::FileChooserType type, bool show_hidden)
-    : ViewImpl(form)
+  FileChooserImpl(::mforms::FileChooser *form, ::mforms::Form *owner, ::mforms::FileChooserType type, bool show_hidden)
+    : ViewImpl(form), _options_table(0)
   {
     // TODO: enable showing hidden files/folders.
     //FileChooserDialog *dialog = 0;
@@ -160,6 +223,12 @@ class FileChooserImpl : public ViewImpl
       _dlg->add_button(Gtk::Stock::OPEN, RESPONSE_OK);
       break;      
     }
+    if (owner)
+    {
+      FormImpl *fi = owner->get_data<FormImpl>();
+      if (fi && fi->get_window())
+        _dlg->set_transient_for(*fi->get_window());
+    }
   }
 
   virtual ~FileChooserImpl()
@@ -179,6 +248,9 @@ class FileChooserImpl : public ViewImpl
     f->_filechooser_impl.set_directory = &FileChooserImpl::set_directory;
     f->_filechooser_impl.get_directory = &FileChooserImpl::get_directory;
     f->_filechooser_impl.get_path = &FileChooserImpl::get_path;
+    f->_filechooser_impl.set_path = &FileChooserImpl::set_path;
+    f->_filechooser_impl.add_selector_option = &FileChooserImpl::add_selector_option;
+    f->_filechooser_impl.get_selector_option_value = &FileChooserImpl::get_selector_option_value;
   }
 };
 
