@@ -17,13 +17,14 @@
  * 02110-1301  USA
  */
 
+#include "../lf_mforms.h"
 #include "../lf_toolbar.h"
 #include "../lf_view.h"
 #include <gtkmm/box.h>
 #include "image_cache.h"
 #include "base/wb_iterators.h"
 #include "base/log.h"
-ENABLE_LOG("mforms.linux")
+DEFAULT_LOG_DOMAIN("mforms.linux")
 
 class ColorComboColumns : public Gtk::TreeModel::ColumnRecord
 {
@@ -71,6 +72,7 @@ static Gtk::Orientation toolbar_orientation_from_type(const mforms::ToolBarType
 //==============================================================================
 //
 //==============================================================================
+
 struct ToolBarImpl : public mforms::gtk::ViewImpl
 {
   mutable Gtk::Box           *_toolbar;
@@ -93,6 +95,9 @@ struct ToolBarImpl : public mforms::gtk::ViewImpl
   {
     delete _toolbar;
   }
+
+ protected:
+  virtual void set_padding_impl(int left, int top, int right, int bottom);
 };
 
 //------------------------------------------------------------------------------
@@ -132,8 +137,6 @@ void mforms::gtk::ToolBarImpl::insert_item(mforms::ToolBar *toolbar, int index,
   if (!w)
     return;
 
-  const ToolBarItemType type = (ToolBarItemType)((long)(w->get_data("type")));
-
   if (item && item->get_type() == SeparatorItem)
   {
     Gtk::Alignment *align = dynamic_cast<Gtk::Alignment*>(w);
@@ -159,11 +162,11 @@ void mforms::gtk::ToolBarImpl::insert_item(mforms::ToolBar *toolbar, int index,
     index = -1;
   if (impl && w)
   {
-    bool expand = false;
+    bool expand = item->get_expandable();
     bool fill   = false;
-    if (type == mforms::ExpanderItem)
+    if (item->get_type() == mforms::ExpanderItem)
       expand = fill = true;
-    
+
     impl->_toolbar->pack_start(*Gtk::manage(w), expand, fill);
     impl->_toolbar->reorder_child(*w, index);
 
@@ -200,9 +203,10 @@ bool mforms::gtk::ToolBarImpl::create_tool_item(mforms::ToolBarItem *item, ToolB
 
   switch ( type )
   {
+    case mforms::TextActionItem:
     case mforms::ActionItem:
     {
-      Gtk::Button *btn = new Gtk::Button();
+      Gtk::Button *btn = Gtk::manage(new Gtk::Button());
       btn->set_focus_on_click(false);
       btn->set_border_width(0);
       btn->set_relief(Gtk::RELIEF_NONE);
@@ -213,7 +217,7 @@ bool mforms::gtk::ToolBarImpl::create_tool_item(mforms::ToolBarItem *item, ToolB
     }
     case mforms::ToggleItem:
     {
-      Gtk::ToggleButton *btn = new Gtk::ToggleButton();
+      Gtk::ToggleButton *btn = Gtk::manage(new Gtk::ToggleButton());
       btn->set_focus_on_click(false);
       btn->set_relief(Gtk::RELIEF_NONE);
       btn->signal_toggled().connect(sigc::bind(sigc::ptr_fun(process_ctrl_action), btn, item));
@@ -224,7 +228,7 @@ bool mforms::gtk::ToolBarImpl::create_tool_item(mforms::ToolBarItem *item, ToolB
     }
     case mforms::SeparatorItem:
     {
-      Gtk::Alignment *align = new Gtk::Alignment(0.5, 0.5);
+      Gtk::Alignment *align = Gtk::manage(new Gtk::Alignment(0.5, 0.5));
       align->set_padding(2, 1, 2, 1);
       w = align;
       break;
@@ -275,12 +279,25 @@ bool mforms::gtk::ToolBarImpl::create_tool_item(mforms::ToolBarItem *item, ToolB
     case mforms::ExpanderItem:
     case mforms::LabelItem:
     {
-      Gtk::Label *label = new Gtk::Label("", 0.0, 0.5);
+      Gtk::Label *label = Gtk::manage(new Gtk::Label("", 0.0, 0.5));
       w = label;
       break;
     }
     case mforms::ImageBoxItem:
     {
+      Gtk::Image *image = Gtk::manage(new Gtk::Image());
+      w = image;
+      break;
+    }
+    case mforms::SegmentedToggleItem:
+    {
+      Gtk::ToggleButton *btn = Gtk::manage(new Gtk::ToggleButton());
+      btn->set_focus_on_click(false);
+      btn->set_relief(Gtk::RELIEF_NONE);
+      btn->signal_toggled().connect(sigc::bind(sigc::ptr_fun(process_ctrl_action), btn, item));
+      btn->set_inconsistent(false);
+
+      w = btn;
       break;
     }
 
@@ -289,10 +306,9 @@ bool mforms::gtk::ToolBarImpl::create_tool_item(mforms::ToolBarItem *item, ToolB
   if (w)
   {
     w->show();
-    w->set_data("type", (void*)type);
   }
   else
-    log_error("create_tool_item, widget is 0 for passed type %i", type);
+    log_error("create_tool_item, widget is 0 for passed type %i\n", type);
 
   item->set_data(w);
 
@@ -336,6 +352,7 @@ static void swap_icons(Gtk::ToggleButton* btn)
   else
     img = cast<Gtk::Image*>(btn->get_data("icon"));
 
+  img->show();
   btn->set_image(*img);
 }
 
@@ -375,7 +392,7 @@ void mforms::gtk::ToolBarImpl::set_item_text(mforms::ToolBarItem *item, const st
       Gtk::Label* lbl = cast<Gtk::Label*>(item->get_data_ptr());
       if (lbl)
       {
-        lbl->set_text(label);
+        lbl->set_markup("<small>"+label+"</small>");
         lbl->set_name(label);
       }
       break;
@@ -526,6 +543,9 @@ void mforms::gtk::ToolBarImpl::set_selector_items(ToolBarItem* item, const std::
       for (int i = 0; i < size; ++i)
         w->append_text(values[i]);
 
+      if (w->get_active_row_number() < 0)
+        w->set_active_text(values[0]);
+
       w->set_data("ignore_signal", 0);
     }
   }
@@ -556,11 +576,20 @@ void mforms::gtk::ToolBarImpl::set_selector_items(ToolBarItem* item, const std::
 
       w->set_model(model);
 
+      if (w->get_active_row_number() < 0)
+        w->set_active(0);
+
       w->set_data("ignore_signal", 0);
     }
   }
 }
 
+//------------------------------------------------------------------------------
+void ::ToolBarImpl::set_padding_impl(int left, int top, int right, int bottom)
+{
+  _toolbar->set_border_width(left);
+}
+
 //------------------------------------------------------------------------------
 void mforms::gtk::lf_toolbar_init()
 {
