@@ -18,7 +18,6 @@
  */
 
 #include "stdafx.h"
-#include <mforms/mforms.h>
 
 using namespace mforms;
 
@@ -30,6 +29,12 @@ Menu::Menu()
 }
 
 
+bool Menu::empty() const
+{
+  return _item_map.size() == 0;
+}
+
+
 void Menu::remove_item(int i)
 {
   _menu_impl->remove_item(this, i);
@@ -134,7 +139,8 @@ void Menu::popup()
 
 void Menu::handle_action(const std::string &action)
 {
-  _action_handler(action);
+  if (_action_handler)
+    _action_handler(action);
 }
 
 //--------------------------------------------------------------------------------------------------
