@@ -20,6 +20,8 @@
 #ifndef _MFORMS_FORM_H_
 #define _MFORMS_FORM_H_
 
+#include "base/trackable.h"
+
 #include <mforms/base.h>
 #include <mforms/view.h>
 
@@ -68,14 +70,14 @@ namespace mforms {
 
   /** A standalone, top-level window.
    */
-  class MFORMS_EXPORT Form : public View, public sigc::trackable
+  class MFORMS_EXPORT Form : public View, public base::trackable
   {
     FormImplPtrs *_form_impl;
     View *_content;
     bool _fixed_size;
     bool _release_on_close;
 
-    sigc::signal<void> _closed_signal;
+    boost::signals2::signal<void ()> _closed_signal;
 
     Form();
   public:
@@ -141,17 +143,16 @@ namespace mforms {
      
      In Python use add_closed_callback()
      */
-    sigc::signal<void>& signal_closed() { return _closed_signal; }
+    boost::signals2::signal<void ()>* signal_closed() { return &_closed_signal; }
 #endif
 
     void was_closed()
     {
-      _closed_signal.emit();
+      _closed_signal();
       if (_release_on_close)
         release();
     }
 
-    virtual void needs_resize();
   };
 };
 
