@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,11 +23,6 @@
 
 #include "stdafx.h"
 
-#include <mforms/view.h>
-#include <mforms/mforms.h>
-#include <mforms/app.h>
-#include <stdio.h>
-
 using namespace mforms;
 using namespace MySQL;
 
@@ -57,6 +52,7 @@ View::~View()
   {
     bool managed = iter->second;
     View *sv = iter->first;
+    sv->_parent = NULL;
     
     _subviews.erase(iter);
     if (managed)
@@ -72,6 +68,7 @@ View::~View()
 
 //--------------------------------------------------------------------------------------------------
 
+// XXX: no longer needed, objects can be queried if they are managed.
 void View::set_managed()
 {
   Object::set_managed();
@@ -90,6 +87,8 @@ void View::set_managed()
 
 //--------------------------------------------------------------------------------------------------
 
+// XXX: add and remove should retain and release. Note: some subclasses already do this on their own.
+// This needs to be consolidated here.
 void View::add_subview(View *sv)
 {
   _subviews.push_back(std::make_pair(sv, sv->_managed));
@@ -300,18 +299,9 @@ void View::set_size(int width, int height)
 
 //--------------------------------------------------------------------------------------------------
 
-void View::set_padding(const Geometry::Padding& padding)
+void View::client_to_screen(int& x, int& y)
 {
-  set_layout_dirty(true);
-  if (_view_impl->set_padding)
-    (*_view_impl->set_padding)(this, padding);
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void View::client_to_screen(Geometry::Point& position)
-{
-  (*_view_impl->client_to_screen)(this, position);
+  (*_view_impl->client_to_screen)(this, x, y);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -352,14 +342,6 @@ void View::set_needs_repaint()
 
 //--------------------------------------------------------------------------------------------------
 
-void View::needs_resize()
-{
-  if (_parent)
-    _parent->needs_resize();
-}
-
-//--------------------------------------------------------------------------------------------------
-
 void View::set_layout_dirty(bool value)
 {
   _layout_dirty= value;
@@ -385,14 +367,6 @@ void View::relayout()
 
 //--------------------------------------------------------------------------------------------------
 
-void View::on_resize()
-{
-  needs_resize();
-  _signal_resized();
-}
-
-//--------------------------------------------------------------------------------------------------
-
 void View::suspend_layout()
 {
   if (_view_impl->suspend_layout)
