@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,7 +25,7 @@
 #include "grtpp_util.h"
 #include "base/string_utilities.h"
 
-#include <glib/gbase64.h>
+#include <glib.h>
 #include <functional>
 
 using namespace grt;
@@ -66,20 +66,23 @@ std::string grt::get_guid()
 #elif defined(__APPLE__)
   CFUUIDRef uid;
   CFStringRef str;
-  char *data;
+  char data[40];
   
   uid= CFUUIDCreate(NULL);
   str= CFUUIDCreateString(NULL, uid);
   
-  data= (char*)g_malloc(sizeof(gchar)*(CFStringGetLength(str)+1));
+#ifdef ENABLE_DEBUG
+  assert(sizeof(data) > CFStringGetLength(str));
+#endif
+  //data= (char*)g_malloc(sizeof(gchar)*(CFStringGetLength(str)+1));
   
-  CFStringGetCString(str, data, CFStringGetLength(str)+1, kCFStringEncodingUTF8);
+  CFStringGetCString(str, data, sizeof(data), kCFStringEncodingUTF8);
   
   CFRelease(uid);
   CFRelease(str);
   
   std::string ret= data;
-  g_free(data);
+  //g_free(data);
   
   return ret;
 #else
