@@ -138,7 +138,7 @@ static void call_refresh(DbMysqlRoutineEditor *self)
 - (NSView*)dockableView
 {
   // the view to be docked to the main window
-  return tabView;
+  return [tabView superview];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -267,4 +267,10 @@ static int process_syntax_error(const int line, const int err_tok_line_pos, cons
 {
   return mBackEnd;
 }
+
+
+- (NSImage*)titleIcon
+{
+  return [NSImage imageNamed: @"tab_icon_db.Routine.png"];
+}
 @end
