@@ -258,6 +258,8 @@ namespace MySQL.GUI.Workbench.Plugins
 		{
 			if (e.Node != null && e.Node.Tag != null)
 			{
+        int selIndex = e.Node.Index;
+
         GrtListNode node = e.Node.Tag as GrtListNode;
 
         // Activate column placeholder row if we a value for it must be set here.
@@ -327,8 +329,6 @@ namespace MySQL.GUI.Workbench.Plugins
 							grtList.set_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Default, e.Value as String);
 					}
 
-          int selIndex = (this.ListView.SelectedNode != null) ? this.ListView.SelectedNode.Index : -1;
-
 					RefreshModel();
 
           if (selIndex > -1)
