@@ -34,8 +34,10 @@ import stat
 import time
 import traceback
 
-from wb_common import PermissionDeniedError, InvalidPasswordError, OperationCancelledError, dprint_ex
+from wb_common import PermissionDeniedError, InvalidPasswordError, OperationCancelledError
 
+from grt import log_info, log_error, log_warning, log_debug
+_this_file = os.path.basename(__file__)
 
 try:
     import paramiko
@@ -46,7 +48,7 @@ except:
     paramiko = None
     socket = None
 
-if paramiko and paramiko.__version_info__[1] in (7,) and paramiko.__version_info__[2] in (4,5,6):
+if paramiko and paramiko.__version_info__ >= (1, 7, 4):
     from paramiko.message import Message
     from paramiko.common import *
     from paramiko.channel import Channel
@@ -114,6 +116,7 @@ if paramiko and paramiko.__version_info__[1] in (7,) and paramiko.__version_info
     paramiko.Transport.open_channel = wba_open_channel
 else:
   print "Warning! Can't use connect with timeout in paramiko", paramiko and paramiko.__version__
+  log_warning(_this_file, 'Cannot use connect with timeout in paramiko version %s\n' % paramiko.__version__)
 
 #===============================================================================
 class ConnectionError(Exception):
@@ -127,7 +130,7 @@ class SSHDownException(Exception):
 #===============================================================================
 #
 #===============================================================================
-class WbAdminSFTP:
+class WbAdminSFTP(object):
   def __init__(self, sftp):
     self.sftp = sftp
 
@@ -200,18 +203,17 @@ class WbAdminSFTP:
 #===============================================================================
 #
 #===============================================================================
-class WbAdminSSH:
+class WbAdminSSH(object):
   def wrapped_connect(self, settings, password_delegate):
     assert hasattr(password_delegate, "get_password_for")
     assert hasattr(password_delegate, "reset_password_for")
-  
-    dprint_ex(2, "Enter")
+
+    log_debug(_this_file, "%s: starting connect\n" % self.__class__.__name__)
     self.client = None
 
     #loginInfo = settings.loginInfo
     #serverInfo = settings.serverInfo
 
-    print type(settings)
     host = settings.ssh_hostname #loginInfo['ssh.hostName']
     usekey = settings.ssh_usekey #int(loginInfo['ssh.useKey'])
     pwd = None # It is ok to keep pwd set to None even if we have it in server settings
@@ -266,6 +268,10 @@ class WbAdminSSH:
           else:
               password_delegate.reset_password_for("ssh")
           raise exc
+      except paramiko.PasswordRequiredException, e:
+          pwd = password_delegate.get_password_for("sshkey")
+          self.connect(host, port, settings.ssh_username, pwd, usekey, key_filename)
+          # Retry key pwd
     else:
       #if pwd is None:
       #  accepted, pwd = mforms.Utilities.find_or_ask_for_password("Remote SSH Login (%s)" % serverInfo['sys.system'], "ssh@%s:%s" % (host,port or 22), loginInfo['ssh.userName'], False)
@@ -278,7 +284,7 @@ class WbAdminSSH:
           password_delegate.reset_password_for("ssh")
       #else:
       #  raise OperationCancelledError("Cancelled login")
-    dprint_ex(2, "Leave")
+    log_debug(_this_file, "%s: Leave\n" % self.__class__.__name__)
 
   def connect(self, host, port, user, pwd, usekey = 0, key = None):
     if port == None or port == 0:
@@ -291,16 +297,21 @@ class WbAdminSSH:
       key = os.path.expanduser(key)
 
     self.client = paramiko.SSHClient()
-    self.client.load_system_host_keys()
+    if usekey:
+      self.client.load_system_host_keys()
 
     # TODO: Check if file with 
-    self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
+    #self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
+    self.client.set_missing_host_key_policy(paramiko.WarningPolicy()) # This is a temp workaround, later we need to have UI with buttons accept
+                                                                      
     try:
       if 'timeout' in paramiko.SSHClient.connect.func_code.co_varnames:
-        self.client.connect(hostname = host, port = int(port), username = user, password = pwd, pkey = None, key_filename = key, timeout = 10)
+        self.client.connect(hostname = host, port = int(port), username = user, password = pwd, pkey = None,
+                            key_filename = key, timeout = 10, look_for_keys=False, allow_agent=bool(usekey) )
       else:
-        self.client.connect(hostname = host, port = int(port), username = user, password = pwd, pkey = None, key_filename = key)
-      dprint_ex(1, "Connected via ssh to ", host, " client = ", self.client)
+        self.client.connect(hostname = host, port = int(port), username = user, password = pwd, pkey = None,
+                            key_filename = key, look_for_keys=False, allow_agent=bool(usekey) )
+      log_info(_this_file, "%s: Connected via ssh to %s\n" % (self.__class__.__name__, host) )
     except socket.error, exc:
       self.client = None
       raise SSHDownException()
@@ -367,6 +378,7 @@ class WbAdminSSH:
       sftp.close()
     except IOError, e:
       print "WbAdminSSH.get of file '" + source + "' failed: " + str(e)
+      log_error(_this_file, '%s: Retreival of file "%s" failed: %s\n' % (self.__class__.__name__, source, str(e)) )
 
     return ret
     
@@ -416,13 +428,14 @@ class WbAdminSSH:
       sftp.put(source, dest)
       ret = True
       sftp.close()
-    except:
+    except Exception, e:
       print "WbAdminSSH.put failed"
+      log_error(_this_file, '%s: Sending of file "%s" to the server failed: %s\n' % (self.__class__.__name__, source, str(e)) )
 
     return ret
 
   def exec_cmd(self, cmd, as_admin = 0, admin_password = None, output_handler = None, read_size = 128, get_channel_cb = None):
-    dprint_ex(3, "Enter cmd = ", cmd)
+    log_debug(_this_file, "cmd = %s\n" % cmd)
     out = ""
     ret = None
     # For some reason we have bool in pwd
@@ -453,14 +466,14 @@ class WbAdminSSH:
           stderr = chan.makefile_stderr('rb', bufsize)
 
           if get_channel_cb is not None:
-            dprint_ex(3, "Getting channel via passed cb", get_channel_cb)
+            log_debug(_this_file, "%s.exec_cmd: Getting channel via passed cb (%s)\n" % (self.__class__.__name__, get_channel_cb.__name__) )
             get_channel_cb(chan)
 
           # There can be possible long delay if command executed expects password
           # and the password is None
           if (as_admin and admin_password is not None):
             time.sleep(1)
-            dprint_ex(3, "Sending password")
+            log_debug(_this_file, "%s.exec_cmd: Sending password\n" % self.__class__.__name__)
             try:
               stdin.write(admin_password+"\n")
               stdin.flush()
@@ -494,7 +507,7 @@ class WbAdminSSH:
           if chan.exit_status_ready():
             ret = chan.recv_exit_status()
           else:
-            dprint_ex(1, "Read from the peer is done, but status code is not available")
+            log_warning(_this_file, "%s: Read from the peer is done, but status code is not available\n" % self.__class__.__name__)
 
       except paramiko.SSHException, e:
         if chan and chan.recv_ready():
@@ -519,7 +532,7 @@ class WbAdminSSH:
 
     if out is not None and admin_password is not None:
       out = out.replace(admin_password, "")
-    dprint_ex(5, "SSH.exec_cmd(cmd =", cmd, ", output =", out, ". Retcode =", ret, ")")
+    log_debug(_this_file, "%s.exec_cmd(cmd=%s, output=%s. Retcode = %s)\n" % (self.__class__.__name__, cmd, str(out), str(ret)) )
     return (out, ret)
 
   def getftp(self):
@@ -527,7 +540,6 @@ class WbAdminSSH:
     return self.sftp
 
   def close(self):
-    dprint_ex(2, " ")
     if self.client is not None:
       self.client.close()
 
