@@ -32,9 +32,12 @@ import wb_admin_control
 from wb_server_control import PasswordHandler
 from wb_server_control import ServerProfile
 from db_utils import MySQLConnection, MySQLError
-from wb_common import OperationCancelledError, InvalidPasswordError, dprint_ex
+from wb_common import OperationCancelledError, InvalidPasswordError, NoDriverInConnection
 from wb_server_management import local_run_cmd, local_get_cmd_output
 
+from grt import log_info, log_error, log_warning, log_debug
+_this_file = os.path.basename(__file__)
+
 # define this Python module as a GRT module
 ModuleInfo = DefineModule(name= "WbAdmin", author= "Oracle Corp.", version="2.0")
 
@@ -69,8 +72,9 @@ class AdministratorTab(AppView):
           valid_versions = ((4,0), (4,1), (5,0), (5,1), (5,2), (5,4), (5,5), (5,6), (6,0))
           if version[:2] not in valid_versions:
             print version, "UNSUPPORTED"
+            log_warning(_this_file, "%s: Server version %s is NOT supported\n" % (self.__class__.__name__, str(version)) )
           else:
-            dprint_ex("Server version %s is supported" % str(version))
+            log_info(_this_file, "%s: Server version %s is supported\n" % (self.__class__.__name__, str(version)) )
 
         self.on_close(wb_admin_utils.weakcb(self, "handle_on_close"))
 
@@ -135,6 +139,10 @@ def do_open_administrator(server_instance):
     except OperationCancelledError, exc:
         app.set_status_text("Cancelled (%s)"%exc)
         return None
+    except NoDriverInConnection, exc:
+        Utilities.show_error('Missing connection driver', str(exc), 'OK', '', '')
+        app.set_status_text("Could not Open WB Admin")
+        return None
     except Exception, exc:
         import traceback
         traceback.print_exc()
@@ -156,7 +164,7 @@ def do_open_administrator(server_instance):
 
     app.set_status_text("WB Admin Opened")
 
-    dprint_ex(1, "Completed")
+    log_info(_this_file, "do_open_administrator(): Completed\n")
     return adminTab
 
 
@@ -215,6 +223,10 @@ def openRemoteFileSelector(serverInstance):
 
 
 def selectServer(title):
+    # No need to select an instance if there's only one:
+    if len(grt.root.wb.rdbmsMgmt.storedInstances) == 1:
+        return grt.root.wb.rdbmsMgmt.storedInstances[0]
+
     window = mforms.Form(None)
     window.set_title(title)
     box = mforms.newBox(False)
@@ -233,6 +245,16 @@ def selectServer(title):
 
     for inst in grt.root.wb.rdbmsMgmt.storedInstances:
       listbox.add_item(inst.name)
+      
+    if len(grt.root.wb.rdbmsMgmt.storedInstances) > 0:
+        listbox.set_selected(0)
+    else:
+        Utilities.show_warning("No Database Server Instances", 
+                    '''You have not defined a database server instance. At least one
+    server instance is needed. Please define one by clicking in 
+    "New Server Instance" and retry.''',
+                    "OK", "", "")
+        return None
     
     bbox = mforms.newBox(True)
     box.add(bbox, False, True)
@@ -266,7 +288,7 @@ def openExportImport():
         tab = do_open_administrator(server)
         if tab:   
             tab.wait_server_check(4)
-            tab.configuration.switch_to("DATA EXPORT / RESTORE", "Data Export and Restore")
+            tab.configuration.switch_to("DATA EXPORT / RESTORE", "Data Export")
             return 1
     return 0
 
