@@ -168,7 +168,7 @@ static void call_refresh(ImageEditor *self)
 - (NSView*)dockableView
 {
   // the view to be docked to the main window
-  return tabView;
+  return [tabView superview];
 }
 
 
