@@ -112,7 +112,7 @@ static void call_refresh(NoteEditor *self)
 - (NSView*)dockableView
 {
   // the view to be docked to the main window
-  return tabView;
+  return [tabView superview];
 }
 
 - (void)controlTextDidEndEditing:(NSNotification *)aNotification
