@@ -98,8 +98,8 @@ static void call_refresh(StoredNoteEditor *self)
       [noteText setEditable: NO];
       [noteText setString: NSLocalizedString(@"Data is not UTF8 and cannot be displayed.", @"attached file editor")];
     }
-    [applyButton setEnabled: NO];
-    [revertButton setEnabled: NO];
+    [mApplyButton setEnabled: NO];
+    [mRevertButton setEnabled: NO];
   }
 }
 
@@ -121,7 +121,7 @@ static void call_refresh(StoredNoteEditor *self)
 - (NSView*)dockableView
 {
   // the view to be docked to the main window
-  return tabView;
+  return [tabView superview];
 }
 
 
@@ -166,7 +166,7 @@ static void call_refresh(StoredNoteEditor *self)
 
 - (IBAction)revertChanges:(id)sender
 {
-  [revertButton setEnabled: NO];
+  [mRevertButton setEnabled: NO];
   [self refresh];
 }
 
@@ -183,8 +183,8 @@ static void call_refresh(StoredNoteEditor *self)
                                                      repeats: NO];
   if ([aNotification object] == noteText)
   {
-    [applyButton setEnabled: YES];
-    [revertButton setEnabled: YES];
+    [mApplyButton setEnabled: YES];
+    [mRevertButton setEnabled: YES];
   }
 }
 
