@@ -123,7 +123,7 @@ static void call_refresh(PhysicalLayerEditor *self)
 - (NSView*)dockableView
 {
   // the view to be docked to the main window
-  return tabView;
+  return [tabView superview];
 }
 
 - (void)controlTextDidEndEditing:(NSNotification *)aNotification
