@@ -171,7 +171,7 @@ class MySQLConnection:
           code = modules.DbMySQLQuery.lastErrorCode()
           error = modules.DbMySQLQuery.lastError()
           self.send_status(code, error)
-          raise QueryError("Error executing '%s'\n%s" %(error, query), code)
+          raise QueryError("Error executing '%s'\n%s" %(query, error), code)
 
         self.send_status(0)
         return result != 0
@@ -188,7 +188,7 @@ class MySQLConnection:
           code = modules.DbMySQLQuery.lastErrorCode()
           error = modules.DbMySQLQuery.lastError()
           self.send_status(code, error)
-          raise QueryError("Error executing '%s'\n%s"%(error, query), code)
+          raise QueryError("Error executing '%s'\n%s"%(query, error), code)
 
         self.send_status(0)
         return MySQLResult(result)
